/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "support.h"
#include "ApplicationManager.h"

#include <glib.h>
#include <stdlib.h> /* atoi */

#include "data/CatalogFileInfo.h"

#include "tools/PropertiesFile.h"
#include "tools/File.h"
#include "tools/Mail.h"

#include "GuiManager.h"
#include "Catalog.h"


#define GW_APPLICATION_HOME ".GWhere"
#define GW_APPLICATION_SETTINGS_FILE "GWhere.cfg"
#define GW_APPLICATION_SETTINGS_FILE_SAVE_EXTENSION "bak"

GWApplicationSettings my_application_settings;


gint gw_application_manager_init ( )
{
	gint result = -1;
	gchar *tmp;
	GWSettings *settings = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

#ifdef GW_DEBUG_MODE
	g_print ( _( "%s %s is running in debug mode\n"), PACKAGE, VERSION);

	#if defined ( OS_LINUX)
	g_print ( _( "Is running on GNU/Linux\n"));
	#elif defined ( OS_CYGWIN)
	g_print ( _( "Is running on Cygwin\n"));
	#elif defined ( OS_FREEBDS)
	g_print ( _( "Is running on FreeBSD\n"));
	#elif defined ( OS_OPENBDS)
	g_print ( _( "Is running on OpenBSD\n"));
	#elif defined ( OS_NETBDS)
	g_print ( _( "Is running on NetBSD\n"));
	#elif defined ( OS_WINDOWS)
	g_print ( _( "Is running on MS Windows\n"));
	#elif defined ( OS_UNKNOWN)
	g_print ( _( "Is running on unkown OS\n"));
	#endif
#endif

	my_application_settings.user_name = g_get_user_name ( );

	tmp = g_get_home_dir ( );
	my_application_settings.user_gwhere_home = g_strconcat ( tmp, G_DIR_SEPARATOR_S, GW_APPLICATION_HOME, NULL);
	g_free ( tmp);

	my_application_settings.user_gwhere_settings_file = g_strconcat ( my_application_settings.user_gwhere_home, G_DIR_SEPARATOR_S, GW_APPLICATION_SETTINGS_FILE, NULL);

#ifdef GW_DEBUG_MODE
	g_print ( "Configuration file : %s\n", my_application_settings.user_gwhere_settings_file);
#endif

	if ( gw_properties_file_load_all_settings ( my_application_settings.user_gwhere_settings_file, &settings) == -1)
	{
		/* If the properties file doesn't exist, allocates a new GWSettings. */
		settings = gw_settings_new ( );
	}

	my_application_settings.settings = settings;

#ifdef GW_DEBUG_MODE
	gw_settings_display ( my_application_settings.settings);
#endif

	return result;
}


gboolean gw_application_manager_can_send_mail ( )
{
	gboolean result = FALSE;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_EMAIL_ADDRESS) != NULL )
	{
		if ( gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_SERVER_ADDRESS) != NULL )
		{
			if ( gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_SERVER_PORT) != NULL )
			{
				result = TRUE;
			}
		}
	}

	return result;
}


gchar * gw_application_manager_get_settings ( const gchar *name)
{
	gchar *value = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( name != NULL )
	{
		value = gw_settings_get_value ( my_application_settings.settings, (gchar *)name);
	}

#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s() : pair is %s/%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__,name, value);
#endif


	return value;
}


gint gw_application_manager_set_settings ( const gchar *name, gchar *value)
{
	gint result = -1;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s() : pair is %s/%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__,name, value);
#endif

#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s() : old pair is %s/%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__,name, gw_application_manager_get_settings ( name));
#endif

	if ( name != NULL )
	{
		gw_settings_add_pair ( my_application_settings.settings, (gchar *)name, value);

		result = 0;
	}

#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s() : new pair is %s/%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__,name, gw_application_manager_get_settings ( name));
#endif

	return result;
}


gint gw_application_manager_exit ( )
{
	gint result = -1;
	gchar *save_file = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( gw_file_mkdir ( my_application_settings.user_gwhere_home) == 0 )
	{
#ifdef GW_DEBUG_MODE
		g_print ( "Configuration file : is %s exists?\n", my_application_settings.user_gwhere_settings_file);
#endif
		if ( gw_file_exists ( my_application_settings.user_gwhere_settings_file) == 0 )
		{
			save_file = g_strconcat ( my_application_settings.user_gwhere_settings_file, "." , GW_APPLICATION_SETTINGS_FILE_SAVE_EXTENSION, NULL);
			gw_file_copy ( my_application_settings.user_gwhere_settings_file, save_file);
		}

#ifdef GW_DEBUG_MODE
		g_print ( "Configuration file : %s\n", my_application_settings.user_gwhere_settings_file);
		gw_settings_display ( my_application_settings.settings);
#endif

		if ( gw_properties_file_save_all_settings ( my_application_settings.user_gwhere_settings_file, &(my_application_settings.settings), NULL) != -1)
		{
#ifdef GW_DEBUG_MODE
			g_print ( "*** GW - %s (%d) :: %s() : properties file saved\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

			result = 0;
		}
		else
		{
#ifdef GW_DEBUG_MODE
			g_print ( "*** GW - %s (%d) :: %s() : warning unabble to save the properties file!!\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif
		}

		g_free ( save_file);
	}

	return result;
}


gint gw_application_manager_send_mail ( GWMail *mail)
{
	gint result = -1;
	gboolean good = TRUE;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( mail != NULL )
	{
		/* Checks if can send a mail. */
		if ( gw_application_manager_can_send_mail ( ) )
		{
			/* To do more and more checking tests. */
			good = good && gw_mail_check_email_list ( gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_EMAIL_ADDRESS));
			good = good && gw_mail_check_email_list ( mail->to);
			good = good && gw_mail_check_email_list ( mail->cc);
			good = good && gw_mail_check_email_list ( mail->bcc);

			if ( good )
			{
				result = gw_mail_send_mail ( gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_SERVER_ADDRESS),
						atoi( gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_SERVER_PORT)),
						gw_application_manager_get_settings ( GW_VALUE_APP_MAIL_EMAIL_ADDRESS),
						mail->to,
						mail->cc,
						mail->bcc,
						mail->object,
						mail->message,
						mail->file_path);
			}
		}
	}

	return result;
}


gint gw_application_manager_load_catalog ( const char *filepath)
{
	gint result = -1;
	struct catalog_file_info *file_info = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( filepath != NULL )
	{
		/* Checks if the file exists. */
		if ( gw_file_exists ( filepath) == 0 )
		{
			/* Build the catalog file informations. */
			file_info = catalog_file_info_new ( );
			catalog_file_info_set_title ( file_info, gw_file_get_name ( filepath));
			catalog_file_info_set_full_name ( file_info, g_strdup ( filepath));

			tree = interface_get_tree ( );
			root = interface_get_tree_root ( );

			/* Starts the catalog loading. */
			result = catalog_load_file ( file_info, tree, root);
		}
	}

	return result;
}
