/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/*!
 * @header	ApplicationManager.h
 *		GWhere<br>
 *		Copyright (C) 2000  Sbastien LECACHEUR<br><br>
 *
 *		This Distributed under the terms of the <a href="http://www.gnu.org/copyleft/gpl.html">GNU General Public Licence</a><br><br>
 *
 *		This component was developed for the GWhere project.<br><br>
 *
 * @abstract	This is a component to manage GWhere application.
 * @author	Sbastien LECACHEUR <zero@gwhere.org>
 * @author	Grgory BLOQUEL <pipocrouch@gwhere.org>
 * @version	1.0 2001/08/21
 * @see		support.h
 * @see		data/Settings.h
 * @link	GWhere Web Site <http://www.gwhere.org>
 * @link	GTK+ Web Site <http://www.gtk.org>
 * @ref		ApplicationManager.c
 * @author	Author <zero@gwhere.org>
 */


#ifndef GW_APPLICATION_MANAGER_H
#define GW_APPLICATION_MANAGER_H

#include "data/Settings.h"


/*!
 * @typedef     GWApplicationSettings
 * @field	user_name			name of the current user
 * @field	user_gwhere_home		path of the user gwhere home
 * @field	user_gwhere_settings_file	path of the user gwhere settings file
 * @field	settings			current settings
 */
typedef struct gw_application_settings
{
	gchar *user_name;
	gchar *user_gwhere_home;
	gchar *user_gwhere_settings_file;
	GWSettings *settings;
}GWApplicationSettings;


typedef struct gw_mail_s
{
/*	gchar *server_name;
	int server_port;
	gchar *from;
*/	gchar *to;
	gchar *cc;
	gchar *bcc;
	gchar *object;
	gchar *message;
	gchar *file_path;
}GWMail;


/*! @define	GW_VALUE_APP_RECENT	The variable name for recents opened files */
#define GW_VALUE_APP_RECENT "GWhere.application.recents.size"
/*! @define	GW_VALUE_APP_RECENT_SIZE	The variable name for the size of recents opened files list */
#define GW_VALUE_APP_RECENT_SIZE "GWhere.application.recents.size"
/*! @define	GW_VALUE_APP_RECENT_SIZE_DEFAULT	The default size of recents opened file list */
#define GW_VALUE_APP_RECENT_SIZE_DEFAULT 5
/*! @define	GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL	The variable name for the catalog compression ratio */
#define GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL "GWhere.application.catalog.compression.level"
/*! @define	GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_DEFAULT	The default catalog compression ratio */
#define GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_DEFAULT 3
/*! @define	GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_MIN	The minimum catalog compression ratio */
#define GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_MIN 0
/*! @define	GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_MAX	The maximun catalog compression ratio */
#define GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_MAX 9
/*! @define	GW_VALUE_APP_MAIL_EMAIL_ADDRESS	The variable name for email adress */
#define GW_VALUE_APP_MAIL_EMAIL_ADDRESS "GWhere.application.mail.email.address"
/*! @define	GW_VALUE_APP_MAIL_SERVER_ADDRESS	The variable name for outgoing mail server name */
#define GW_VALUE_APP_MAIL_SERVER_ADDRESS "GWhere.application.mail.server.address"
/*! @define	GW_VALUE_APP_MAIL_SERVER_PORT	The variable name for outgoing mail server port */
#define GW_VALUE_APP_MAIL_SERVER_PORT "GWhere.application.mail.server.port"
/*! @define	GW_VALUE_APP_MAIL_SERVER_PORT_DEFAULT	The default outgoing mail server port */
#define GW_VALUE_APP_MAIL_SERVER_PORT_DEFAULT "25"


/*!
 * @function	gw_application_manager_init
 * @abstract	Initializes the application settings.
 * @discussion	This function initializes the application settings.
 * @result	an error code.<br>
 *		Returns <i>0</i> when it's done.<br>
 *		Returns <i>-1</i> when an error occured.<br>
 */
gint gw_application_manager_init ( );


/*!
 * @function	gw_application_manager_can_send_mail
 * @abstract	Checks if can send mails.
 * @discussion	This function checks if can send mails. To
 *		send mails, the user must configure mail
 *		options in the settings.
 * @result	Returns <i>TRUE</i> if can send mails.<br>
 *		Returns <i>FALSE</i> if cannot send mails or
 *		when an error occured.<br>
 */
gboolean gw_application_manager_can_send_mail ( );


/*!
 * @function	gw_application_manager_get_settings
 * @abstract	Gets the value of a setting.
 * @discussion	This function gGets the value of a setting.
 * @param	name	name of setting
 * @result	the value of setting.<br>
 *		Returns <i>NULL</i> when an error occured.<br>
 */
gchar * gw_application_manager_get_settings ( const gchar *name);

/*!
 * @function	gw_application_manager_get_settings
 * @abstract	Gets the value of a setting.
 * @discussion	This function gGets the value of a setting.
 * @param	name	name of setting
 * @param	value	value of setting
 * @result	an error code.<br>
 *		Returns <i>0</i> when it's done.<br>
 *		Returns <i>-1</i> when an error occured.<br>
 */
gint gw_application_manager_set_settings ( const gchar *name, gchar *value);


/*!
 * @function	gw_application_manager_exit
 * @abstract	Exits the application and processes to all savings.
 * @discussion	This function exits the application and processes to all savings.
 * @result	an error code.<br>
 *		Returns <i>0</i> when it's done.<br>
 *		Returns <i>-1</i> when an error occured.<br>
 */
gint gw_application_manager_exit ( );


/*!
 * @function	gw_application_manager_send_mail
 * @abstract	Sends an mail from the application.
 * @discussion	This function sends an mail from the application.
 * @result	an error code.<br>
 *		Returns <i>0</i> when it's done.<br>
 *		Returns <i>-1</i> when an error occured.<br>
 */
gint gw_application_manager_send_mail ( GWMail *mail);


/*!
 * @function	gw_application_manager_load_catalog
 * @abstract	Loads a catalog in the application.
 * @discussion	This function loads a catalog in the application.
 * @param	filepath	catalog file path
 * @result	an error code.<br>
 *		Returns <i>0</i> when it's done.<br>
 *		Returns <i>-1</i> when an error occured.<br>
 */
gint gw_application_manager_load_catalog ( const char *filepath);


#endif
