/*  GWhere
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "support.h"
#include "Catalog.h"

#if defined ( OS_CYGWIN) || defined ( OS_WIN32)
	/* A port for Cygwin OS, the GLib should be define this macro, but it doesn't work on my Cygwin OS */
	#define NAME_MAX 255
#elif defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* For NAME_MAX and PATH_MAX */
	#include <sys/syslimits.h>
#endif

#include <gtk/gtk.h>
#include <sys/stat.h>
#include <sys/types.h> /* opendir */
#include <dirent.h> /* opendir */

/* Notes that exist win32 implementation of these librairies. */
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
    #include <pwd.h>
    #include <grp.h>
#endif

#include <time.h>
#include <stdlib.h> /* atoi */
#include <stdio.h> /* perror */
#include <string.h> /* strncmp */
#include <errno.h>

/* To open and close files */
#include <fcntl.h>
#include <unistd.h> /* chdir readlink */

#include "ApplicationManager.h"
#include "GuiManager.h"

#include "gui/MsgBox.h"

/* Used datas strucutures */
#include "data/Category.h"
#include "data/DirData.h"
#include "data/DiskData.h"

#include "res/Icons.h"
#include "tools/GWString.h"
#include "tools/File.h"


gint catalog_load_file ( struct catalog_file_info *catalog_info, GtkCTree *ctree, GtkCTreeNode *parent)
{
	char buf[MAX_LINE];
	gchar **tmp;
	gchar *file;
	gint fic;
	gzFile zfic;
	GtkCList *clist;
	GtkWindow *window = interface_get_main_window ( );
	GtkCTreeNode *root = NULL;


#if GW_DEBUG_TIME
	time_t begin;
	time ( &begin);
#endif

#if GW_DEBUG_MODE
	g_print ( "%s %d :: catalog_load_file\n", __FILE__, __LINE__);
#endif

	file = catalog_file_info_get_full_name ( catalog_info);

#if GW_DEBUG_MODE
	g_print ( "%s %d :: catalog_load_file : file name = %s\n", __FILE__, __LINE__, file);
#endif

	/* Opens the catalog file */
	if ( (fic = open ( file, O_RDONLY)) == -1 )
	{
#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_load_file : cannot open this file\n", __FILE__, __LINE__);
#endif

		msg_box_create ( window, _( "Load file"), _( "Error : cannot open this file."));

		return -1;
	}

	if ( (zfic = gzdopen ( fic, "r")) == NULL )
	{
#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_load_file : cannot open this file (z)\n", __FILE__, __LINE__);
#endif

		msg_box_create ( window, _( "Load file"), _( "Error : cannot open this file."));
		close ( fic);

		return -1;
	}

        /* Reads the file version */
	if ( gw_zfile_readline ( zfic, buf, MAX_LINE) == NULL )
	{
#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_load_file : cannot read the file version\n", __FILE__, __LINE__);
#endif

		msg_box_create ( window, _( "Load file"), _( "Error : cannot read the file version."));

		if ( gzclose ( zfic) != Z_OK )
		{
			msg_box_create ( window, _( "Load catalog"), _( "Error : cannot close file."));
		}

		return -1;
	}

	/* Checks the file version */
	tmp = g_strsplit ( buf, ":", 0);

	if ( strcmp ( tmp[0], "GWhere") != 0 )
	{
#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_load_file : it's not a GWhere file\n", __FILE__, __LINE__);
#endif

		msg_box_create ( window, _( "Load file"), _( "Error : it's not a GWhere file."));
		g_strfreev ( tmp);

		if ( gzclose ( zfic) != Z_OK )
		{
			msg_box_create ( window, _( "Load catalog"), _( "Error : cannot close file."));
		}

		return -1;
	}

	/* Sets the new window title */
	gtk_window_set_title ( window, g_strconcat ( "GWhere - ", catalog_file_info_get_full_name ( catalog_info), NULL));

	/* Freezes the exploration tree */
	clist = GTK_CLIST ( ctree);
	gtk_clist_freeze ( clist);

	/* Sets file version */
	catalog_file_info_set_version ( catalog_info, g_strdup ( tmp[2]));

	/* Sets the programme maker of the file */
	if ( tmp[3] != NULL )
	{
		catalog_file_info_set_program_maker ( catalog_info, g_strdup ( tmp[3]));
	}
	else
	{
		catalog_file_info_set_program_maker ( catalog_info, g_strconcat ( "GWhere ", tmp[1], NULL));
	}

	/* Checks the file version */
	switch ( atoi ( tmp[2]))
	{
		case 1:	/* Loads la 1th version of GWhere file */
			catalog_load_file_version_1 ( catalog_info, fic, ctree, parent);

			break;

		case 2:	/* Loads the 2nd version of GWhere file */
			catalog_load_file_version_2 ( catalog_info, zfic, ctree, parent);

			break;

		default:/* This version is unknown */
#if GW_DEBUG_MODE
			g_print ( "%s %d :: catalog_load_file : this GWhere version file is unknown\n", __FILE__, __LINE__);
#endif

			msg_box_create ( window, _( "Load file"), _( "Error : this GWhere version file is unknown."));

			break;
	}

	if ( gzclose ( zfic) == -1 )
	{
#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_load_file : cannot close this file after loading\n", __FILE__, __LINE__);
#endif

		g_strfreev ( tmp);
		msg_box_create ( window, _( "Load file"), _( "Error : cannot close this file after loading."));

		return -1;
	}

	/* Refreshes the exploration tree */
	root = interface_get_tree_root ( );
	gtk_ctree_sort_recursive ( ctree, root);
	gtk_clist_thaw ( clist);

#if GW_DEBUG_MODE
	g_print ( "%s %d :: catalog_load_file : select tree's node root\n", __FILE__, __LINE__);
#endif

	gtk_ctree_select ( ctree, root);

	g_strfreev ( tmp);

#if GW_DEBUG_TIME
	g_print ( "Elapsed time to load the catalog: %lu secs\n", time ( NULL) - begin);
#endif
	return 0;
}


gint catalog_load_file_version_1 ( struct catalog_file_info *catalog_info, gint fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	gchar *buf = NULL, *text [1];
	gchar **tmp = NULL;
	GtkCTreeNode *node = NULL;
	struct disk_data *disk_data = NULL;
	struct disk_info *disk_info = NULL;
	GdkPixmap *pix;
	GdkBitmap *msk;
	GtkCTreeNode *root = interface_get_tree_root ( );
	GtkWindow *window = interface_get_main_window ( );


#if GW_DEBUG_MODE
	g_print ( "%s %d :: catalog_load_file_version_1\n", __FILE__, __LINE__);
#endif

	text[0] = g_strdup ( catalog_file_info_get_title ( catalog_info));

	/* Creates catalog root node */
	root = gtk_ctree_insert_node ( ctree, NULL, NULL, text, 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);
	interface_set_tree_root ( root);
	gtk_ctree_node_set_row_data_full ( ctree, root, catalog_info, (GtkDestroyNotify)catalog_file_info_free);
	parent = root;

	// FIX BUG : may be memory leak at each gw_file_readline call
//	while ( (buf = gw_file_readc ( fic, "\n")) != NULL)
	while ( (buf = gw_file_readline ( fic, buf, MAX_LINE)) != NULL)
	{
		if ( strcmp ( buf, "//") == 0 )
		{
//			buf = gw_file_readc ( fic, "\n");
			buf = gw_file_readline ( fic, buf, MAX_LINE);
			tmp = g_strsplit ( buf, ":", 0);

			/* Creates disk node */
			disk_info = catalog_disk_info_new_from_table ( tmp, 1);
			disk_data = disk_data_new ();
			disk_data_set_info ( disk_data, disk_info);
			pix = disk_info_get_pixmap ( disk_info);
			msk = disk_info_get_bitmap ( disk_info);
			text[0] = disk_info_get_name ( disk_info);
			node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, pix, msk, pix, msk, FALSE, FALSE);
			gtk_ctree_node_set_row_data_full( ctree, node, disk_data, (GtkDestroyNotify)disk_data_free);

			g_strfreev ( tmp);

			/* Loads disk */
			if ( catalog_load_disk_1 ( fic, ctree, node) == -1 )
			{
				msg_box_create ( window, _( "Load file"), _( "Error : during the loading of disk."));
				return -1;
			}
		}
		else
		{
			/* Writes descriptions */
		}
	}

	return 0;
}


gint catalog_load_file_version_2 ( struct catalog_file_info *catalog_info, gzFile fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	gchar *text [1];
	char buf[MAX_LINE];
	gchar **tmp = NULL;
	GtkCTreeNode *node = NULL;
	struct disk_data *disk_data = NULL;
	struct disk_info *disk_info = NULL;
	struct category *category;
	GdkPixmap *pix;
	GdkBitmap *msk;
	GtkCTreeNode *root = interface_get_tree_root ( );
	GtkWindow *window = interface_get_main_window ( );


#if GW_DEBUG_MODE
	g_print ( "%s %d :: catalog_load_file_version_2\n", __FILE__, __LINE__);
#endif

	/* Loads catalog informations */
	if ( gw_zfile_readline(fic, buf, MAX_LINE) != NULL )
	{
		tmp = g_strsplit ( buf, ":", 0);

		/* Reads catalog name */
		if ( tmp[0] != NULL)
		{
			if ( strcmp ( tmp[0], "" ) != 0 )
			{
				/* Case : the catalog gets a name */
				catalog_file_info_set_name ( catalog_info, file2gchar ( tmp[0]));
				text[0] = g_strdup ( catalog_file_info_get_name ( catalog_info));
			}
			else
			{
				/* Case : default catalog name is file name */
				text[0] = g_strdup ( catalog_file_info_get_title ( catalog_info));
			}
		}

		/* Reads catalog description */
		if ( tmp[1] != NULL )
		{
			if ( strcmp ( tmp[1], "" ) != 0 )
			{
				catalog_file_info_set_description ( catalog_info, file2gchar ( g_strdup ( tmp[1])));
			}
		}
	}

	/* Creates catalog root node */
	root = gtk_ctree_insert_node ( ctree, NULL, NULL, text, 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);
	interface_set_tree_root ( root);
	gtk_ctree_node_set_row_data_full ( ctree, root, catalog_info, (GtkDestroyNotify)catalog_file_info_free);
	parent = root;

	while ( gw_zfile_readline ( fic, buf, MAX_LINE) != NULL )
	{
		if ( strcmp ( buf, "//") == 0 )
		{
			gw_zfile_readline ( fic, buf, MAX_LINE);
			tmp = g_strsplit ( buf, ":", 0);

			/* Creates a disk node */
			disk_info = catalog_disk_info_new_from_table ( tmp, 2);
			disk_info_set_category ( disk_info, catalog_file_info_get_category_by_index ( catalog_info, atoi ( tmp[10])));
			disk_data = disk_data_new ();
			disk_data_set_info ( disk_data, disk_info);
			pix = disk_info_get_pixmap ( disk_info);
			msk = disk_info_get_bitmap ( disk_info);
			text[0] = disk_info_get_name ( disk_info);
			node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, pix, msk, pix, msk, FALSE, FALSE);
			gtk_ctree_node_set_row_data_full( ctree, node, disk_data, (GtkDestroyNotify)disk_data_free);

			g_strfreev ( tmp);

			/* Loads disk */
			if ( catalog_load_disk_2 ( fic, ctree, node) == -1 )
			{
				msg_box_create ( window, _( "Load file"), _( "Error : during the loading of disk."));
				return -1;
			}
		}
		else
		{
			/* Loads categories */
			tmp = g_strsplit ( buf, ":", 2);

			if ( tmp[0] != NULL )
			{
				category = category_new ();
				category_set_name ( category, file2gchar ( tmp[0]));
				category_set_description ( category, file2gchar ( tmp[1]));

				catalog_file_info_append_category ( catalog_info, category);
			}

			g_strfreev ( tmp);
		}
	}

	return 0;
}


gint catalog_load_disk_1 (gint fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	guint ret, compt = 0;
	gchar *buf = NULL, *text [1];
	struct disk_data *disk_data = NULL;
	struct dir_data *dir_data = NULL;
	struct file_info *file_info = NULL;
	gchar **tmp;
	GtkCTreeNode *node = NULL;


	/* Gets disk informations */
	disk_data = gtk_ctree_node_get_row_data( ctree, parent);


	// FIX BUG : may be memory leak at each gw_file_readline call
//	while ( strcmp ( (buf = gw_file_readc ( fic, "\n")), "\\") != 0 )
	while ( strcmp ( (buf = gw_file_readline ( fic, buf, MAX_LINE)), "\\") != 0 )
	{
		if ( strcmp ( buf, "/") == 0 )
		{
			/* Case : is a folder */
			compt++;
//			buf = gw_file_readc ( fic, "\n");
			buf = gw_file_readline ( fic, buf, MAX_LINE);
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates memory for datas structures */
			file_info = catalog_file_info_new_from_table ( tmp, TRUE, 1);
			g_strfreev ( tmp);

			/* Creates folder node */
			dir_data = dir_data_new ();
			dir_data_set_info ( dir_data, file_info);
			text[0] = file_info_get_name ( file_info);
			node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, FALSE, FALSE);
			gtk_ctree_node_set_row_data_full( ctree, node, dir_data, (GtkDestroyNotify)dir_data_free);

			/* Loads folder */
			if ( (ret = catalog_load_dir_1 ( fic, ctree, node)) == 0)
			{
				gtk_ctree_set_node_info( ctree, node, text[0], 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);
			}
		}
		else
		{
			/* Case : is a file */
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates memory for datas structures */
			file_info = catalog_file_info_new_from_table ( tmp, FALSE, 1);
			g_strfreev ( tmp);

			/* Adds to files list the file informations */
			disk_data_append_child ( disk_data, file_info);
		}
	}

	return compt;
}


gint catalog_load_disk_2 (gzFile fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	guint ret, compt = 0;
	gchar *text [1];
	char buf[MAX_LINE];
	struct catalog_file_info *catalog_info = NULL;
	struct disk_data *disk_data = NULL;
	struct dir_data *dir_data = NULL;
	struct file_info *file_info = NULL;
	gchar **tmp;
	GtkCTreeNode *node = NULL;
	GtkCTreeNode *root = interface_get_tree_root ( );


	/* Gets catalog informations */
	catalog_info = gtk_ctree_node_get_row_data ( ctree, root);

	/* Gets disk information */
	disk_data = gtk_ctree_node_get_row_data ( ctree, parent);


	while ( strcmp ( gw_zfile_readline ( fic, buf, MAX_LINE), "\\") != 0 )
	{
		if ( strcmp ( buf, "/") == 0 )
		{
			/* Case : is a folder */
			compt++;
			gw_zfile_readline ( fic, buf, MAX_LINE);
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates datas structure */
			file_info = catalog_file_info_new_from_table ( tmp, TRUE, 2);
			file_info_set_category ( file_info, catalog_file_info_get_category_by_index ( catalog_info, atoi ( tmp[9])));
			g_strfreev ( tmp);

			/* Creates folder node */
			dir_data = dir_data_new ();
			dir_data_set_info ( dir_data, file_info);
			text[0] = file_info_get_name ( file_info);
			node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, FALSE, FALSE);
			gtk_ctree_node_set_row_data_full( ctree, node, dir_data, (GtkDestroyNotify)dir_data_free);

			/* Loads folder */
			if ( (ret = catalog_load_dir_2 ( fic, ctree, node)) == 0)
			{
				gtk_ctree_set_node_info( ctree, node, text[0], 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);
			}
		}
		else
		{
			/* Case : is a file */
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates datas structure */
			file_info = catalog_file_info_new_from_table ( tmp, FALSE, 2);
			file_info_set_category ( file_info, catalog_file_info_get_category_by_index ( catalog_info, atoi ( tmp[9])));
			g_strfreev ( tmp);

			/* Adds to files list the file informations */
			disk_data_append_child ( disk_data, file_info);
		}
	}

	return compt;
}


gint catalog_load_dir_1 (gint fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	guint ret, compt = 0;
	gchar *buf = NULL, *text [1];
	struct dir_data *dir_data = NULL;
	struct file_info *file_info = NULL;
	gchar **tmp;
	GtkCTreeNode *node = NULL;


	/* Rcupre les informations du rpertoire */
	dir_data = gtk_ctree_node_get_row_data( ctree, parent);


	// FIX BUG : may be memory leak at each gw_file_readline call
//	while ( strcmp ( (buf = gw_file_readc ( fic, "\n")), "\\") != 0 )
	while ( strcmp ( (buf = gw_file_readline ( fic, buf, MAX_LINE)), "\\") != 0 )
	{
		if ( strcmp ( buf, "/") == 0 )
		{
			/* Case : is a folder */
			compt++;
//			buf = gw_file_readc ( fic, "\n");
			buf = gw_file_readline ( fic, buf, MAX_LINE);
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates datas structure */
			file_info = catalog_file_info_new_from_table ( tmp, TRUE, 1);
			g_strfreev ( tmp);

			/* Creates folder node */
			dir_data = dir_data_new ();
			dir_data_set_info ( dir_data, file_info);
			text[0] = file_info_get_name ( file_info);
			node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, FALSE, FALSE);
			gtk_ctree_node_set_row_data_full( ctree, node, dir_data, (GtkDestroyNotify)dir_data_free);

			/* Loads folder with a recursive call */
			if ( (ret = catalog_load_dir_1 ( fic, ctree, node)) == 0)
			{
				gtk_ctree_set_node_info( ctree, node, text[0], 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);
			}
		}
		else
		{
			/* Case : is a file */
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates datas structure */
			file_info = catalog_file_info_new_from_table ( tmp, FALSE, 1);
			g_strfreev ( tmp);

			/* Adds to files list the file informations */
			dir_data_append_child ( dir_data, file_info);
		}
	}

	return compt;
}


gint catalog_load_dir_2 (gzFile fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	guint ret, compt = 0;
	gchar *text [1];
	static char buf[MAX_LINE];
	struct catalog_file_info *catalog_info = NULL;
	struct dir_data *dir_data = NULL;
	struct file_info *file_info = NULL;
	gchar **tmp;
	GtkCTreeNode *node = NULL;
	GtkCTreeNode *root = interface_get_tree_root ( );


	/* Gets catalog informations */
	catalog_info = gtk_ctree_node_get_row_data ( ctree, root);

	/* Gets folder informations */
	dir_data = gtk_ctree_node_get_row_data( ctree, parent);


	while ( strcmp ( gw_zfile_readline ( fic, buf, MAX_LINE), "\\") != 0 )
	{
		if ( strcmp ( buf, "/") == 0 )
		{
			/* Case : is a folder */
			compt++;
			gw_zfile_readline ( fic, buf, MAX_LINE);
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates datas structures */
			file_info = catalog_file_info_new_from_table ( tmp, TRUE, 2);
			file_info_set_category ( file_info, catalog_file_info_get_category_by_index ( catalog_info, atoi ( tmp[9])));
			g_strfreev ( tmp);

			/* Create folder node */
			dir_data = dir_data_new ();
			dir_data_set_info ( dir_data, file_info);
			text[0] = file_info_get_name ( file_info);
			node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, FALSE, FALSE);
			gtk_ctree_node_set_row_data_full( ctree, node, dir_data, (GtkDestroyNotify)dir_data_free);

			/* Loads folder with a recursive call */
			if ( (ret = catalog_load_dir_2 ( fic, ctree, node)) == 0)
			{
				gtk_ctree_set_node_info( ctree, node, text[0], 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);
			}
		}
		else
		{
			/* Case : is a file */
			tmp = g_strsplit ( buf, ":", 0);

			/* Allocates datas structure */
			file_info = catalog_file_info_new_from_table ( tmp, FALSE, 2);
			file_info_set_category ( file_info, catalog_file_info_get_category_by_index ( catalog_info, atoi ( tmp[9])));
			g_strfreev ( tmp);

			/* Adds to files list the file informations */
			dir_data_append_child ( dir_data, file_info);
		}
	}

	return compt;
}


gint catalog_add_disk ( struct scan_disk_info *scan_disk, GtkCTree *ctree, GtkCTreeNode *parent)
{
	GtkCTreeNode *node = NULL;
	gchar *text [1];
	gchar past_dir[512];
	struct disk_info * disk_info = NULL;
	struct disk_data * disk_data = NULL;
	struct dir_info *dir_info = NULL;
	struct catalog_file_info *file_info = NULL;
	gulong size = 0, date;
	GdkPixmap *pix;
	GdkBitmap *msk;
	gchar *error_msg = NULL;
	struct vfs_stats *vfs = NULL;
	struct category *category = NULL;


	text[0] = scan_disk_info_get_disk_name ( scan_disk);
	vfs = scan_disk_info_get_vfs ( scan_disk);
	category = scan_disk_info_get_disk_category ( scan_disk);

#if GW_DEBUG_MODE
	g_print ( "%s %d :: catalog_add_disk()\n", __FILE__, __LINE__);
#endif

	/* Creates disk node */
	disk_info = disk_info_new ();
	disk_info_set_name ( disk_info, g_strdup ( scan_disk_info_get_disk_name ( scan_disk)));
	disk_info_set_fsname ( disk_info, g_strdup ( vfs_stats_get_name ( vfs)));
	disk_info_set_dir ( disk_info, g_strdup ( vfs_stats_get_dir ( vfs)));
	disk_info_set_num ( disk_info, scan_disk_info_get_disk_num_archive ( scan_disk));
	disk_info_set_type ( disk_info, g_strdup ( vfs_stats_get_type ( vfs)));
	disk_info_set_volume ( disk_info, g_strdup ( vfs_stats_get_label ( vfs)));
	disk_info_set_full ( disk_info, vfs_stats_get_total_capacity ( vfs));
	disk_info_set_free ( disk_info, vfs_stats_get_total_free_space ( vfs));
	disk_info_set_date ( disk_info, time ( &date));
	disk_info_set_serial ( disk_info, g_strdup ( vfs_stats_get_serial_number ( vfs)));
	disk_info_set_category ( disk_info, scan_disk_info_get_disk_category ( scan_disk));
	disk_info_set_description ( disk_info, NULL);

	disk_data = disk_data_new ();
	disk_data_set_info ( disk_data, disk_info);
	pix = disk_info_get_pixmap ( disk_info);
	msk = disk_info_get_bitmap ( disk_info);
	node = gtk_ctree_insert_node( ctree, parent, NULL, text, 5, pix, msk, pix, msk, FALSE, FALSE);
	gtk_ctree_node_set_row_data_full ( ctree, node, disk_data, (GtkDestroyNotify)disk_data_free);

	/* Gets current folder */
	if ( getcwd ( past_dir, 512) == NULL )
	{
		error_msg = g_strconcat ( _( "Error : cannot get current folder."), "\n", strerror ( errno), NULL);
		scan_disk_info_set_error_msg ( scan_disk, error_msg);

		/* Remove the new disk */
		gtk_ctree_remove_node ( ctree, node);
		gtk_clist_thaw ( GTK_CLIST ( ctree));

		return -1;
	}

	/* Freezes the display of exploration tree */
	gtk_clist_freeze ( GTK_CLIST ( ctree));
	dir_info = dir_info_new ();

	/* Adds the new disk */
	if ( catalog_add_dir ( scan_disk, vfs_stats_get_dir ( vfs), dir_info, ctree, node) == -1 )
	{
		/* Remove the new disk */
		gtk_ctree_remove_node ( ctree, node);
		gtk_clist_thaw ( GTK_CLIST ( ctree));

		/* Goes to the current application directory */
		chdir ( past_dir);

		return -1;
	}

	size = dir_info_get_size ( dir_info);
	disk_info_set_full ( disk_info, size);
	dir_info_free ( dir_info);
	gtk_ctree_sort_recursive ( ctree, parent);
	gtk_clist_thaw ( GTK_CLIST ( ctree));

	/* Goes to the begining folder */
	if ( chdir ( past_dir) == -1 )
	{
		error_msg = g_strconcat ( _( "The disk has been added"), "\n", _( "Error : cannot go to working folder."), "\n", strerror ( errno), NULL);
		scan_disk_info_set_error_msg ( scan_disk, error_msg);
	}

	file_info = gtk_ctree_node_get_row_data ( ctree, parent);
	catalog_file_info_set_ismodified ( file_info, TRUE);

	/* BUG : fix me please */
	/* The total used space different from all files size*/
	if ( !scan_disk_info_get_stop ( scan_disk) )
	{
		scan_disk_info_get_scan_disk_set_current_statment ( scan_disk)( scan_disk_info_get_statment_controler ( scan_disk), vfs_stats_get_total_used_space ( vfs));
	}

	/* Refreshes the exploration tree */
	gtk_ctree_collapse_recursive ( ctree, parent);
	gtk_ctree_expand ( ctree, parent);
	gtk_ctree_select ( ctree, parent);

	return 0;
}


gint catalog_add_dir ( struct scan_disk_info *scan_disk, gchar *directory, struct dir_info *d, GtkCTree *ctree, GtkCTreeNode *parent)
{
	DIR *dir;
	struct dirent *file;
	struct stat f_info;
	struct passwd *f_proprio;
	struct group *f_group;
	guint ret;
	struct dir_data *dir_data = NULL, *tmp_dir_data = NULL;
	struct dir_info *dir_info = NULL;
	struct file_info *file_data = NULL;
	gchar *info_droits = NULL;
	gchar *info_description = NULL;
	gchar buf[1024];
	gchar *text[1];
	gchar *name, *error_msg;
	GtkCTreeNode *node = NULL;
	struct category *category = NULL;


	if ( scan_disk_info_get_stop ( scan_disk) )
	{
#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_add_dir() : stop scan disk\n", __FILE__, __LINE__);
#endif

		error_msg = g_strdup ( _( "The disk hasn't been added\nAdding canceled by user"));
		scan_disk_info_set_error_msg ( scan_disk, error_msg);

		return -1;
	}

	category = scan_disk_info_get_disk_category ( scan_disk);

	if ( chdir ( directory) == -1 )
	{
		switch ( errno )
		{
			case EACCES :	/* Folder is locked */
					gtk_ctree_node_get_text ( ctree, parent, 0, &name);
					gtk_ctree_set_node_info( ctree, parent, name, 5, PixLockedFolder, MskLockedFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);

					return 0;

			case EFAULT:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "%s outside the process's allocated address space."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENAMETOOLONG:error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "A component of the following pathname exceeded %d characters, or an entire path name exceeded %d characters : %s."), NAME_MAX, PATH_MAX, directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOENT:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "The following directory does not exist : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOMEM:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Insufficient memory to complete the operation."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOTDIR:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "A component of the path prefix is not a directory : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
			case ELOOP:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "Too many symbolic links were encountered in resolving following path : %s ."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;
#endif
			case EIO:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "An I/O error occurred when change directory to : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			default:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Unknown error"), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;
		}
	}

	if ( ( dir = opendir ( ".")) == NULL )
	{
		switch (errno )
		{
			case EACCES : /* Folder is locked */
					gtk_ctree_node_get_text ( ctree, parent, 0, &name);
					gtk_ctree_set_node_info( ctree, parent, name, 5, PixLockedFolder, MskLockedFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);

					return 0;

			case EMFILE:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Too many file descriptors in use by process."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENFILE:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Too many files are currently open in the system."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOENT:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "The following directory does not exist, or name is an empty string : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOMEM:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Insufficient memory to complete the operation."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOTDIR:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "The following directory name is not a directory : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			default:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Unknown error"), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;
		}
	}


	dir_data = gtk_ctree_node_get_row_data ( ctree, parent);


	while ( (file = readdir ( dir)) != NULL )
	{
		if ( (ret = stat ( file->d_name, &f_info)) == -1 )
		{
			continue;
		}
		else
		{
			file_data = file_info_new ( );
			file_info_set_isdirectory ( file_data, FALSE);

			/* File name */
			file_info_set_name ( file_data, g_strdup ( file->d_name));
			text[0] = file_info_get_name ( file_data);

			/* File rights */
			file_info_set_rights ( file_data, g_strdup ( "----------"));
			info_droits = file_info_get_rights ( file_data);

#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
			if ( S_ISLNK ( f_info.st_mode) )
			{
				info_droits[0] = 'l';

				memset ( buf, 0, 1024);
				if ( readlink ( g_strconcat ( directory, file->d_name, NULL), buf, 1023) != -1 )
				{
					file_info_set_name ( file_data, g_strconcat ( file->d_name, " -> ", buf, NULL));
				}
			}
			else
			{
#endif
				switch ( S_IFMT & f_info.st_mode )
				{
					case S_IFBLK:	info_droits[0] = 'b';

							break;

#if defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
					case S_IFLNK:	info_droits[0] = 'l';

							memset ( buf, 0, 1024);
							if ( readlink ( g_strconcat ( directory, file->d_name, NULL), buf, 1023) != -1 )
							{
								file_info_set_name ( file_data, g_strconcat ( file->d_name, " -> ", buf, NULL));
							}

							break;

#endif

					case S_IFDIR:	info_droits[0] = 'd';
							file_info_set_isdirectory ( file_data, TRUE);

							/* If is a directory (different to "." and "..") add a subfolder to the parent folder */
							if ( file_info_get_is_real_directory ( file_data) )
							{
								dir_info_add_sub_folder ( d);
							}

							break;

					case S_IFCHR:	info_droits[0] = 'c';

							break;

					case S_IFIFO:	info_droits[0] = 'p';

							break;
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
					case S_IFSOCK:	info_droits[0] = 's';

							break;
#endif
				}
#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
			}
#endif

			/* Owner rights */
			if (S_IRUSR & f_info.st_mode) info_droits[1] = 'r';
			if (S_IWUSR & f_info.st_mode) info_droits[2] = 'w';
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
			if (S_ISUID & f_info.st_mode)
			{
				if (S_IXUSR & f_info.st_mode)
				{
					info_droits[3] = 's';
				}
				else
				{
					info_droits[3] = 'S';
				}
			}
			else
			{
				if (S_IXUSR & f_info.st_mode)
				{
					info_droits[3] = 'x';
				}
			}
#else
					info_droits[3] = 'x';
#endif

			/* Group rights */
			if (S_IRUSR & f_info.st_mode) info_droits[4] = 'r';
			if (S_IWUSR & f_info.st_mode) info_droits[5] = 'w';
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
			if (S_ISUID & f_info.st_mode)
			{
				if (S_IXUSR & f_info.st_mode)
				{
					info_droits[6] = 's';
				}
				else
				{
					info_droits[6] = 'S';
				}
			}
			else
			{
				if (S_IXUSR & f_info.st_mode)
				{
					info_droits[6] = 'x';
				}
			}
#else
					info_droits[3] = 'x';
#endif

			/* Others rights */
			if (S_IRUSR & f_info.st_mode) info_droits[7] = 'r';
			if (S_IWUSR & f_info.st_mode) info_droits[8] = 'w';
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
			if (S_ISUID & f_info.st_mode)
			{
				if (S_IXUSR & f_info.st_mode)
				{
					info_droits[9] = 't';
				}
				else
				{
					info_droits[9] = 'T';
				}
			}
			else
			{
				if (S_IXUSR & f_info.st_mode)
				{
					info_droits[9] = 'x';
				}
			}
#else
					info_droits[3] = 'x';
#endif


#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
			/* Owner file */
			f_proprio = getpwuid ( f_info.st_uid);
			if ( f_proprio != NULL )
			{
				file_info_set_owner ( file_data, g_strdup ( f_proprio->pw_name));
			}
#endif

#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
			/* Owner group file */
			f_group = getgrgid ( f_info.st_gid);
			if ( f_group != NULL )
			{
				file_info_set_group ( file_data, g_strdup ( f_group->gr_name));
			}
#endif

			/* File size */
			file_info_set_size ( file_data, f_info.st_size);

			if ( !scan_disk_info_get_stop ( scan_disk) )
			{
				scan_disk_info_get_scan_disk_set_current_statment ( scan_disk)( scan_disk_info_get_statment_controler ( scan_disk), f_info.st_size);
			}

			/* File inode */
			file_info_set_inode ( file_data, f_info.st_ino);

			/* Creation file date */
			file_info_set_cdate ( file_data, f_info.st_ctime);

			/* Last file access date */
			file_info_set_adate ( file_data, f_info.st_atime);

			/* Last file modification date */
			file_info_set_mdate ( file_data, f_info.st_mtime);

			/* Category file */
			file_info_set_category ( file_data, category);

			/* Description file */
			file_info_set_description ( file_data, g_strdup ( info_description));

			if ( file_info_get_isdirectory ( file_data) == TRUE )
			{
				/* Case : is a folder */
				ret = 0;

				if ( file_info_get_is_real_directory ( file_data) )
				{
					/* Adds the folder node to tree */
					node = gtk_ctree_insert_node ( ctree, parent, NULL, text, 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, FALSE, FALSE);
					tmp_dir_data = dir_data_new ();
					dir_data_set_info ( tmp_dir_data, file_data);
					gtk_ctree_node_set_row_data_full( ctree, node, tmp_dir_data, (GtkDestroyNotify)dir_data_free);

					/* Recursive call to catalog_add_dir */
					dir_info = dir_info_new ();

					if ( catalog_add_dir ( scan_disk, file->d_name, dir_info, ctree, node) == -1 )
					{
						return -1;
					}

					dir_info_add_size ( d, dir_info_get_size ( dir_info));
					file_info_set_size ( file_data, dir_info_get_size ( dir_info));

					if ( dir_info_get_sub_folder ( dir_info) == 0)
					{
						gtk_ctree_set_node_info ( ctree, node, text[0], 5, PixFolder, MskFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);
					}

					dir_info_free ( dir_info);
				}
				else
				{
					dir_data_append_child ( dir_data, file_data);
				}
			}
			else
			{
			  	/* Case : is a file */
			  	/* Adds th efile to file list */
			  	dir_info_add_size ( d, file_info_get_size ( file_data));
				dir_data_append_child ( dir_data, file_data);
			}
		}
	}

	if ( closedir ( dir) == -1 )
	{
		error_msg = g_strdup ( _( "The disk hasn't been added\nError : cannot close current folder."));
		scan_disk_info_set_error_msg ( scan_disk, error_msg);

		return -1;
	}

	if ( chdir ( "..") == -1 )
	{
		error_msg = g_strdup ( _( "The disk hasn't been added\nError : cannot go to parent folder."));
		scan_disk_info_set_error_msg ( scan_disk, error_msg);

		return -1;
	}

	return 0;
}


gint catalog_save_file ( gchar *file, GtkCTree *ctree, GtkCTreeNode *parent)
{
	return catalog_save_file_version_2 ( file, ctree, parent);
}


gint catalog_save_file_version_1 ( gchar *file, GtkCTree *ctree, GtkCTreeNode *parent)
{
	gchar *buf;
	gint fic;
	GtkCTreeNode *node = NULL;
	struct catalog_file_info *file_info = NULL;
	struct disk_data *disk_data = NULL;
	struct disk_info *disk_info = NULL;
	GtkWindow *window = interface_get_main_window ( );


	/* Creates file */
	if ( (fic = open ( file, O_WRONLY|O_TRUNC|O_CREAT, 0666)) == -1 )
	{
		msg_box_create ( window, _( "Save catalog"), _( "Error : cannot create file."));
		return -1;
	}

	/* Writes file version */
	buf = g_strconcat ( PACKAGE, ":", VERSION, ":1\n", NULL);

	if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
	{
		msg_box_create ( window, _( "Save catalog"), _( "Error : cannot write file version."));
		return -1;
	}


	/* Saves all disks */
	node = GTK_CTREE_ROW( parent)->children;
	while ( node != NULL )
	{
		/* Gets disk informations */
		disk_data = gtk_ctree_node_get_row_data ( ctree, node);
		disk_info = disk_data_get_info ( disk_data);
		buf = catalog_disk_info_to_file ( disk_info, 1);

		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}

		/* Recursive call to catalog_save_file_dir_1 */
		catalog_save_file_dir_1 ( fic, ctree, node);
		buf = g_strdup ( "\\\n");
		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}

		/* Goes to the next disk */
		node = GTK_CTREE_ROW ( node)->sibling;
	}


	/* Closes file */
	if ( close ( fic) == -1 )
	{
		msg_box_create ( window, _( "Add disk"), _( "Error : cannot close catalog file."));
		return -1;
	}


	/* Catalog modifications were saved */
	file_info = gtk_ctree_node_get_row_data ( ctree, parent);
	catalog_file_info_set_ismodified ( file_info, FALSE);

	return 0;
}


gint catalog_save_file_version_2 ( gchar *file, GtkCTree *ctree, GtkCTreeNode *parent)
{
	gchar *buf;
	gzFile fic;
	GtkCTreeNode *node = NULL;
	struct catalog_file_info *catalog_info = NULL;
	struct disk_data *disk_data = NULL;
	struct disk_info *disk_info = NULL;
	GList *categories;
	gint i = 0;
	gint size = 0;
	gchar *compression_level = NULL;
	gchar *mode = NULL;
	GtkWindow *window = interface_get_main_window ( );


#if GW_DEBUG_TIME
	time_t begin;
	time ( &begin);
#endif

	/* Gets the file compression level. */
	compression_level = gw_application_manager_get_settings ( GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL);

	if ( compression_level == NULL )
	{
		compression_level = g_strdup_printf ( "%d", GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_DEFAULT);
	}

	/* Builds the open file mode. */
	mode = g_strdup_printf ( "wb%s", compression_level);

#if GW_DEBUG_MODE
		g_print ( "%s %d :: catalog_add_dir() : open file with '%s' mode\n", __FILE__, __LINE__, mode);
#endif

	/* Creates file */
	if ( (fic = gzopen ( file, mode)) == NULL )
	{
		msg_box_create ( window, _( "Save catalog"), _( "Error : cannot create file."));

		return -1;
	}

	g_free ( mode);

	/* Writes file version */
	buf = g_strconcat ( PACKAGE, ":", VERSION, ":2\n", NULL);

	if ( strlen ( buf) != gzputs ( fic, buf))
	{
		msg_box_create ( window, _( "Save catalog"), _( "Error : cannot write file version."));

		if ( gzclose ( fic) != Z_OK)
		{
			msg_box_create ( window, _( "Save catalog"), _( "Error : cannot close file."));
		}

		return -1;
	}

	/* Writes catalog informations */
	catalog_info = gtk_ctree_node_get_row_data ( ctree, parent);
	buf = g_strconcat ( gchar2file ( catalog_file_info_get_name ( catalog_info)), ":", gchar2file ( catalog_file_info_get_description ( catalog_info)), "\n", NULL);

	if ( strlen ( buf) != gzputs ( fic, buf))
	{
		perror ( "write");

		if ( gzclose ( fic) != Z_OK )
		{
			msg_box_create ( window, _( "Save catalog"), _( "Error : cannot close file."));
		}

		return -1;
	}

	/* Writes catelories list */
	if ( (categories = catalog_file_info_get_categories ( catalog_info)) != NULL)
	{
		/* Had to save categories */
		/* Puts one catagory by line */
		categories = g_list_first ( categories);
		size = g_list_length ( categories);

		for ( i = 1; i < size; i++ )
		{
			categories = g_list_next ( categories);
			buf = g_strdup_printf ( "%s:%s\n", gchar2file ( category_get_name ( categories->data)), gchar2file ( category_get_description ( categories->data)));

			if ( strlen ( buf) != gzputs ( fic, buf))
			{
				perror ( "write");

				if ( gzclose ( fic) != Z_OK )
				{
					msg_box_create ( window, _( "Save catalog"), _( "Error : cannot close file."));
				}

				return -1;
			}
		}
	}
	else
	{
		buf = g_strconcat ( "\n", NULL);
		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");

			if ( gzclose(fic) != Z_OK )
			{
				msg_box_create ( window, _( "Save catalog"), _( "Error : cannot close file."));
			}

			return -1;
		}
	}


	/* Saves all disks */
	node = GTK_CTREE_ROW( parent)->children;
	while ( node != NULL )
	{
		/* Gets disk informations */
		disk_data = gtk_ctree_node_get_row_data ( ctree, node);
		disk_info = disk_data_get_info ( disk_data);
		buf = catalog_disk_info_to_file ( disk_info, 2);

		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");

			if ( gzclose ( fic) != Z_OK )
			{
				msg_box_create ( window, _( "Save catalog"), _( "Error : cannot close file."));
			}

			return -1;
		}

		/* Recursive call to catalog_save_file_dir_2 */
		catalog_save_file_dir_2 ( fic, ctree, node);
		buf = g_strdup("\\\n");
		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");

			if ( gzclose ( fic) != Z_OK )
			{
				msg_box_create ( window, _( "Save catalog"), _( "Error : cannot close file."));
			}

			return -1;
		}

		/* Goes to next disk */
		node = GTK_CTREE_ROW ( node)->sibling;
	}


	/* Closes file */
	if ( gzclose ( fic) == -1 )
	{
		msg_box_create ( window, _( "Add disk"), _( "Error : cannot close catalog file."));
		return -1;
	}


	/* Catalog informations were saved */
	catalog_info = gtk_ctree_node_get_row_data ( ctree, parent);
	catalog_file_info_set_ismodified ( catalog_info, FALSE);

#if GW_DEBUG_TIME
	g_print ( _( "Elapsed time to save the catalog : %lu secs\n"), time ( NULL) - begin);
#endif

	return 0;
}


gint catalog_save_file_disk_1 ( gint fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	return -1;
}


gint catalog_save_file_disk_2 ( gint fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	return -1;
}


gint catalog_save_file_dir_1 ( gint fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	gchar *buf;
	struct dir_data *dir_data = NULL;
	struct file_info *file_info = NULL;
	GList *dir = NULL;
	GtkCTreeNode *node = NULL;


	/* Gets current folder informations */
	dir_data = gtk_ctree_node_get_row_data( ctree, parent);

	/* Gets folder files list */
	dir = dir_data_get_child ( dir_data);
	dir = g_list_first ( dir);
	while ( dir != g_list_last ( dir) )
	{
		/* Writes file informations */
		/* Checks if it's a folder different from "." and ".." */
		if ( file_info_get_isdirectory ( dir->data) == TRUE )
		{
			if ( (strcmp ( file_info_get_name ( dir->data), ".") != 0) && (strcmp ( file_info_get_name ( dir->data), "..") != 0) )
			{
				buf = g_strdup ("/\n");
				if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
				{
					perror ( "write");
					return -1;
				}
			}
		}

		/* Gets file informations */
		buf = catalog_file_info_to_file ( dir->data, 1);

		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}

		/* Goes to the next file */
		dir = g_list_next ( dir);
	}

	if ( dir != NULL )
	{
		/* Writes file informations */
		if ( file_info_get_isdirectory ( dir->data) == TRUE )
		{
			if ( (strcmp ( file_info_get_name ( dir->data), ".") != 0) && (strcmp ( file_info_get_name ( dir->data), "..") != 0) )
			{
				buf = g_strdup ( "/\n");
				if ( strlen ( buf) != write( fic, buf, strlen ( buf)))
				{
					perror ( "write");
					return -1;
				}
			}
		}

		/* Gets file informations */
		buf = catalog_file_info_to_file ( dir->data, 1);

		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}
	}


	/* Saves all subfolders */
	node = GTK_CTREE_ROW ( parent)->children;
	while ( node != NULL )
	{
		/* Gets subfolder informations */
		dir_data = gtk_ctree_node_get_row_data ( ctree, node);
		file_info = dir_data_get_info ( dir_data);
		buf = g_strdup ( "/\n");
		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}

		/* Gets subfolder informations */
		buf = catalog_file_info_to_file ( dir->data, 1);

		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}

		/* Recursive call to catalog_save_file_dir_1 */
		catalog_save_file_dir_1 ( fic, ctree, node);
		buf = g_strdup ( "\\\n");
		if ( strlen ( buf) != write ( fic, buf, strlen ( buf)))
		{
			perror ( "write");
			return -1;
		}

		/* Goes to the next subfolder */
		node = GTK_CTREE_ROW ( node)->sibling;
	}

	return 0;
}


gint catalog_save_file_dir_2 ( gzFile fic, GtkCTree *ctree, GtkCTreeNode *parent)
{
	gchar *buf;
	struct dir_data *dir_data = NULL;
	struct file_info *file_info = NULL;
	GList *dir = NULL;
	GtkCTreeNode *node = NULL;


	/* Gets current folder informations */
	dir_data = gtk_ctree_node_get_row_data ( ctree, parent);

	/* Gets folder files list */
	dir = dir_data_get_child ( dir_data);
	dir = g_list_first ( dir);
	while ( dir != g_list_last ( dir) )
	{
		/* Writes file informations */
		/* Checks if it's a folder different from "." and ".." */
		if ( file_info_get_isdirectory ( dir->data) == TRUE )
		{
			if ( (strcmp ( file_info_get_name ( dir->data), ".") != 0) && (strcmp ( file_info_get_name ( dir->data), "..") != 0) )
			{
				buf = g_strdup ( "/\n");
				if ( strlen ( buf) != gzputs ( fic, buf))
				{
					perror( "write");
					return -1;
				}
			}
		}

		/* Gets file informations */
		buf = catalog_file_info_to_file ( dir->data, 2);

		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");
			return -1;
		}

		/* Goes to the next file */
		dir = g_list_next ( dir);
	}

	if ( dir != NULL )
	{
		/* Writes file informations */
		if ( file_info_get_isdirectory ( dir->data) == TRUE )
		{
			if ( (strcmp ( file_info_get_name ( dir->data), ".") != 0) && (strcmp ( file_info_get_name ( dir->data), "..") != 0) )
			{
				buf = g_strdup ( "/\n");
				if ( strlen ( buf) != gzputs ( fic, buf))
				{
					perror ( "write");
					return -1;
				}
			}
		}

		/* Gets file informations */
		buf = catalog_file_info_to_file ( dir->data, 2);

		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");
			return -1;
		}
	}


	/* Saves all subfolders */
	node = GTK_CTREE_ROW( parent)->children;
	while ( node != NULL )
	{
		/* Gets subfolder informations */
		dir_data = gtk_ctree_node_get_row_data ( ctree, node);
		file_info = dir_data_get_info ( dir_data);
		buf = g_strdup ( "/\n");
		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");
			return -1;
		}

		/* Gets subfolder informations */
		buf = catalog_file_info_to_file ( file_info, 2);

		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");
			return -1;
		}

		/* Recursive call to catalog_save_file_dir_2 */
		catalog_save_file_dir_2 ( fic, ctree, node);
		buf = g_strdup ( "\\\n");
		if ( strlen ( buf) != gzputs ( fic, buf))
		{
			perror ( "write");
			return -1;
		}

		/* Goes to the next subfolder */
		node = GTK_CTREE_ROW ( node)->sibling;
	}

	return 0;
}


gint catalog_ctree_data_free( GList *p)
{
	if ( p == NULL )
	{
		return -1;
	}

	g_list_foreach ( p, (GFunc)g_free, NULL);
	g_list_free ( p);

	return 0;
}


struct disk_info * catalog_disk_info_new_from_table ( gchar **attrib, gint file_version)
{
	struct disk_info *p = NULL;


	p = disk_info_new ();

	switch ( file_version )
	{
		case 1:	disk_info_set_name ( p, file2gchar ( attrib[0]));
			disk_info_set_fsname ( p, g_strdup (NULL)); /* on peut le deviner */

			if ( strncmp ( attrib[2], "CD-ROM", 6) == 0 )
			{
				disk_info_set_fsname ( p, g_strdup ( "/dev/cdrom"));
			}
			else { if ( strncmp ( attrib[2], "HD", 2) == 0 )
			{
				disk_info_set_fsname ( p, g_strdup ( "/dev/hd"));
			}
			else { if ( strncmp ( attrib[2], "Floppy", 6) == 0 )
			{
				disk_info_set_fsname ( p, g_strdup ( "/dev/fd"));
			}
			else { if ( strncmp ( attrib[2], "Zip", 3) == 0 )
			{
				disk_info_set_fsname ( p, g_strdup ( "/dev/zip"));
			}
			else
			{
				disk_info_set_fsname ( p, g_strdup ( ""));
			}}}}

			disk_info_set_dir ( p, g_strdup ( NULL));/* difficile  trouver */
			disk_info_set_num ( p, atoi ( attrib[1]));

			if ( strncmp ( attrib[2], "CD-ROM", 6) == 0 )
			{
				disk_info_set_type ( p, g_strdup ( "iso9660"));
			}
			else
			{
				disk_info_set_type ( p, g_strdup ( ""));
			}

			if ( strncmp ( attrib[3], "(null)", 6 ) != 0 )
			{
				if ( strncmp ( attrib[3], "(null)", 6 ) != 0 )
				{
					disk_info_set_volume ( p, file2gchar ( attrib[3]));
				}
			}

			disk_info_set_full ( p, atoi ( attrib[4]));
			disk_info_set_free ( p, atoi ( attrib[5]));
			disk_info_set_date ( p, atoi ( attrib[6]));

			if ( strncmp ( attrib[7], "(null)", 6 ) != 0 )
			{
				disk_info_set_serial ( p, g_strdup ( attrib[7]));
			}

			disk_info_set_description ( p, file2gchar ( attrib[8]));
			break;

		case 2:	disk_info_set_name ( p, file2gchar ( attrib[0]));
			disk_info_set_num ( p, atoi ( attrib[1]));
			disk_info_set_fsname ( p, g_strdup ( attrib[2]));
			disk_info_set_dir ( p, g_strdup ( attrib[3]));
			disk_info_set_type ( p, g_strdup ( attrib[4]));

			if ( strncmp ( attrib[5], "(null)", 6 ) != 0 )
			{
				if ( strncmp ( attrib[5], "", 6 ) != 0 )
				{
					disk_info_set_volume ( p, file2gchar ( attrib[5]));
				}
			}

			disk_info_set_full ( p, atoi ( attrib[6]));
			disk_info_set_free ( p, atoi ( attrib[7]));
			disk_info_set_date ( p, atoi ( attrib[8]));

			if ( strncmp ( attrib[9], "(null)", 6 ) != 0 )
			{
				disk_info_set_serial ( p, g_strdup ( attrib[9]));
			}
			/* Bug to fix!! */
			/* disk_info_set_category ( p, g_strdup ( attrib[10])); */
			disk_info_set_description ( p, file2gchar ( attrib[11]));
			break;

		default:disk_info_free ( p);
			return NULL;
	}

	return p;
}


gchar * catalog_disk_info_to_file ( struct disk_info *p, gint file_version)
{
	gchar *buf;


	if ( p == NULL )
	{
		return NULL;
	}

	switch ( file_version)
	{
		case 1:	buf = g_strdup_printf ( "//\n%s:%d:%s:%s:%ld:%ld:%ld:%s:%s\n",
						gchar2file ( disk_info_get_name ( p)),
						disk_info_get_num ( p),
						disk_info_get_type ( p),
						gchar2file ( disk_info_get_volume ( p)),
						disk_info_get_full ( p),
						disk_info_get_free ( p),
						disk_info_get_date ( p),
						disk_info_get_serial ( p),
						gchar2file ( disk_info_get_description ( p)));
			return buf;

		case 2: buf = g_strdup_printf ( "//\n%s:%d:%s:%s:%s:%s:%ld:%ld:%ld:%s:%d:%s\n",
						gchar2file ( disk_info_get_name ( p)),
						disk_info_get_num ( p),
						disk_info_get_fsname ( p),
						disk_info_get_dir ( p),
						disk_info_get_type ( p),
						gchar2file ( disk_info_get_volume ( p)),
						disk_info_get_full ( p),
						disk_info_get_free ( p),
						disk_info_get_date ( p),
						disk_info_get_serial ( p),
						disk_info_get_category_index ( p),
						gchar2file ( disk_info_get_description ( p)));
			return buf;

		default:return NULL;
	}

	return NULL;
}


struct file_info * catalog_file_info_new_from_table ( gchar **attrib, gboolean folder, gint file_version)
{
	struct file_info *p = NULL;


	p = file_info_new ();

	switch ( file_version )
	{
		case 1:	if ( folder )
			{
				file_info_set_name ( p, file2gchar ( attrib[0]));
				file_info_set_isdirectory ( p, TRUE);
				file_info_set_rights ( p, g_strdup ( attrib[1]));
				file_info_set_owner ( p, g_strdup ( attrib[2]));
				file_info_set_group ( p, g_strdup ( attrib[3]));
				file_info_set_inode ( p, atoi ( attrib[4]));
				file_info_set_size ( p, atoi ( attrib[5]));
				file_info_set_cdate ( p, atoi ( attrib[6]));
				file_info_set_adate ( p, atoi ( attrib[7]));
				file_info_set_mdate ( p, atoi ( attrib[8]));
				file_info_set_description ( p, file2gchar ( attrib[9]));
			}
			else
			{
				file_info_set_name ( p, file2gchar ( attrib[0]));

				if ( (strcmp( attrib[0], ".") != 0) && (strcmp( attrib[0], "..") != 0) )
				{
					file_info_set_isdirectory ( p, FALSE);
				}
				else
				{
					file_info_set_isdirectory ( p, TRUE);
				}

				file_info_set_rights ( p, g_strdup ( attrib[1]));
				file_info_set_owner ( p, g_strdup ( attrib[2]));
				file_info_set_group ( p, g_strdup ( attrib[3]));
				file_info_set_inode ( p, atoi ( attrib[4]));
				file_info_set_size ( p, atoi ( attrib[5]));
				file_info_set_cdate ( p, atoi ( attrib[6]));
				file_info_set_adate ( p, atoi ( attrib[7]));
				file_info_set_mdate ( p, atoi ( attrib[8]));
				file_info_set_description ( p, file2gchar ( attrib[9]));
			}

			break;

		case 2:	if ( folder )
			{
				file_info_set_name ( p, file2gchar ( attrib[0]));
				file_info_set_isdirectory ( p, TRUE);
				file_info_set_rights ( p, g_strdup ( attrib[1]));
				file_info_set_owner ( p, g_strdup ( attrib[2]));
				file_info_set_group ( p, g_strdup ( attrib[3]));
				file_info_set_inode ( p, atoi ( attrib[4]));
				file_info_set_size ( p, atoi ( attrib[5]));
				file_info_set_cdate ( p, atoi ( attrib[6]));
				file_info_set_adate ( p, atoi ( attrib[7]));
				file_info_set_mdate ( p, atoi ( attrib[8]));
				file_info_set_description ( p, file2gchar ( attrib[10]));
			}
			else
			{
				file_info_set_name ( p, file2gchar ( attrib[0]));

				if ( (strcmp( attrib[0], ".") != 0) && (strcmp( attrib[0], "..") != 0) )
				{
					file_info_set_isdirectory ( p, FALSE);
				}
				else
				{
					file_info_set_isdirectory ( p, TRUE);
				}

				file_info_set_rights ( p, g_strdup ( attrib[1]));
				file_info_set_owner ( p, g_strdup ( attrib[2]));
				file_info_set_group ( p, g_strdup ( attrib[3]));
				file_info_set_inode ( p, atoi ( attrib[4]));
				file_info_set_size ( p, atoi ( attrib[5]));
				file_info_set_cdate ( p, atoi ( attrib[6]));
				file_info_set_adate ( p, atoi ( attrib[7]));
				file_info_set_mdate ( p, atoi ( attrib[8]));
				file_info_set_description ( p, file2gchar ( attrib[10]));
			}
			break;

		default:file_info_free ( p);
			return NULL;
	}

	return p;
}


gchar * catalog_file_info_to_file ( struct file_info *p, gint file_version)
{
	gchar *buf;


	if ( p == NULL )
	{
		return NULL;
	}

	switch ( file_version)
	{
		case 1:	buf = g_strdup_printf ( "%s:%s:%s:%s:%ld:%ld:%ld:%ld:%ld:%s\n",
						gchar2file ( file_info_get_name ( p)),
						file_info_get_rights ( p),
						file_info_get_owner ( p),
						file_info_get_group ( p),
						file_info_get_inode ( p),
						file_info_get_size ( p),
						file_info_get_cdate ( p),
						file_info_get_adate ( p),
						file_info_get_mdate ( p),
						gchar2file ( file_info_get_description ( p)));
			return buf;

		case 2:	buf = g_strdup_printf ( "%s:%s:%s:%s:%ld:%ld:%ld:%ld:%ld:%d:%s\n",
						gchar2file ( file_info_get_name ( p)),
						file_info_get_rights ( p),
						file_info_get_owner ( p),
						file_info_get_group ( p),
						file_info_get_inode ( p),
						file_info_get_size ( p),
						file_info_get_cdate ( p),
						file_info_get_adate ( p),
						file_info_get_mdate ( p),
						file_info_get_category_index ( p),
						gchar2file ( file_info_get_description ( p)));
			return buf;

		default:return NULL;
	}

	return NULL;
}
