/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "support.h"
#include "CatalogStats.h"

#include <gtk/gtk.h>

#include <stdlib.h> /* atoi */
#include <string.h> /* strncmp */

#include "data/FileInfo.h"
#include "data/DirInfo.h"
#include "data/DirData.h"
#include "data/DiskInfo.h"
#include "data/DiskData.h"

#include "GuiManager.h"


gint catalog_get_total_disks ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	gint n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW ( node)->children;
	while ( child != NULL )
	{
		n++;
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint catalog_get_total_folders ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	int n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		/* Counts the number of files for each disk. */
		n = n + disk_get_total_folders ( ctree, child);
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint catalog_get_total_files ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	int n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		/* Counts the number of folders for each disk. */
		n = n + disk_get_total_files ( ctree, child);
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


long double catalog_get_total_capacity ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	long double n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		/* Counts the size of each disk. */
		n = n + disk_info_get_full ( disk_data_get_info ( gtk_ctree_node_get_row_data( ctree, child))) + disk_info_get_free ( disk_data_get_info ( gtk_ctree_node_get_row_data( ctree, child)));
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


long double catalog_get_total_used_space ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	long double n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		/* Counts the used space of each disk. */
		n = n + disk_info_get_full ( disk_data_get_info ( gtk_ctree_node_get_row_data( ctree, child)));
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


long double catalog_get_total_free_space ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	long double n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		/* Counts the free space of each disk. */
		n = n + disk_info_get_free ( disk_data_get_info ( gtk_ctree_node_get_row_data( ctree, child)));
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint catalog_get_num_disk_with_type ( GtkCTree *ctree, GtkCTreeNode *node, gchar *type)
{
	GtkCTreeNode *child = NULL;
	struct disk_info *disk_info;
	gint n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		if ( (disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, child))) == NULL )
		{
			disk_info = NULL;
			child = GTK_CTREE_ROW ( child)->sibling;
			continue;
		}

		/* Checks the disk type. */
		if ( strncmp ( type, disk_info_get_type ( disk_info), strlen ( type)) == 0 )
		{
			if ( strncmp ( type, "iso9660", 7) == 0 )
			{
				n++;
			}
		}
		else { if ( strncmp ( type, disk_info_get_fsname ( disk_info), strlen ( type)) == 0 )
		{
			if ( strncmp ( type, "/dev/fd", 7) == 0 )
			{
				n++;
			}
			else { if ( strncmp ( type, "/dev/zip", 8) == 0 )
			{
				n++;
			}
			else { if ( strncmp ( type, "/dev/hd", 7) == 0 )
			{
				n++;
			}}}
		}
		else { if ( strncmp ( type, "Other", 5) == 0 )
		{
			n++;
			if ( strncmp ( "iso9660", disk_info_get_type ( disk_info ), 7) == 0 )
			{
				n--;
			}
			else { if ( strncmp ( disk_info_get_fsname ( disk_info), "/dev/fd", 7) == 0 )
			{
				n--;
			}
			else { if ( strncmp ( disk_info_get_fsname ( disk_info), "/dev/zip", 8) == 0 )
			{
				n--;
			}
			else { if ( strncmp ( disk_info_get_fsname ( disk_info), "/dev/hd", 7) == 0 )
			{
				n--;
			}}}}
		}}}
		
/*		if ( strncmp ( type, disk_info_get_type ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, child))), strlen ( type)) == 0 )
		{
			n++;
		}
*/
		disk_info = NULL;
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint disk_get_total_folders ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	int n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		n++;
		n = n + disk_get_total_folders ( ctree, child);
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint disk_get_total_files ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	int n = 0;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Counts the folder's number of files. */
	n = dir_data_get_num_file ( gtk_ctree_node_get_row_data( ctree, node));

	child = GTK_CTREE_ROW( node)->children;
	while ( child != NULL )
	{
		/* Does the same thing for each subfolder. */
		n = n + disk_get_total_files ( ctree, child);
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint disk_get_max_num ( GtkCTree *ctree, GtkCTreeNode *root)
{
	gint max = 0;
	GtkCTreeNode *node = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Checks if a catalog is opened. */
	if ( root != NULL )
	{
		node = GTK_CTREE_ROW( root)->children;

		/* Checks if there are some disk. */
		while ( node != NULL )
		{
			if ( disk_info_get_num ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node))) > max )
			{
				max = disk_info_get_num ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node)));
			}

			/* Goes to the next branch (ie. the next disk). */
			node = GTK_CTREE_ROW ( node)->sibling;
		}
	}
	else
	{
		/* If no opened disk. */
		return 0;
	}

	return max;
}


gboolean disk_name_is_exist ( gchar *name)
{
	GtkCTreeNode *node = NULL;
	GtkCTree *ctree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	ctree = interface_get_tree ( );
	root = interface_get_tree_root ( );

	/* Checks if a catalog is opened. */
	if ( root != NULL )
	{
		node = GTK_CTREE_ROW( root)->children;
		while ( node != NULL )
		{
			if ( strcmp ( disk_info_get_name ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node))), name) == 0 )
			{
				return TRUE;
			}

			/* Goes to the next branch (ie. the next disk). */
			node = GTK_CTREE_ROW ( node)->sibling;
		}
	}
	else
	{
		/* If no opened disk. */
		return FALSE;
	}

	return FALSE;
}


gboolean disk_num_is_exist ( gint num)
{
	GtkCTreeNode *node = NULL;
	GtkCTree *ctree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	ctree = interface_get_tree ( );
	root = interface_get_tree_root ( );

	/* Checks if a catalog is opened. */
	if ( root != NULL )
	{
		node = GTK_CTREE_ROW( root)->children;
		while ( node != NULL )
		{
			if ( disk_info_get_num ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node))) == num )
			{
				return TRUE;
			}

			/* Goes to the next branch (ie. the next disk). */
			node = GTK_CTREE_ROW ( node)->sibling;
		}
	}
	else
	{
		/* If no opened disk. */
		return FALSE;
	}

	return FALSE;
}


GtkCTreeNode * folder_get_catalog ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *parent = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	parent = GTK_CTREE_ROW( node)->parent;
	while ( GTK_CTREE_ROW ( parent)->parent != NULL )
	{
		parent = GTK_CTREE_ROW( parent)->parent;
	}

	return parent;
}


GtkCTreeNode * folder_get_disk ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *parent = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	root = interface_get_tree_root ( );

	if ( GTK_CTREE_ROW ( node)->parent == root)
	{
		return node;
	}

	parent = GTK_CTREE_ROW( node)->parent;
	while ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( parent)->parent)->parent != NULL )
	{
		parent = GTK_CTREE_ROW( parent)->parent;
	}

	return parent;
}


gchar * folder_get_disk_name ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *parent = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	root = interface_get_tree_root ( );

	if ( GTK_CTREE_ROW ( node)->parent == root )
	{
		return disk_info_get_name ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node)));
	}

	parent = GTK_CTREE_ROW( node)->parent;
	while ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( parent)->parent)->parent != NULL )
	{
		parent = GTK_CTREE_ROW( parent)->parent;
	}

	return disk_info_get_name ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, parent)));
}


gchar * folder_get_disk_type ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *parent = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	parent = GTK_CTREE_ROW( node)->parent;
	while ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( parent)->parent)->parent != NULL )
	{
		parent = GTK_CTREE_ROW( parent)->parent;
	}

	return disk_info_get_type ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, parent)));
}


gchar * folder_get_location_on_disk ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *parent = NULL;
	GtkCTreeNode *root = NULL;
	gchar *text;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	root = interface_get_tree_root ( );

	if ( GTK_CTREE_ROW ( node)->parent == root )
	{
		text = g_strdup ( "/");
		return text;
	}

	text = g_strconcat ( "/", file_info_get_name ( dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node))), NULL);

	parent = GTK_CTREE_ROW( node)->parent;
	while ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( parent)->parent)->parent != NULL )
	{
		text = g_strconcat ( "/", file_info_get_name ( dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree, parent))), text, NULL);
		parent = GTK_CTREE_ROW( parent)->parent;
	}

	return text;
}


gint disk_get_num_archive ( gchar *num)
{
	GtkCTree *ctree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	ctree = interface_get_tree ( );
	root = interface_get_tree_root ( );

	if ( strcmp ( num, "") == 0 )
	{
		/* If number archive no filled. */
		return disk_get_max_num ( ctree, root) + 1;
	}
	else { if ( disk_num_is_exist ( atoi ( num)) )
	{
		/* If the filled archive number exists. */
		return disk_get_max_num ( ctree, root) + 1;
	}
	else
	{
		/* If the filled archive number is valid. */
		return atoi ( num);
	}}
}


gchar * disk_get_name ( gchar *name)
{
	gchar *tmp_name;
	gint i = 2;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	tmp_name = g_strdup ( name);

	while ( disk_name_is_exist ( tmp_name) )
	{
		tmp_name = g_strdup_printf ( "%s (%d)", name, i++);
	}

	return tmp_name;
}
