/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "support.h"
#include "DeviceManager.h"


/* To open and close files */
#include <unistd.h> /* lseek read close */
#include <fcntl.h> /* open */

#include <stdlib.h> /* exit */
#include <errno.h> /* codes d'erreur */
#include <string.h> /* strncmp malloc*/
#include <stdio.h> /* perror*/

/* To mount device */
#include <sys/stat.h> /* stat */

#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
    #include <mntent.h> /* setmntent getmntent endmntent */
    #include <sys/mount.h> /* mount umount */
    #include <sys/wait.h>
#endif

/* To get information on the file system */
#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
    #include <sys/vfs.h> /* statfs */
#endif

#include "data/VirtualFileSystemStats.h"
#include "tools/FileSystem.h"


gint vfs_stats_load ( struct vfs_stats *p, gchar *dir)
{
	/* Loads all file system  informations */
#if defined ( OS_CYGWIN)
	gchar *device_name = NULL;
	gchar *volume_name = NULL;
#endif


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( p == NULL )
	{
		/* The virtual file system hasn't been allocated */
		g_print ( "p == (null)\n");

		return -1;
	}

	if ( dir == NULL )
	{
		/* The file system path is NULL */
		g_print ( "dir == (null)\n");

		return -1;
	}

#if defined ( OS_LINUX)
	/* Checks if the device is in the mounted device table (/etc/mtab) */
	if ( vfs_stats_disk_search_info ( p, "/etc/mtab", dir) == 0 )
	{
		return 0;
	}

	/* Checks if the device is in the mountabled device table (/etc/fstab) */
	if ( vfs_stats_disk_search_info ( p, "/etc/fstab", dir) == 0 )
	{
		return 0;
	}
#elif defined ( OS_CYGWIN)
	disk_get_device_name ( dir, &device_name);
	vfs_stats_set_name ( p , device_name);
	vfs_stats_set_dir ( p, g_strdup ( dir));
	vfs_stats_set_options ( p, NULL);
	vfs_stats_set_type ( p , g_strdup ( "vfat"));
	vfs_stats_set_ismounted ( p, TRUE);
	vfs_stats_set_nb_files ( p, 0);
	vfs_stats_set_blocks_size ( p, 0);
	vfs_stats_set_blocks ( p, 0);
	vfs_stats_set_blocks_free ( p, 0);
//	vfs_stats_set_label ( p, vfs_stats_disk_get_label ( p));
	gw_fs_get_volume_name ( device_name, &volume_name);
	vfs_stats_set_label ( p, volume_name);
#endif

	/* If the path file system doesn't exist */
	return -1;
}




gboolean vfs_stats_disk_try_mount ( struct vfs_stats *p)
{
/*
// First methode failed : single user cannot use mount()

	if ( mount ( vfs_stats_get_name ( p), vfs_stats_get_dir ( p), vfs_stats_get_type ( p), 0xc0ed0000|MS_RDONLY, "") == -1 )
	{
		if ( errno == EBUSY )
		{
g_print ( "try_mount : device busy\n");
			return TRUE;
		}
		else
		{
g_print ( "try_mount : cannot mount\n");
			return FALSE;
		}
	}
	else
	{
		umount ( vfs_stats_get_dir ( p));

g_print ( "try_mount : can mount\n");
		return TRUE;
	}
*/
/*
// Second methode failed : cannot detect if the device is allready mounted
	if ( vfs_stats_disk_mount ( p) == TRUE )
	{
		vfs_stats_disk_umount ( p);

		return TRUE;
	}

	return FALSE;
*/
/*
// Third methode failed : umount an umounted device doesn't failed
	if ( vfs_stats_disk_umount ( p) == TRUE )
	{
		vfs_stats_disk_mount ( p);

		return TRUE;
	}
	else
	{
		if ( vfs_stats_disk_mount ( p) == TRUE )
		{
			vfs_stats_disk_umount ( p);

			return TRUE;
		}

	return FALSE;
	}

	return FALSE;
*/
	/* Gets the label of a disk */
	struct stat statbuf;
	guint fic;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Gets rights of the disk */
	if ( stat ( vfs_stats_get_name ( p), &statbuf) == -1 )
	{
		perror ( "stat");

		return FALSE;
	}

	/* If it's a regular file */
	if ( S_ISREG ( statbuf.st_mode) )
	{
		return FALSE;
	}

	/* Cannot read only the file */
	if ( access ( vfs_stats_get_name ( p), R_OK) != 0 )
	{
        		return FALSE;
	}

	/* Opens the device */
	if ( ( fic = open ( vfs_stats_get_name ( p), O_RDONLY ) ) == -1 )
	{
		perror ( "open");

		return FALSE;
	}

	/* Closes the device file system */
	close ( fic);

	return TRUE;
}


gint vfs_stats_disk_search_info ( struct vfs_stats *p, gchar *file_tab, gchar *dir)
{
	FILE *tab = NULL;
#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
	struct mntent *partition = NULL;
	struct statfs sts;
	gchar *volume_name = NULL;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( p == NULL )
	{
		/* The virtual file system hasn't been allocated */
		return -1;
	}

	if ( dir == NULL )
	{
		/* The file system path is NULL */
		return -1;
	}

	tab = (FILE*)malloc ( sizeof ( tab));
	if ( (partition = (struct mntent*)malloc ( sizeof ( struct mntent))) == NULL)
	{
		return -1;
	}

	/* Opens the device table file */
	tab = setmntent ( file_tab, "r");

	while ( (partition = getmntent ( tab)) != NULL)
	{
             	if ( strcmp ( partition->mnt_dir, dir) == 0 )
		{
			/* Bug to fix : must free all data before set new informations */
			vfs_stats_set_name ( p , g_strdup ( partition->mnt_fsname));
			vfs_stats_set_dir ( p, g_strdup ( partition->mnt_dir));
			vfs_stats_set_options ( p, g_strsplit ( partition->mnt_opts, ",", 0));
			vfs_stats_set_type ( p , g_strdup ( partition->mnt_type));

			if ( strncmp ( file_tab, "/etc/mtab", 9) == 0 )
			{
				/* If the device table file is /etc/mtab */
				vfs_stats_set_ismounted ( p, TRUE);
			}
			else { if ( strncmp ( file_tab, "/etc/fstab", 10) == 0 )
			{
				/* If the device table file is /etc/fstab */
				vfs_stats_set_ismounted ( p, FALSE);
			}}

			if ( vfs_stats_can_mounted ( p) )
			{

				if ( vfs_stats_is_mounted ( p) )
				{
//					vfs_stats_set_label ( p, vfs_stats_disk_get_label ( p));
					gw_fs_get_volume_name ( vfs_stats_get_name ( p), &volume_name);
					vfs_stats_set_label ( p, volume_name);
				}
				else
				{
					vfs_stats_disk_mount ( p);
//					vfs_stats_set_label ( p, vfs_stats_disk_get_label ( p));
					gw_fs_get_volume_name ( vfs_stats_get_name ( p), &volume_name);
					vfs_stats_set_label ( p, volume_name);

					if ( !vfs_stats_get_ismounted ( p) )
					{
                                                                  	vfs_stats_disk_umount ( p);
					}
				}
			}
			else
			{
				/* The virtual file system cannot be mounted */
				vfs_stats_set_ismounted ( p, FALSE);
			}

			if ( statfs ( dir, &sts) == -1 )
			{
				perror ( "statfs");

				return -1;
			}

			vfs_stats_set_nb_files ( p, sts.f_files);
			vfs_stats_set_blocks_size ( p, sts.f_bsize);
			vfs_stats_set_blocks ( p, sts.f_blocks);
			vfs_stats_set_blocks_free ( p, sts.f_bfree);
//			vfs_stats_set_id ( p, sts.f_fsid);

//			free ( &sts);
//			g_free ( partition);
			endmntent ( tab);

#ifdef GW_DEBUG_MODE
			vfs_stats_display ( p);
#endif

			return 0;
		}
	}

	endmntent ( tab);
#endif
	return -1;
}

#if defined ( OS_LINUX) || defined ( OS_CYGWIN)
gboolean vfs_stats_disk_mount ( struct vfs_stats *p)
{
	/* Mount device */
	int pid;		/* pid number		*/
	int status;	/* process status		*/
	int fd[2];		/* pipe descriptor		*/
	char *argv[4] = {"mount", vfs_stats_get_dir ( p), 0};
 	char retour[201];


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Creates pipe */
	if ( pipe ( fd) != 0 )
	{
		/* Cannot create pipe */

		perror ( "pipe");

		return FALSE;
	}

	/* Creates a fork */
	pid = fork (  );

	if ( pid == 0 )
	{
		/* If it's the child */
		close ( fd[0]);
		close ( 2);
		close ( 1);

		/* Redirect the output error to the pipe */
		dup2 ( fd[1], 2);
		close ( fd[1]);

//		if ( execv( "/bin/mount", argv) == -1 )
		if ( execv( GW_PROG_PATH_MOUNT, argv) == -1 )
		{
			perror ( "execv");
		}

		exit ( 127);
	}
	else
	{
		/* If it's the parent */
		if ( pid == -1 )
		{
			perror ( "fork");
		}
	}

	close ( fd[1]);

        	/* Gets the pipe data */
	pipe_get_data ( fd[0], retour);

	if ( waitpid ( pid, & status, 0) == -1 )
	{
		perror ( "waitpid");
	}

	 if ( WEXITSTATUS ( status) != 0 )
	 {
//		 msg_box_create ( "Mount device", retour);

		 return FALSE;
	 }

	 return TRUE;
}


gboolean vfs_stats_disk_umount( struct vfs_stats *p)
{
	/* Umout device */
	gint pid;		/* pid number		*/
	gint status;	/* process status		*/
	gint fd[2];		/* pipe descriptor		*/
	gchar *argv[4] =  {"umount", vfs_stats_get_dir ( p), 0};
 	gchar retour[201];


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Creates pipe */
	if ( pipe ( fd) != 0 )
	{
		perror ( "pipe");

		return FALSE;
	}

	/* Creates fork */
	pid = fork ( );
	if ( pid == 0 )
	{
		/* If it's the child */
		close ( fd[0]);
		close ( 2);
		close ( 1);

		/* Redirect the output error to the pipe */
		dup2 ( fd[1], 2);
		close ( fd[1]);

//		if ( execv ( "/bin/umount", argv) == -1 )
		if ( execv ( GW_PROG_PATH_UMOUNT, argv) == -1 )
		{
			perror ( "execv");
		}

		exit ( 127);
	}
	else
	{
		/* If it's the parent */
		if ( pid == -1 )
		{
			perror ( "fork");
		}
	}

	close ( fd[1]);

        	/* Gets the pipe data */
	pipe_get_data ( fd[0], retour);

	/* Waits the child process end */
	if ( waitpid ( pid, & status, 0) == -1 )
	{
		perror ( "waitpid");
	}

	 if ( WEXITSTATUS ( status) != 0 )
	 {
//		 msg_box_create ( "Mount device", retour);

		 return FALSE;
	 }

	return TRUE;
}
#endif

gint pipe_get_data ( gint entry, gchar * line)
{
	gint nb;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	while ( (nb = read ( entry, line, 200)) > 0 )
	{
		line[nb] = '\0';
	};

	if ( nb < 0 )
	{
		perror ( "read");
	};

 	return 0;
}


gint disk_get_device_name ( const gchar *mount_point, gchar **device_name)
{
	gint result = -1;


#ifdef GW_DEBUG_MODE
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( mount_point != NULL )
	{
		if ( strlen ( mount_point) > 10 )
		{
			switch ( mount_point[10])
			{
				case 'a' : 	*device_name = g_strdup ( "/dev/fd0");
						result = 0;
				
						break;

				case 'b' : 	*device_name = g_strdup ( "/dev/fd1");
						result = 0;
				
						break;

				case 'c' : 	*device_name = g_strdup ( "/dev/sda1");
						result = 0;

						break;

				case 'd' : 	*device_name = g_strdup ( "/dev/sda2");
						result = 0;
				
						break;

				default :	break;

			}
		}
	}

	return result;
}


gboolean vfs_stats_is_mounted ( struct vfs_stats *p)
{
#ifdef GW_DEBUG_DATA_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( p != NULL )
	{
		if ( strncmp ( vfs_stats_get_type ( p), "supermount", 10) == 0 )
		{
			/* If the device is in supermount mode, try to acces it */

			return vfs_stats_disk_try_mount ( p);
		}
		else
		{
			/* Returns the mount state */

			return vfs_stats_get_ismounted ( p);
		}
	}

	return FALSE;
}


gboolean vfs_stats_can_mounted ( struct vfs_stats *p)
{
#ifdef GW_DEBUG_DATA_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( p != NULL )
	{
		if ( vfs_stats_get_ismounted ( p) == TRUE)
		{
			return TRUE;
		}
		else
		{
			return vfs_stats_disk_try_mount ( p);
		}
	}

	return FALSE;
}
