/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "support.h"
#include "GuiManager.h"

#include <gtk/gtk.h>

#include "gui/Menu.h"
#include "gui/MenuFileCallBack.h"
#include "gui/ToolBar.h"
#include "gui/NoteBook.h"
#include "gui/StatusBar.h"

#include "gui/GWReferences.h"


GtkWindow *window;
GtkCTreeNode *root;


GtkWindow * interface_create ( )
{
	GtkWidget *MainWindow;
	GtkWidget *VbMainWindow;
	GtkWidget *menu_bar = NULL;
	GtkWidget *tool_bar = NULL;
	GtkWidget *notebook = NULL;
	GtkWidget *status_bar = NULL;
	GtkAccelGroup *accel_group;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	accel_group = gtk_accel_group_new ();

	/* Main window */
	MainWindow = gtk_window_new ( GTK_WINDOW_TOPLEVEL);
	window = GTK_WINDOW ( MainWindow);

	gtk_window_set_policy ( window, TRUE, TRUE, FALSE);
	gtk_widget_set_usize ( MainWindow, 800, 600);
	gtk_window_set_title ( GTK_WINDOW ( MainWindow), PACKAGE);
  	gtk_signal_connect ( GTK_OBJECT( MainWindow), "delete_event", GTK_SIGNAL_FUNC ( menu_file_exit_click), NULL);
  	gtk_signal_connect ( GTK_OBJECT( MainWindow), "destroy", GTK_SIGNAL_FUNC ( menu_file_exit_click), NULL);
	gtk_window_set_position ( window, GTK_WIN_POS_CENTER);
	gtk_widget_realize ( MainWindow);

	/* Vertical box */
	VbMainWindow = gtk_vbox_new ( FALSE, 0);
	gtk_widget_ref ( VbMainWindow);
	gtk_object_set_data_full ( GTK_OBJECT ( MainWindow), "VbMainWindow", VbMainWindow, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( MainWindow), VbMainWindow);

	/* Menu bar */
	menu_bar = menu_create ( window, accel_group, VbMainWindow);
	gtk_box_pack_start ( GTK_BOX ( VbMainWindow), menu_bar, FALSE, FALSE, 0);

	/* Tool bar */
	tool_bar = tool_bar_create_and_add ( window, VbMainWindow);

	/* Notebook */
	notebook = notebook_create ( window, VbMainWindow);
	gtk_box_pack_start ( GTK_BOX ( VbMainWindow), notebook, TRUE, TRUE, 0);

	/* Status bar */
	status_bar = status_bar_create_and_add ( window, VbMainWindow);


	gtk_window_add_accel_group ( window, accel_group);

	return window;
}


gint interface_set_tree_root ( GtkCTreeNode *node)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	root = node;

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	if ( root == NULL )
	{
		g_print ( "*** GW - %s (%d) :: %s() : root node is NULL\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
	}
#endif

	return 0;
}



GtkWindow * interface_get_main_window ( void)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	if ( window == NULL )
	{
		g_print ( "*** GW - %s (%d) :: %s()\n : main window is NULL", __FILE__, __LINE__, __PRETTY_FUNCTION__);
	}
#endif

	return window;
}


GtkWindow * interface_get_progress_bar_box ( void)
{
	GtkWindow *progress_bar_box = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	progress_bar_box = gtk_object_get_data ( GTK_OBJECT ( window), GW_REF_PROGRESS_BAR_BOX);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	if ( progress_bar_box == NULL )
	{
		g_print ( "*** GW - %s (%d) :: %s()\n : progress bar box is NULL", __FILE__, __LINE__, __PRETTY_FUNCTION__);
	}
#endif

	return progress_bar_box;
}


GtkCTree * interface_get_tree ( void)
{
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	tree = GTK_CTREE ( gtk_object_get_data ( GTK_OBJECT ( window), GW_REF_TREE));

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	if ( tree == NULL )
	{
		g_print ( "*** GW - %s (%d) :: %s() : tree is NULL\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
	}
#endif

	return tree;
}


GtkCTreeNode * interface_get_tree_root ( void)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : root node is NULL\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return root;
}


void interface_update ( void)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	while ( gtk_events_pending ( ) )
	{
		gtk_main_iteration ( );
	}
}
