/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "support.h"
#include "Supermount.h"

#include <glib.h>

/* To open and close files */
#include <unistd.h>
#include <fcntl.h>

#include <stdlib.h> /* malloc free */
#include <errno.h> /* codes d'erreur */
#include <string.h> /* strncmp */
#include <stdio.h> /* sprintf */

/* To mount partition */
#include <sys/stat.h>

#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* To get informations about a file system */
	#include <sys/vfs.h>

	/* To eject a CD */
	#include <sys/ioctl.h>
	#include <sys/mtio.h>
#endif

#if defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* To eject a CD */
	#include <sys/cdio.h>
	#define CDROMEJECT DIOCEJECT
#endif

/* Must be after support.h!! */
#if defined ( OS_LINUX)
	/* #include <linux/fs.h> */
	#include <linux/cdrom.h>
#endif

#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* #include <fstab.h> */
	#include <mntent.h>
	#include <sys/mount.h>
#endif

#if defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	#include <sys/param.h>
#endif

#if defined ( OS_CYGWIN)
	#include <sys/cygwin.h> /* cygwin_conv_to_full_win32_path */
#endif

#if defined ( OS_CYGWIN) || defined ( OS_WINDOWS)
	#include <sys/types.h> /* opendir */
	#include <dirent.h> /* opendir */
#elif defined ( OS_WINDOWS)
    #include <io.h>
    #include <sys/stat.h>
#endif


GList * Charger_media( GList *g)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Definie une liste avec tous les points de montage et renvoie un pointeur soit sur la liste */
	FILE *fstab;
	struct mntent *partition;
	gchar *fsname = NULL;


	fstab = (FILE*)malloc(sizeof(fstab));
	partition = (struct mntent*)malloc(sizeof(struct mntent));
	/*g_print("Nom : %s\nChemin : %s\nType : %s\nOptions : %s\nFrquence : %d\nOrdre : %d\n\n",partition->mnt_fsname,partition->mnt_dir,partition->mnt_type,partition->mnt_opts,partition->mnt_freq,partition->mnt_passno);*/

	/* Ouvre le fichier /etc/fstab */
	fstab = setmntent( "/etc/fstab", "r");

	while ( (partition = getmntent( fstab)) != NULL)
	{
		if (strcmp(partition->mnt_type,"proc") != 0)
		{
			if (strcmp(partition->mnt_type,"devpts") != 0)
			{
				if (strcmp(partition->mnt_type,"swap") != 0)
				{
 					fsname = g_strdup( partition->mnt_dir);
					g = g_list_insert_sorted( g, fsname, (GCompareFunc)strcmp);
				}
			}
		}
	}

	endmntent ( fstab);
	fstab = setmntent( "/etc/mtab", "r");

	while ( (partition = getmntent( fstab)) != NULL)
	{
		if (strcmp(partition->mnt_type,"proc") != 0)
		{
			if (strcmp(partition->mnt_type,"devpts") != 0)
			{
				if (strcmp(partition->mnt_type,"swap") != 0)
				{
 					fsname = g_strdup( partition->mnt_dir);

	 				if ( g_list_find_custom ( g, fsname, (GCompareFunc)strcmp) == NULL )
					{
						g = g_list_insert_sorted( g, fsname, (GCompareFunc)strcmp);
					}
				}
			}
		}
	}

	endmntent ( fstab);
	free ( partition);
#elif defined ( OS_CYGWIN)
	DIR *dir;
	struct dirent *file;
//char *posix_name = NULL;
//char win32_name[PATH_MAX];


	if ( ( dir = opendir ( "/cygdrive")) == NULL )
	{
/*		switch (errno )
		{
			case EACCES : 	gtk_ctree_node_get_text ( ctree, parent, 0, &name);
					gtk_ctree_set_node_info( ctree, parent, name, 5, PixLockedFolder, MskLockedFolder, PixOpenFolder, MskOpenFolder, TRUE, FALSE);

					return 0;

			case EMFILE:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Too many file descriptors in use by process."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENFILE:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Too many files are currently open in the system."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOENT:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "The following directory does not exist, or name is an empty string : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOMEM:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Insufficient memory to complete the operation."), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			case ENOTDIR:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), g_strdup_printf ( _( "The following directory name is not a directory : %s."), directory), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

					return -1;

			default:	error_msg = g_strconcat ( _( "The disk hasn't been added\n"), _( "Unknown error"), NULL);
					scan_disk_info_set_error_msg ( scan_disk, error_msg);

				return -1;
		}
*/
		g_print ( "error : cannot open %s\n", "/cygdrive");

		return NULL;
	}

	while ( (file = readdir ( dir)) != NULL )
	{
		if ( strncmp ( ".", file->d_name, 1) != 0 )
		{
			if ( strncmp ( "..", file->d_name, 2) != 0 )
			{
				g = g_list_insert_sorted ( g, g_strconcat ( "/cygdrive/", file->d_name, NULL), (GCompareFunc)strcmp);
//posix_name = g_strconcat ( "/cygdrive/", file->d_name, NULL);
//cygwin_conv_to_full_win32_path ( posix_name, win32_name);
//g_print ( "Unix path = %s and Win32 path = %s\n", posix_name, win32_name);
			}
		}
	}

	if ( closedir ( dir) == -1 )
	{
		g_print ( "error : cannot close %s\n", "/cygdrive");

		/* BUG TO FIX : closedir() allways fails on Cygwin
		return NULL;
		*/
	}
#endif

	return g;
}


gboolean Eject_media ( gchar *mount_point)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Ejecte le disk "mount_point" qui est un char * du point de montage */
	FILE *fstab;
	struct mntent *pm;
	gint fd=0;
	gint status;

	fstab = (FILE*)malloc(sizeof(fstab));
	pm = (struct mntent*)malloc(sizeof(struct mntent));

	fstab = setmntent ( "/etc/fstab", "r");

	while ( (pm = getmntent ( fstab)) != NULL)
	{
		if ( strcmp ( pm->mnt_dir, mount_point) == 0 )
		{
			/* Recherche du type de montage */
			if ( strcmp(pm->mnt_type,"supermount") == 0 )
			{
				/* On recupere le nom du dev */
				char *temp;
				temp = strtok ( hasmntopt ( pm, "dev"), "=");
				temp = strtok ( NULL, "=");

				if ( temp )
				{
					fd=open(temp, O_RDONLY|O_NONBLOCK );
				}

			}
			else
			{
				fd = open ( pm->mnt_fsname, O_RDONLY|O_NONBLOCK );
			}

			if ( fd >= 0 )
			{
				status = ioctl ( fd, CDROMEJECT);

				close ( fd);
			}

			endmntent ( fstab);

			//free ( pm);
			return TRUE;
		}
	}

	endmntent ( fstab);
	//free ( pm);
#endif

	return FALSE;
}


gint Mount_disk ( gchar *disk)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Cette fonction sert  monter les points de montage qui se trouvent dans le fichier /ect/fstab	*/
	/* Elle retourne :	0 si le montage c'est effectu sans problmes					*/
	/*			1 si le point de montage tait dj mont					*/
	/*			-1 sinon (cad que le montage n'est pas fait)					*/


	static gchar *filesystemtype[] = {"iso9660", "fat", "ext2", "vfat", "msdos", "minix", "ext"};
	FILE *fstab;
	struct mntent *pm;
	gint j = 0;

	fstab = (FILE*)malloc(sizeof(fstab));
	pm = (struct mntent*)malloc(sizeof(struct mntent));

	fstab = setmntent ( "/etc/fstab", "r");

	while ( (pm = getmntent ( fstab)) != NULL)
	{
		if ( strcmp ( pm->mnt_dir, disk) == 0 )
		{
			/* Recherche si le point de montage n'est pas automatique */
			if ( hasmntopt ( pm, "noauto") != NULL )
			{
				if ( strcmp ( pm->mnt_type, "auto") == 0)
				{
					/* Si le filesystemtype n'est pas prcis */
					/* On en essaye parmis les plus courants */
					while ( j <= 6 )
					{
						if ( mount ( pm->mnt_fsname, pm->mnt_dir, filesystemtype[j++], 0xc0ed0000|MS_RDONLY, "") == 0 )
						{
							/* Le point de montage a t mont correctement */
							endmntent ( fstab);

							return 0;
						}
						else
						{
							switch ( errno )
							{
								case EPERM :		g_print ( "L'utilisateur n'est pas le  Super-User\n");
											endmntent ( fstab);

											return -1;
											break;

								case ENODEV :		/* Ce type de systme de fichier n'existe pas */
											g_print ( "Warning : %s doesn't exist!!\n", filesystemtype[j-1]);
											endmntent ( fstab);

											return -1;
											break;

								case ENOTBLK :		g_print ( "Specialfile n'est pas un fichier special en mode bloc\n");
											endmntent ( fstab);

											return -1;
											break;

								case EBUSY :		/* Le point de montage est dj mont */
											endmntent ( fstab);

											return -1;
											break;

								case EINVAL :		/* Le type de systme de fichier est invalide */
											break;

								case EFAULT :		g_print ( "L'un des arguments pointe en-dehors de l'espace d'adressage accessible.\n");
											endmntent ( fstab);

											return -1;
											break;

								case ENOMEM :		g_print ( "Pas assez de mmoire pour le noyau.\n");
											endmntent ( fstab);

											return -1;
											break;

								case ENAMETOOLONG :	g_print ( "Un des arguments est trop long.\n");
											endmntent ( fstab);

											return -1;
											break;

								case ENOENT :		g_print ( "Un rpertoire des arguments n'existe pas ou est un lien symbolique pointant nulle part.\n");
											endmntent ( fstab);

											return -1;
											break;

								case ENOTDIR :		g_print ( "Le point de montage, ou l'un des lments du chemin d'accs au priphrique n'est pas un rpertoire.\n");
											endmntent ( fstab);

											return -1;
											break;

								case EACCES :		g_print ( "Un lment de chemin d'accs n'est pas consultable,\n");
											endmntent ( fstab);

											return -1;
											break;

								case ENXIO :		g_print ( "Le nombre majeur du priphrique specialfile est invalide.\n");
											endmntent ( fstab);

											return -1;
											break;

								case EMFILE :		g_print ( "Table de montage pleine.\n");
											endmntent ( fstab);

											return -1;
											break;

								default :		/* Problme lors du montage */
											g_print ( "Aie!!\n");
											endmntent ( fstab);

											return -1;
											break;
							}
						}
					}

					g_print ( "Le filesystemtype de ce point de montage n'est pas reconnu.\n");
					g_print ( "Cela est surement du par la prsence du paramtre \"auto\" dans le fichier /etc/fstab.\n");
					g_print ( "Pour pouvoir utiliser ce point de montage avec l'automount dans GWhere veuillez\n");
					g_print ( "prciser le filesystemtype dans le fichier /etc/fstab\n");
				}
				else
				{
					/* Si le filesystemtype est reconnu */
					if ( mount ( pm->mnt_fsname, pm->mnt_dir, pm->mnt_type, 0xc0ed0000|MS_RDONLY, "") == 0 )
					{
						endmntent ( fstab);

						return 0;
					}
					else
					{
						/* Dans le cas o le montage  chou */
						switch ( errno)
						{
							case EPERM :		g_print ( "L'utilisateur n'est pas le  Super-User\n");
										break;

							case ENODEV :		g_print ( "Le fylesystemtype n'est pas configur ds le noyau\n");
										break;

							case ENOTBLK :		g_print ( "Specialfile n'est pas un fichier special en mode bloc\n");
										break;

							case EBUSY :		g_print ( "Specialfile est dj mont");
										endmntent ( fstab);
										return 1;
										break;

							case EINVAL :		g_print ( "Specialfile a un superbloc invalide\n");
										break;

							case EFAULT :		g_print ( "L'un des arguments pointe en-dehors de l'espace d'adressage accessible.\n");
										break;

							case ENOMEM :		g_print ( "Pas assez de mmoire pour le noyau.\n");
										break;

							case ENAMETOOLONG :	g_print ( "Un des arguments est trop long.\n");
										break;

							case ENOENT :		g_print ( "Un rpertoire des arguments n'existe pas ou est un lien symbolique pointant nulle part.\n");
										break;

							case ENOTDIR :		g_print ( "Le point de montage, ou l'un des lments du chemin d'accs au priphrique n'est pas un rpertoire.\n");
										break;

							case EACCES :		g_print ( "Un lment de chemin d'accs n'est pas consultable,\n");
										break;

							case ENXIO :		g_print ( "Le nombre majeur du priphrique specialfile est invalide.\n");
										break;

							case EMFILE :		g_print ( "Table de montage pleine.\n");
										break;

							default :		g_print ( "Erreur non rfrence!!\n");
										break;
						}
					}

					/* Si le montage a chou */
					g_print ( "Aie!! (bis)\n");
					endmntent ( fstab);

					return -1;
				}
			}

			/* Si le point de montage n'est pas automatique */
			/* La fonction ne cherche pas  monter le point de montage */
			endmntent ( fstab);

			return 1;
		}
	}

	/* Dans le cas (impossible) o l'on ne trouve pas le point de montage dans /etc/fstab */
	g_print ( "Aie!! (ter)\n");
	endmntent ( fstab);
#endif

	return -1;
}


gboolean Umount_disk ( gchar *disk)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	FILE *fstab;
	struct mntent *pm;

	fstab = (FILE*)malloc(sizeof(fstab));
	pm = (struct mntent*)malloc(sizeof(struct mntent));

	fstab = setmntent ( "/etc/fstab", "r");

	while ( (pm = getmntent ( fstab)) != NULL)
	{
		if ( strcmp ( pm->mnt_dir, disk) == 0 )
		{
			if ( umount ( pm->mnt_fsname/*, pm->mnt_dir, pm->mnt_type, 0xc0ed0000|MS_RDONLY, ""*/) == 0 )
			{
				endmntent ( fstab);

				return TRUE;
			}
			else
			{
				endmntent ( fstab);

				return FALSE;
			}
		}
	}

	endmntent ( fstab);
#endif

	return FALSE;
}


gchar * type_de_montage ( gchar *m)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Renvoie le type de partition */
	FILE *mtab, *fstab;
	struct mntent *partition = NULL;
	gchar * msg;
	gchar **options = NULL, **fs_type = NULL, **fs_device = NULL;
	gint i;

	mtab = (FILE*)malloc ( sizeof ( mtab));
	partition = (struct mntent*)malloc ( sizeof ( struct mntent));

	/* Ouvre le fichier /etc/mtab */
	mtab = setmntent ( "/etc/mtab", "r");

	while ( ( partition = getmntent ( mtab)) != NULL )
	{
		if ( strcmp ( partition->mnt_dir, m) == 0 )
		{
			if ( strncmp ( "supermount", partition->mnt_type, 10) == 0 )
			{
				endmntent ( mtab);

				/* Rcupre les options de montage et recherche l'option auto */
				options = g_strsplit ( partition->mnt_opts, ",", 0);

				for ( i = 0; options[i] != NULL; i++)
				{
					if ( strncmp ( options[i], "fs=", 3) == 0 )
					{
						fs_type = g_strsplit ( options[i], "=", 2);
					}

					if ( strncmp ( options[i], "dev=", 4) == 0 )
					{
						fs_device = g_strsplit ( options[i], "=", 2);
					}
				}

				msg = g_strdup_printf ( "%s [ %s ]", fs_device[1], fs_type[1]);
				g_strfreev ( options);
				g_strfreev ( fs_type);
				g_strfreev ( fs_device);

				return msg;
			}
			else { if ( strncmp ( "/dev/hd", partition->mnt_fsname, 7) == 0 )
			{
				endmntent ( mtab);
				msg = g_strdup_printf ( "HD [ %s ]", partition->mnt_type);

				return msg;
			}
			else { if ( strncmp ( "/dev/cdrom", partition->mnt_fsname, 10) == 0 )
			{
				endmntent ( mtab);
				msg = g_strdup_printf ( "CD-ROM [ %s ]", partition->mnt_type);

				return msg;
			}
			else { if ( strncmp ( "/dev/scd", partition->mnt_fsname, 8) == 0 )
			{
				endmntent ( mtab);
				msg = g_strdup_printf ( "CD-ROM [ %s ]", partition->mnt_type);

				return msg;
			}
			else { if ( strncmp ( "/dev/fd", partition->mnt_fsname, 7) == 0 )
			{
				endmntent ( mtab);
				msg = g_strdup_printf ( "Floppy [ %s ]", partition->mnt_type);

				return msg;
			}
			else
			{
				msg = g_strdup ( partition->mnt_type);
				endmntent ( mtab);

				return ( msg);
			}}}}}
		}
	}

	if ( partition == NULL )
	{
		/* Si le disk n'est pas mont */
		endmntent ( mtab);
		fstab = (FILE*)malloc ( sizeof ( fstab));

		/* Ouvre le fichier /etc/fstab */
		fstab = setmntent ( "/etc/fstab", "r");

		while ( ( partition = getmntent ( fstab)) != NULL )
		{
			if ( strcmp ( partition->mnt_dir, m) == 0 )
			{
				if ( strncmp ( "supermount", partition->mnt_type, 10) == 0 )
				{
					endmntent ( mtab);

					/* Rcupre les options de montage et recherche l'option auto */
					options = g_strsplit ( partition->mnt_opts, ",", 0);

					for ( i = 0; options[i] != NULL; i++)
					{
						if ( strncmp ( options[i], "fs=", 3) == 0 )
						{
							fs_type = g_strsplit ( options[i], "=", 2);
						}

						if ( strncmp ( options[i], "dev=", 4) == 0 )
						{
							fs_device = g_strsplit ( options[i], "=", 2);
						}
					}

					msg = g_strdup_printf ( "%s [ %s ]", fs_device[1], fs_type[1]);
					g_strfreev ( options);
					g_strfreev ( fs_type);
					g_strfreev ( fs_device);

					return msg;
				}
				else { if ( strncmp ( "/dev/hd", partition->mnt_fsname, 7) == 0 )
				{
					endmntent ( fstab);
					msg = g_strdup_printf ( "HD [ %s ]", partition->mnt_type);

					return msg;
				}
				else { if ( strncmp ( "/dev/cdrom", partition->mnt_fsname, 10) == 0 )
				{
					endmntent ( fstab);
					msg = g_strdup_printf ( "CD-ROM [ %s ]", partition->mnt_type);

					return msg;
				}
				else { if ( strncmp ( "/dev/scd", partition->mnt_fsname, 8) == 0 )
				{
					endmntent ( fstab);
					msg = g_strdup_printf ( "CD-ROM [ %s ]", partition->mnt_type);

					return msg;
				}
				else { if ( strncmp ( "/dev/fd", partition->mnt_fsname, 7) == 0 )
				{
					endmntent ( fstab);
					msg = g_strdup_printf ( "Floppy [ %s ]", partition->mnt_type);

					return msg;
				}
				else { if ( strncmp ( "/dev/zip", partition->mnt_fsname, 8) == 0 )
				{
					endmntent ( fstab);
					msg = g_strdup_printf ( "Zip [ %s ]", partition->mnt_type);

					return msg;
				}
				else
				{
					msg = g_strdup ( partition->mnt_type);
					endmntent ( fstab);

					return ( msg);
				}}}}}}
			}
		}

		endmntent ( fstab);
	}

	endmntent ( mtab);
#endif

	return ( NULL);
}


gchar * nom_de_montage ( gchar *m)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Renvoie le nom du priphrique */
	FILE *tab;
	struct mntent *partition;
	gchar **fs_device;
	gchar **options;
	int i = 0;


	tab = (FILE*)malloc ( sizeof ( tab));
	partition = (struct mntent*)malloc ( sizeof ( struct mntent));

	/* Ouvre le fichier /etc/fstab */
	/* Ouvre le fichier /etc/mtab car l'info ne doit tre affiche que si le priphrique est mont */
//	tab = setmntent ( "/etc/fstab", "r");
	tab = setmntent( "/etc/mtab", "r");

	while ( (partition = getmntent ( tab)) != NULL)
	{
		if ( strcmp ( partition->mnt_dir,m) == 0 )
		{
			if ( strncmp ( "supermount", partition->mnt_type, 10) == 0 )
			{
				/* Rcupre les options de montage et recherche l'option auto */
				options = g_strsplit ( partition->mnt_opts, ",", 0);

				for ( i = 0; options[i] != NULL; i++)
				{
					if ( strncmp ( options[i], "dev=", 4) == 0 )
					{
						fs_device = g_strsplit ( options[i], "=", 2);
						endmntent ( tab);
						g_strfreev ( options);

						return fs_device[1];
					}
				}
			}

			endmntent ( tab);

			return ( partition->mnt_fsname);
		}
	}

	endmntent ( tab);
#endif

	return(NULL);
}


gboolean est_monte ( gchar *m)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Indique si le priphrique est mont */
	FILE *tab;
	struct mntent *partition;


	tab = (FILE*)malloc ( sizeof ( tab));
	partition = (struct mntent*)malloc ( sizeof ( struct mntent));

	/* Ouvre le fichier /etc/mtab (fichier o se trouve toutes les partitions montes) */
	tab = setmntent ( "/etc/mtab", "r");

	while ( (partition = getmntent ( tab)) != NULL)
	{
		if ( strcmp ( partition->mnt_dir, m) == 0 )
		{
			/* Si le disk se trouve dans /etc/mtab alors il est mont */
			endmntent ( tab);

			return TRUE;
		}
	}

	endmntent ( tab);
#endif


	return FALSE;
}


gboolean est_montable ( gchar *m)
{
#if defined ( OS_LINUX) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Indique si le priphrique est mont */
	FILE *tab;
	struct mntent *partition;
	gchar **options = NULL;
	gint i;


	tab = (FILE*)malloc ( sizeof ( tab));
	partition = (struct mntent*)malloc ( sizeof ( struct mntent));

	/* Ouvre le fichier /etc/mtab (fichier o se trouve toutes les partitions montes) */
	tab = setmntent ( "/etc/fstab", "r");

	while ( (partition = getmntent ( tab)) != NULL)
	{
		if ( strcmp ( partition->mnt_dir, m) == 0 )
		{
			/* Si le disk se trouve dans /etc/mtab alors il est mont */
			endmntent ( tab);

			/* Rcupre les options de montage et recherche l'option auto */
			options = g_strsplit ( partition->mnt_opts, ",", 0);

			for ( i = 0; options[i] != NULL; i++)
			{
				if ( strncmp ( options[i], "auto", 4) == 0 )
				{
					g_strfreev ( options);

					return TRUE;
				}
			}

			g_strfreev ( options);

			return FALSE;
		}
	}

	endmntent ( tab);
#endif

	return FALSE;
}


long double capacite_du_peripherique ( gchar *m)
{
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Renvoie la capacit totale du priphrique */
	struct statfs sts;


	/* Rcupre les droits sur le priphrique */
	if ( statfs ( m, &sts) == -1 )
	{
		perror ( "statfs");

		return -1;
	}

	return sts.f_bsize*sts.f_blocks;
#endif
	return -1;
}


long double espace_libre_du_peripherique ( gchar *m)
{
#if defined ( OS_LINUX) || defined ( OS_CYGWIN) || defined ( OS_FREEBSD) || defined ( OS_OPENBSD) || defined ( OS_NETBSD)
	/* Renvoie l'espace livre du priphrique */
	struct statfs sts;


	/* Rcupre les droits sur le priphrique */
	if ( statfs ( m, &sts) == -1 )
	{
		perror ( "statfs");

		return -1;
	}

	return sts.f_bsize*sts.f_bfree;
#endif

	return -1;
}
