/*  GWhere
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include "../support.h"
#include "About.h"


/* XPM : GWhere logo */
static char *GWhere_logo_xpm[] = {
"70 46 3 1",
". c #669999",
"# c #ff9900",
"  c None",
"                                   ...                                ",
"                         ......................                       ",
"                      .............................                   ",
"                  .................   .................               ",
"                ..........                     ..........             ",
"              ........                             ........           ",
"            .......                                   .......         ",
"           .....                                        ......        ",
"         ......                                           ......      ",
"        .....                                               .....     ",
"       ....                                                  .....    ",
"      ....                                                     ....   ",
"     ....                                                       ....  ",
"    ....                                                         .... ",
"   ....                                                           ....",
"   ...                                                             ...",
"  ...                                                                 ",
"  ...                                                                 ",
" ...                                                                  ",
" ...                  ##   ###   ## ##     #  ####### #######  #######",
" ...                  ##   ###   ## ##     # ##       #     ## #      ",
"...                    #   ###   #  ##     # ##       #     ## #      ",
"...                    ##  # #  ##  ##     # ##       #     ## #      ",
"...                    ## ## ## ##  ######## #######  # #####  #######",
"...                    ## ## ## ##  ######## ##       ######   #      ",
"...                     # ## ## #   ##     # ##       # ##     #      ",
" ..                     ####  ###   ##     # ##       #  ##    #      ",
" ...                    ###   ###   ##     # ##       #   ##   ##     ",
" ...                     ##   ###   ##     #  ######  #    ##   ######",
"  ...                                                                 ",
"  ...                   ..............................................",
"   ...                  ..............................................",
"   ....                                                           ....",
"    ....                                                         .... ",
"     ....                                                       ....  ",
"      ....                                                     ....   ",
"       ....                                                  .....    ",
"        .....                                               .....     ",
"         ......                                           ......      ",
"           ......                                       ......        ",
"            .......                                   ......          ",
"              ........                             .......            ",
"                ..........                     .........              ",
"                  .................   .................               ",
"                      .............................                   ",
"                         ......................                       "};


GtkWidget * about_create ( GtkWindow *window)
{
	/* This window must be single, this property shouldn't be changed */
	GdkColor transparent = { 0 };
	static GtkWidget *w = NULL;
	GtkWidget *vbox_about;
	GtkWidget *label;
	GtkWidget *event_box;
	GtkWidget *logo;
	GdkPixmap *pix_gw_logo;
	GdkBitmap *msk_gw_logo;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* GWhere logo loading */
	pix_gw_logo = gdk_pixmap_create_from_xpm_d ( GTK_WIDGET ( window)->window, &msk_gw_logo, &transparent, GWhere_logo_xpm);


	if ( !w )
	{
		w = gtk_window_new ( GTK_WINDOW_TOPLEVEL);

		gtk_window_set_modal ( GTK_WINDOW ( w), TRUE);
		gtk_window_set_transient_for ( GTK_WINDOW ( w), window);
		gtk_window_set_position ( GTK_WINDOW ( w), GTK_WIN_POS_CENTER);
		gtk_window_set_title ( GTK_WINDOW ( w), _( "About"));

		gtk_signal_connect ( GTK_OBJECT ( w), "destroy", GTK_SIGNAL_FUNC (gtk_widget_destroyed), &w);

		gtk_container_set_border_width ( GTK_CONTAINER ( w), 5);
		gtk_widget_set_usize ( w, 330, 300);

		event_box = gtk_event_box_new ( );
		gtk_container_add ( GTK_CONTAINER ( w), event_box);
		gtk_widget_show ( event_box);

		vbox_about = gtk_vbox_new ( FALSE, 0);
		gtk_container_add ( GTK_CONTAINER ( event_box), vbox_about);

		logo = gtk_pixmap_new ( pix_gw_logo, msk_gw_logo);
		gtk_container_add ( GTK_CONTAINER ( vbox_about), logo);

		label = gtk_label_new ( g_strconcat ( PACKAGE, "-", VERSION, _( "\nRemovable media catalog managment\nBy Sbastien LECACHEUR\n\nzero@gwhere.org\nhttp://www.gwhere.org\n"),
			_( "GWhere allows to manage a database of yours CDs and others removable media (hard disks, floppy drive, Zip drive, CD-ROM, etc...). With GWhere it's easy to browse yours CDs or to make quick search without need to insert yours CDs in the drive at each once."),
			NULL));

		gtk_label_set_justify ( GTK_LABEL ( label), GTK_JUSTIFY_FILL);
		gtk_label_set_line_wrap ( GTK_LABEL ( label), TRUE);
		gtk_container_add ( GTK_CONTAINER ( vbox_about), label);
		gtk_widget_set_usize ( label, 300, 240);

		gtk_widget_set_events ( event_box, GDK_BUTTON_PRESS_MASK);
		gtk_signal_connect_object ( GTK_OBJECT ( event_box), "button_press_event", GTK_SIGNAL_FUNC ( gtk_widget_destroy), GTK_OBJECT ( w));

		gtk_widget_realize ( event_box);
		gdk_window_set_cursor ( event_box->window, gdk_cursor_new ( GDK_HAND2));
	}

	if ( !GTK_WIDGET_VISIBLE ( w) )
	{
#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : show the window\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		gtk_widget_show_all ( w);
	}
	else
	{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : destroy the window\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		gtk_widget_destroy ( w);
	}

	return w;
}
