/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include <string.h> /* strcmp strlen*/

#include "MsgBox.h"
#include "NoteBookManagmentCallBack.h"


#include "../support.h"
#include "CategoriesEditBox.h"


/*! @define	GW_REF_CATEGORIES_EDIT_BOX_MAIN_WINDOW	The main window */
#define GW_REF_CATEGORIES_EDIT_BOX_MAIN_WINDOW "gw_ref_categories_edit_box_main_window"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATALOG_INFO	The catalog file info data */
#define GW_REF_CATEGORIES_EDIT_BOX_CATALOG_INFO "gw_ref_categories_edit_box_catalog_info"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST_INFO	The categories list data */
#define GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST_INFO "gw_ref_categories_edit_box_categories_list_info"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY	The category name */
#define GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY "gw_ref_categories_edit_box_category_name_entry"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY_TOOLTIPS	The category name tooltips */
#define GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY_TOOLTIPS "gw_ref_categories_edit_box_category_name_entry_tooltips"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT	The category description */
#define GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT "gw_ref_categories_edit_box_category_description_text"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT_TOOLTIPS	The category description tooltips */
#define GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT_TOOLTIPS "gw_ref_categories_edit_box_category_description_text_tooltips"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_UPDATE_BUTTON_TOOLTIPS	The update button tooltips */
#define GW_REF_CATEGORIES_EDIT_BOX_UPDATE_BUTTON_TOOLTIPS "gw_ref_categories_edit_box_update_button_tooltips"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON	The add/update button */
#define GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON "gw_ref_categories_edit_box_add_update_button"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON_TOOLTIPS	The add/update button tooltips */
#define GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON_TOOLTIPS "gw_ref_categories_edit_box_add_update_button_tooltips"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_REMOVE_BUTTON_TOOLTIPS	The remove button tooltips */
#define GW_REF_CATEGORIES_EDIT_BOX_REMOVE_BUTTON_TOOLTIPS "gw_ref_categories_edit_box_remove_button_tooltips"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST	The list of categories */
#define GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST "gw_ref_categories_edit_box_categories_list"
/*! @define	GW_REF_CATEGORIES_EDIT_BOX_CLOSE_BUTTON_TOOLTIPS	The close button tooltipd */
#define GW_REF_CATEGORIES_EDIT_BOX_CLOSE_BUTTON_TOOLTIPS "gw_ref_categories_edit_box_close_button_toltips"


GtkWindow * categories_edit_box_create ( GtkWindow *window, struct catalog_file_info *catalog_info)
{
	/* This window must be single, this property may be changed */
	static GtkWidget *w = NULL;
	GtkWidget *vb, *hb, *scr, *list, *bt, *hsp, *ent, *lbl, *frm, *txt;
	guint bt_key;
	GtkAccelGroup *accel;
	GtkTooltips *tips;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Init an accel group for shortcuts */
	accel = gtk_accel_group_new ( );

	/* Init tooltips */
	tips = gtk_tooltips_new ( );

	if ( !w )
	{
		w = gtk_window_new ( GTK_WINDOW_TOPLEVEL);

		gtk_window_set_modal ( GTK_WINDOW ( w), TRUE);
		gtk_window_set_transient_for ( GTK_WINDOW ( w), window);
		gtk_window_set_position ( GTK_WINDOW ( w), GTK_WIN_POS_CENTER);
		gtk_window_set_policy ( GTK_WINDOW ( w), FALSE, FALSE, FALSE);
		gtk_window_set_title ( GTK_WINDOW ( w), _( "Edit categories"));
		gtk_container_set_border_width ( GTK_CONTAINER ( w), 5);

		gtk_object_set_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_MAIN_WINDOW, window);
		gtk_object_set_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATALOG_INFO, catalog_info);
		gtk_object_set_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST_INFO, catalog_file_info_get_categories ( catalog_info));

		gtk_signal_connect ( GTK_OBJECT ( w), "destroy", GTK_SIGNAL_FUNC (gtk_widget_destroyed), &w);

		/* Vertical box */
		vb = gtk_vbox_new ( FALSE, 10);
//		gtk_widget_ref ( vb);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "VbCategories", vb, (GtkDestroyNotify) gtk_widget_unref);
		gtk_container_add ( GTK_CONTAINER ( w), vb);

		/* Frame category description */
		frm = gtk_frame_new ( _( "Categories list : "));
//		gtk_widget_ref ( frm);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "FrmCategoriesList", frm, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( vb), frm, TRUE, TRUE, 0);

		/* 1st horizontal box*/
		hb = gtk_hbox_new ( TRUE, 0);
//		gtk_widget_ref ( hb);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "HbCategories1", hb,(GtkDestroyNotify) gtk_widget_unref);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb), 5);
		gtk_container_add ( GTK_CONTAINER ( frm), hb);

		/* Scrolled panel */
		scr = gtk_scrolled_window_new ( NULL, NULL);
//		gtk_widget_ref ( scr);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "ScrWinCategories", scr,(GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb), scr, TRUE, TRUE, 0);

		/* Categories list */
		list = gtk_clist_new ( 2);
		gtk_widget_set_usize ( list, 100, 100);
		gtk_widget_ref ( list);
		gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST, list,(GtkDestroyNotify) gtk_widget_unref);
//		gtk_signal_connect ( GTK_OBJECT ( list), "click_column", GTK_SIGNAL_FUNC ( categories_edit_box_list_click_column), w);
		gtk_signal_connect ( GTK_OBJECT ( list), "select_row", GTK_SIGNAL_FUNC ( categories_edit_box_list_select_row), w);
		gtk_container_add ( GTK_CONTAINER ( scr), list);
		gtk_clist_set_column_width ( GTK_CLIST ( list), 0, 80);
		gtk_clist_set_column_width ( GTK_CLIST ( list), 1, 20);
		gtk_clist_column_titles_show ( GTK_CLIST ( list));

		/* 1st column label */
		lbl = gtk_label_new ( _( "Name"));
//		gtk_widget_ref ( lbl);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "LblCategoriesName", lbl,(GtkDestroyNotify) gtk_widget_unref);
		gtk_clist_set_column_widget ( GTK_CLIST ( list), 0, lbl);

		/* 2nd column label */
		lbl = gtk_label_new ( _( "Description"));
//		gtk_widget_ref ( lbl);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "LblCategoriesDescription", lbl,(GtkDestroyNotify) gtk_widget_unref);
		gtk_clist_set_column_widget ( GTK_CLIST ( list), 1, lbl);

		/* 2nd horizontal box */
		hb = gtk_hbox_new ( TRUE, 0);
//		gtk_widget_ref ( hb);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "HbCategories2", hb,(GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( vb), hb, TRUE, TRUE, 0);

		/* Update button */
		bt = gtk_button_new_with_label ( "");
		bt_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "_Update"));
		gtk_widget_add_accelerator ( bt, "clicked", accel, bt_key, GDK_MOD1_MASK, 0);
		gtk_signal_connect ( GTK_OBJECT ( bt), "clicked", GTK_SIGNAL_FUNC ( categories_edit_box_update_click), GTK_WINDOW ( w));
		gtk_object_set_user_data ( GTK_OBJECT ( bt), w);
		gtk_box_pack_start ( GTK_BOX ( hb), bt, FALSE, TRUE, 5);
		gtk_tooltips_set_tip ( tips, bt, _( "Allows to update category properties of the selected category. Clicks on Add / Update button to save updates."), GW_REF_CATEGORIES_EDIT_BOX_UPDATE_BUTTON_TOOLTIPS);

		/* Remove button */
		bt = gtk_button_new_with_label ( "");
		bt_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "_Remove"));
		gtk_widget_add_accelerator ( bt, "clicked", accel, bt_key, GDK_MOD1_MASK, 0);
		gtk_signal_connect ( GTK_OBJECT ( bt), "clicked", GTK_SIGNAL_FUNC ( categories_edit_box_remove_click), GTK_WINDOW ( w));
		gtk_object_set_user_data ( GTK_OBJECT ( bt), w);
		gtk_box_pack_start ( GTK_BOX ( hb), bt, FALSE, TRUE, 5);
		gtk_tooltips_set_tip ( tips, bt, _( "Remove the selected category from categories list. This categories may removed only if there's not any item which uses this category."), GW_REF_CATEGORIES_EDIT_BOX_REMOVE_BUTTON_TOOLTIPS);

		/* 1st horizontal separator */
		hsp = gtk_hseparator_new ( );
//		gtk_widget_ref ( hsp);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "Hsp1", hsp, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( vb), hsp, TRUE, TRUE, 0);

		/* 3rd horizontal box */
		hb = gtk_hbox_new ( FALSE, 0);
//		gtk_widget_ref ( hb);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "HbCategories3", hb,(GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( vb), hb, TRUE, TRUE, 0);

		/* Label for category name */
		lbl = gtk_label_new ( _( "Category name : "));
//		gtk_widget_ref ( lbl);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "LblCategoryName", lbl, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb), lbl, FALSE, FALSE, 5);

		/* Field text for category name */
		ent = gtk_entry_new ();
		gtk_widget_ref ( ent);
		gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY, ent, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb), ent, TRUE, TRUE, 5);
		gtk_tooltips_set_tip ( tips, ent, _( "Enter the name of the category."), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY_TOOLTIPS);

		/* Frame category description */
		frm = gtk_frame_new ( _( "Description : "));
//		gtk_widget_ref ( frm);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "FrmCategoryDescription", frm, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( vb), frm, TRUE, TRUE, 0);

		/* 4th horizontal box */
		hb = gtk_hbox_new ( TRUE, 0);
//		gtk_widget_ref ( hb);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "HbCategories4", hb,(GtkDestroyNotify) gtk_widget_unref);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb), 5);
		gtk_container_add ( GTK_CONTAINER ( frm), hb);

		/* Text scrollbar */
		scr = gtk_scrolled_window_new ( NULL, NULL);
		gtk_box_pack_start ( GTK_BOX ( hb), scr, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW ( scr), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

		/* Text area for category description */
		txt = gtk_text_new ( NULL, NULL);
		gtk_text_set_editable ( GTK_TEXT ( txt), TRUE);
		gtk_widget_ref ( txt);
		gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT, txt, (GtkDestroyNotify) gtk_widget_unref);
		gtk_container_add ( GTK_CONTAINER ( scr), txt);
		gtk_tooltips_set_tip ( tips, txt, _( "Enter the description of the category."), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT_TOOLTIPS);

		/* 5th horizontal box */
		hb = gtk_hbox_new ( TRUE, 0);
//		gtk_widget_ref ( hb);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "HbCategories5", hb,(GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( vb), hb, TRUE, TRUE, 0);

		/* Add/Update button */
		bt = gtk_button_new_with_label ( "");
//		gtk_label_set_text ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "Add"));
		gtk_widget_ref ( bt);
		gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON, bt,(GtkDestroyNotify) gtk_widget_unref);
		bt_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "Add / Update"));
		gtk_widget_add_accelerator ( bt, "clicked", accel, bt_key, GDK_MOD1_MASK, 0);
		gtk_signal_connect ( GTK_OBJECT ( bt), "clicked", GTK_SIGNAL_FUNC ( categories_edit_box_add_update_click), w);
		gtk_object_set_user_data ( GTK_OBJECT ( bt), w);
		gtk_box_pack_start ( GTK_BOX ( hb), bt, FALSE, TRUE, 5);
		gtk_tooltips_set_tip ( tips, bt, _( "Save properties changes of category."), GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON_TOOLTIPS);

		/* Close button */
		bt = gtk_button_new_with_label ( "");
//		gtk_widget_ref ( bt);
//		gtk_object_set_data_full ( GTK_OBJECT ( w), "BtClose", bt,(GtkDestroyNotify) gtk_widget_unref);
		bt_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "Close"));
		gtk_widget_add_accelerator ( bt, "clicked", accel, bt_key, GDK_MOD1_MASK, 0);
		gtk_signal_connect_object ( GTK_OBJECT ( bt), "clicked", GTK_SIGNAL_FUNC ( gtk_widget_destroy), GTK_OBJECT ( w));
		gtk_object_set_user_data ( GTK_OBJECT ( bt), w);
		gtk_box_pack_start ( GTK_BOX ( hb), bt, FALSE, TRUE, 5);
		gtk_tooltips_set_tip ( tips, bt, _( "Close the categories properties edit window."), GW_REF_CATEGORIES_EDIT_BOX_CLOSE_BUTTON_TOOLTIPS);

		gtk_window_add_accel_group ( GTK_WINDOW ( w), accel);
		}

	if ( !GTK_WIDGET_VISIBLE ( w) )
	{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : show the window\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		gtk_widget_show_all ( w);
	}
	else
	{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : destroy the window\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		gtk_widget_destroy ( w);
	}

	categories_edit_box_load_categories_list ( GTK_WINDOW ( w));

	return GTK_WINDOW ( w);
}


gint categories_edit_box_load_categories_list ( GtkWindow *w)
{
	GList *categories = categories_edit_box_get_categories_list ( w);


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( categories != NULL )
	{
		categories = g_list_first ( categories);

		if ( g_list_length ( categories) > 1 )
		{
			categories = g_list_next ( categories);

			while ( categories != g_list_last ( categories))
			{
				categories_edit_box_add_category ( w, categories->data);
				categories = g_list_next ( categories);
			}

			categories_edit_box_add_category ( w, categories->data);
		}
	}

	categories_edit_box_set_add_state ( w);

	return 0;
}


gint categories_edit_box_update_click ( GtkWidget *bt, GtkWindow *w)
{
	struct category *category = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	category = categories_edit_box_get_category ( w);

	if ( category == NULL )
	{
		return -1;
	}

	categories_edit_box_set_category_name ( w, category_get_name ( category));
	categories_edit_box_set_category_description ( w, category_get_description ( category));

	catalog_file_info_set_ismodified ( categories_edit_box_get_catalog_info ( w), TRUE);
	categories_edit_box_set_update_state ( w);

	return 0;
}


gint categories_edit_box_remove_click ( GtkWidget *bt, GtkWindow *w)
{
	struct category *category = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	category = categories_edit_box_get_category ( w);

	if ( category == NULL )
	{
		return -1;
	}

	if ( !category_can_removed ( category) )
	{
		msg_box_create ( w, _( "Remove category"), _( "Can't remove this category. It's used by an item."));

		return -1;
	}

	if ( categories_edit_box_remove_category ( w) != -1 )
	{
		catalog_file_info_set_ismodified ( categories_edit_box_get_catalog_info ( w), TRUE);
	}

	categories_edit_box_set_category_name ( w, "");
	categories_edit_box_set_category_description ( w, "");

	categories_edit_box_set_add_state ( w);

	return 0;
}


gint categories_edit_box_add_update_click ( GtkWidget *bt, GtkWindow *w)
{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( categories_edit_box_get_isadd_state ( w) )
	{
		if ( categories_edit_box_add_category ( w, NULL) != -1 )
		{
			catalog_file_info_set_ismodified ( categories_edit_box_get_catalog_info ( w), TRUE);
		}

		categories_edit_box_set_category_name ( w, "");
		categories_edit_box_set_category_description ( w, "");

		categories_edit_box_set_add_state ( w);
		notebook_managment_refresh_categories ( categories_edit_box_get_main_window ( w));

		return 0;
	}
	else { if ( categories_edit_box_get_isupdate_state ( w) )
	{
		categories_edit_box_update_category ( w);
		catalog_file_info_set_ismodified ( categories_edit_box_get_catalog_info ( w), TRUE);
		notebook_managment_refresh_categories ( categories_edit_box_get_main_window ( w));

		return 0;
	}
	else
	{
		/* If they're no state */
		return -1;
	}}

	return -1;
}


gboolean categories_edit_box_list_select_row ( GtkCList *list, gint line, gint column, GdkEvent *event, GtkWindow *w)
{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	categories_edit_box_set_category_name ( w, "");
	categories_edit_box_set_category_description ( w, "");

	categories_edit_box_set_add_state ( w);

	return TRUE;
}


gint categories_edit_box_set_category_name ( GtkWindow *w, gchar *category_name)
{
	GtkEntry *entry = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( entry == NULL )
	{
		return -1;
	}

	entry = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY);


	gtk_entry_set_text ( entry, category_name);

	return 0;
}


gint categories_edit_box_set_category_description ( GtkWindow *w, gchar *category_description)
{
	GtkText *txt = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	txt = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT);

	if ( txt == NULL )
	{
		return -1;
	}

	gtk_text_freeze ( txt);
	gtk_text_set_point ( txt, 0);
	gtk_text_forward_delete ( txt, gtk_text_get_length ( txt));
	gtk_text_insert ( txt, NULL, NULL, NULL, category_description, strlen ( category_description));
	gtk_text_thaw ( txt);

	return 0;
}


gint categories_edit_box_set_add_state ( GtkWindow *w)
{
	GtkButton *bt = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	bt = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON);

	if ( bt == NULL )
	{
		return -1;
	}

	gtk_label_set_text ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "Add"));

	return 0;
}


gint categories_edit_box_set_update_state ( GtkWindow *w)
{
	GtkButton *bt = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	bt = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON);

	if ( bt == NULL )
	{
		return -1;
	}

	gtk_label_set_text ( GTK_LABEL ( GTK_BIN ( bt)->child), _( "Update"));

	return 0;
}


gint categories_edit_box_add_category ( GtkWindow *w, struct category *p)
{
	struct category *category = NULL;
	struct catalog_file_info *catalog_info = NULL;
	GtkCList *list = NULL;
	gchar *info[2];
	guint row;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	list = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST);
	/* Does one have to leave the program if list is NULL? */

	if ( p == NULL )
	{
		category = category_new ();

		if ( strcmp ( categories_edit_box_get_category_name ( w), "") == 0 )
		{
			return -1;
		}

		category_set_name ( category, g_strdup ( categories_edit_box_get_category_name ( w)));
		category_set_description ( category, g_strdup ( categories_edit_box_get_category_description ( w)));

		catalog_info = categories_edit_box_get_catalog_info ( w);
		catalog_file_info_append_category ( catalog_info, category);
//		g_list_append ( categories_edit_box_get_categories_list (w), category);
	}
	else
	{
		category = p;
	}

	info[0] = g_strdup ( category_get_name ( category));
	info[1] = g_strdup ( category_get_description ( category));

	row = gtk_clist_append ( list, info);
	gtk_clist_set_row_data ( list, row, category);

	return 0;
}


gint categories_edit_box_update_category ( GtkWindow *w)
{
	struct category *category = NULL;
	GtkCList *list = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	category = categories_edit_box_get_category ( w);
	list = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST);

	if ( category == NULL )
	{
		return -1;
	}

	if ( list->selection == NULL )
	{
		return -1;
	}

	category_set_name ( category, g_strdup ( categories_edit_box_get_category_name ( w)));
	category_set_description ( category, g_strdup ( categories_edit_box_get_category_description ( w)));

	gtk_clist_set_text ( list, (gint)(list->selection->data), 0, categories_edit_box_get_category_name ( w));
	gtk_clist_set_text ( list, (gint)(list->selection->data), 1, categories_edit_box_get_category_description ( w));

	return 0;
}


gint categories_edit_box_remove_category ( GtkWindow *w)
{
	struct category *category = NULL;
	GtkCList *list = NULL;
//	GList *categories = NULL;
	struct catalog_file_info *catalog_info = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	category = categories_edit_box_get_category ( w);
	list = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST);
//	categories = categories_edit_box_get_categories_list ( w);
	catalog_info = categories_edit_box_get_catalog_info ( w);


	if ( category == NULL )
	{
		return -1;
	}

	if ( list->selection == NULL )
	{
		return -1;
	}

	gtk_clist_remove ( list, (gint)(list->selection->data));
	catalog_file_info_remove_category ( catalog_info, category);
//	g_list_remove ( categories, category);

	return 0;
}


struct catalog_file_info * categories_edit_box_get_catalog_info ( GtkWindow *w)
{
	struct catalog_file_info *catalog_info = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		catalog_info = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATALOG_INFO);
	}

	return catalog_info;
}


GtkWindow * categories_edit_box_get_main_window ( GtkWindow *w)
{
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		window = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_MAIN_WINDOW);
	}

	return window;
}


GList * categories_edit_box_get_categories_list ( GtkWindow *w)
{
	GList *list = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		list = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST_INFO);
	}

	return list;
}


struct category * categories_edit_box_get_category ( GtkWindow *w)
{
	struct category *category = NULL;
	GtkCList *list = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return NULL;
	}

	list = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORIES_LIST);

	if ( list == NULL )
	{
		return NULL;
	}

	if ( list->selection == NULL )
	{
		return NULL;
	}

	category = gtk_clist_get_row_data ( list, (gint)(list->selection->data));

	return category;
}


gchar * categories_edit_box_get_category_name ( GtkWindow *w)
{
	GtkEntry *entry = NULL;
	gchar *category_name = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		entry = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_NAME_ENTRY);

		if ( entry != NULL )
		{
			category_name = gtk_entry_get_text ( entry);
		}
	}

	return category_name;
}


gchar * categories_edit_box_get_category_description ( GtkWindow *w)
{
	GtkText *txt = NULL;
	gchar *category_description = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		txt = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_CATEGORY_DESCRIPTION_TEXT);

		if ( txt != NULL )
		{
			category_description = gtk_editable_get_chars ( GTK_EDITABLE (txt), 0, -1);
		}
	}

	return category_description;
}


gboolean categories_edit_box_get_isadd_state ( GtkWindow *w)
{
	GtkButton *bt = NULL;
	gchar *label;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	bt = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON);

	if ( bt == NULL )
	{
		return FALSE;
	}

	gtk_label_get ( GTK_LABEL ( GTK_BIN ( bt)->child), &label);

	if ( strcmp ( label, _( "Add")) != 0 )
	{
		return FALSE;
	}

	return TRUE;
}


gboolean categories_edit_box_get_isupdate_state ( GtkWindow *w)
{
	GtkButton *bt = NULL;
	gchar *label;


#ifdef GW_DEBUG_GUI_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	bt = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CATEGORIES_EDIT_BOX_ADD_UPDATE_BUTTON);

	if ( bt == NULL )
	{
		return FALSE;
	}

	gtk_label_get ( GTK_LABEL ( GTK_BIN ( bt)->child), &label);

	if ( strcmp ( label, _( "Update")) != 0 )
	{
		return FALSE;
	}

	return TRUE;
}
