/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <string.h> /* strcmp */

#include "../support.h"
#include "ComboBoxCategories.h"


GtkHBox * combo_box_categories_create ( GtkWindow *w, gchar *title, struct catalog_file_info *catalog_info)
{
	GtkWidget *hb, *lbl, *cmb;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Vertical box for categories */
	hb = gtk_hbox_new ( FALSE, 0);
	gtk_widget_ref ( hb);
	gtk_object_set_data_full ( GTK_OBJECT ( w), "HbCategoriesCatalog", hb, (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show ( hb);
	gtk_container_set_border_width ( GTK_CONTAINER ( hb), 2);

	/* Title label for categories */
	if ( title != NULL )
	{
		lbl = gtk_label_new ( title);
		gtk_widget_ref ( lbl);
		gtk_object_set_data_full ( GTK_OBJECT ( hb), "LblCategoriesCatalog", lbl, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb), lbl, FALSE, FALSE, 0);
	}

	/* Combo box for categories */
	cmb = gtk_combo_new ();
	gtk_widget_ref ( cmb);
	gtk_combo_set_use_arrows_always ( GTK_COMBO ( cmb), TRUE);
	gtk_object_set_data_full ( GTK_OBJECT ( hb), "CmbCategoriesCatalog", cmb, (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_editable ( GTK_ENTRY( GTK_COMBO ( cmb)->entry), FALSE);
	gtk_box_pack_start ( GTK_BOX ( hb), cmb, TRUE, TRUE, 0);

	if ( catalog_info != NULL )
	{
		gtk_object_set_data ( GTK_OBJECT ( hb), "CatalogInfo", catalog_info);
	}

	return GTK_HBOX ( hb);
}


gint combo_box_categories_load ( GtkHBox *cbc)
{
	GList *categories = NULL;
	GList *glist = NULL;
	GtkCombo *cmb = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (cmb = combo_box_categories_get_combo ( cbc)) == NULL )
	{
		return -1;
	}

	if ( (glist = combo_box_categories_get_categories ( cbc)) == NULL )
	{
		glist = g_list_append ( glist, "");
		gtk_combo_set_popdown_strings ( cmb, glist);

		return 0;
	}

	glist = g_list_first ( glist);
	while ( glist != g_list_last ( glist))
	{
		categories = g_list_append ( categories, category_get_name ( (struct category*)glist->data));
		glist = g_list_next ( glist);
	}

	categories = g_list_append ( categories, category_get_name ( glist->data));

	if ( categories != NULL )
	{
		gtk_combo_set_popdown_strings ( cmb, categories);
	}
	if ( categories != NULL )
	{
		gtk_entry_set_text ( GTK_ENTRY ( cmb->entry), g_list_first ( categories)->data);
	}
	else
	{
		return -1;
//		 gtk_entry_set_text ( GTK_ENTRY ( cmb->entry), _("none"));
	}

	return 0;
}


gint combo_box_categories_set_catalog_info ( GtkHBox *cbc, struct catalog_file_info *catalog_info)
{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return -1;
	}

	gtk_object_set_data ( GTK_OBJECT ( cbc), "CatalogInfo", catalog_info);

	return 0;
}


gint combo_box_categories_set_selected_category ( GtkHBox *cbc, struct category *category)
{
	GList *categories = NULL;
	GList *selection = NULL;
	GtkCombo *cmb;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return -1;
	}

	if ( category == NULL )
	{
		return -1;
	}

	categories = combo_box_categories_get_categories ( cbc);

	if ( (selection = g_list_find ( categories, category)) == NULL )
	{
		return -1;
	}

	cmb = combo_box_categories_get_combo ( cbc);
	gtk_entry_set_text ( GTK_ENTRY ( cmb->entry), category_get_name ( selection->data));
		
	return 0;
}


gint combo_box_categories_set_selected_category_name ( GtkHBox *cbc, gchar *name)
{
	GList *categories = NULL;
	GList *selection = NULL;
	GtkCombo *cmb;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return -1;
	}

	if ( name == NULL )
	{
		return -1;
	}

	categories = combo_box_categories_get_categories ( cbc);


	if ( (selection = g_list_find_custom ( categories, name, (GCompareFunc)strcmp)) == NULL )
	{
		return -1;
	}

	cmb = combo_box_categories_get_combo ( cbc);
	gtk_entry_set_text ( GTK_ENTRY ( cmb->entry), g_list_first ( selection)->data);
		
	return 0;
}


struct category * combo_box_categories_get_selected_category ( GtkHBox *cbc)
{
	gchar *name;
	struct catalog_file_info *catalog_info = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return NULL;
	}

	catalog_info = combo_box_categories_get_catalog_info ( cbc);
	name = combo_box_categories_get_selected_category_name ( cbc);

	return catalog_file_info_get_category ( catalog_info, name);
}


gchar * combo_box_categories_get_selected_category_name ( GtkHBox *cbc)
{
	GtkCombo *cmb = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return NULL;
	}

	if ( (cmb = gtk_object_get_data ( GTK_OBJECT ( cbc), "CmbCategoriesCatalog")) == NULL)
	{
		return NULL;
	}

	return gtk_entry_get_text ( GTK_ENTRY ( cmb->entry));
}


GtkCombo * combo_box_categories_get_combo ( GtkHBox *cbc)
{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return NULL;
	}

	return GTK_COMBO ( gtk_object_get_data ( GTK_OBJECT ( cbc), "CmbCategoriesCatalog"));
}


struct catalog_file_info * combo_box_categories_get_catalog_info ( GtkHBox *cbc)
{
	struct catalog_file_info *catalog_info = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return NULL;
	}

	catalog_info = gtk_object_get_data ( GTK_OBJECT ( cbc), "CatalogInfo");

	return catalog_info;
}


GList * combo_box_categories_get_categories ( GtkHBox *cbc)
{
	struct catalog_file_info *catalog_info = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( cbc == NULL )
	{
		return NULL;
	}

	catalog_info = combo_box_categories_get_catalog_info ( cbc);

	return catalog_file_info_get_categories ( catalog_info);
}

