/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "MenuAction.h"
#include "MenuActionCallBack.h"


/*! @define	GW_REF_MENU_BAR_ACTION_MENU_HEADER	The action menu header reference */
#define GW_REF_MENU_BAR_ACTION_MENU_HEADER "gw_menu_bar_action_menu_header"
/*! @define	GW_REF_MENU_BAR_ACTION_MENU	The action menu reference */
#define GW_REF_MENU_BAR_ACTION_MENU "gw_menu_bar_action_menu"
/*! @define	GW_REF_MENU_BAR_ACTION_MENU_EDIT_CATEGORIES	The action menu -> edit categories reference */
#define GW_REF_MENU_BAR_ACTION_MENU_EDIT_CATEGORIES "gw_menu_bar_action_menu_edit_categories"


GtkWidget * menu_action_create ( GtkWindow *w, GtkAccelGroup *ag, GtkWidget *parent)
{
	GtkWidget *menu_action_header = NULL;
	GtkWidget *menu_action = NULL;
	GtkWidget *menu_action_edit_categories = NULL;
	GtkAccelGroup *menu_action_ag;
	guint tmp_key;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Menu action header */
	menu_action_header = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_action_header)->child), _( "_Action"));
	gtk_widget_add_accelerator ( menu_action_header, "activate_item", ag, tmp_key, GDK_MOD1_MASK, 0);
	gtk_widget_ref ( menu_action_header);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_ACTION_MENU_HEADER, menu_action_header, (GtkDestroyNotify) gtk_widget_unref);

	menu_action = gtk_menu_new ( );
	gtk_widget_ref ( menu_action);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_ACTION_MENU, menu_action, (GtkDestroyNotify) gtk_widget_unref);
	gtk_menu_item_set_submenu ( GTK_MENU_ITEM ( menu_action_header), menu_action);
	menu_action_ag = gtk_menu_ensure_uline_accel_group ( GTK_MENU ( menu_action));

	/* Menu action -> edit categories */
	menu_action_edit_categories = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_action_edit_categories)->child), _( "Edit _categories"));
	gtk_widget_add_accelerator ( menu_action_edit_categories, "activate_item", menu_action_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_action_edit_categories);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_ACTION_MENU_EDIT_CATEGORIES, menu_action_edit_categories, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_action), menu_action_edit_categories);
	gtk_widget_add_accelerator ( menu_action_edit_categories, "activate", ag, GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_action_edit_categories), "activate", GTK_SIGNAL_FUNC ( menu_action_edit_categories_click), w);

	return menu_action_header;
}
