/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "MenuFileCallBack.h"

#include "GWReferences.h"
#include "../support.h"
#include "MenuFile.h"


/*! @define	GW_REF_MENU_BAR_FILE_MENU_HEADER	The file menu header reference */
#define GW_REF_MENU_BAR_FILE_MENU_HEADER "gw_menu_bar_file_menu_header"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_NEW	The file menu -> new reference */
#define GW_REF_MENU_BAR_FILE_MENU_NEW "gw_menu_bar_file_menu_new"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_OPEN	The file menu -> open reference */
#define GW_REF_MENU_BAR_FILE_MENU_OPEN "gw_menu_bar_file_menu_open"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_SAVE	The file menu -> save reference */
#define GW_REF_MENU_BAR_FILE_MENU_SAVE "gw_menu_bar_file_menu_save"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_SAVEAS	The file menu -> save as reference */
#define GW_REF_MENU_BAR_FILE_MENU_SAVEAS "gw_menu_bar_file_menu_saveas"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_CLOSE	The file menu -> close reference */
#define GW_REF_MENU_BAR_FILE_MENU_CLOSE "gw_menu_bar_file_menu_close"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_SEPARATOR	The file menu -> separator reference */
#define GW_REF_MENU_BAR_FILE_MENU_SEPARATOR "gw_menu_bar_file_menu_separator"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_PROPERTIES	The file menu -> properties reference */
#define GW_REF_MENU_BAR_FILE_MENU_PROPERTIES "gw_menu_bar_file_menu_properties"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_RECENTS_FILES	The file menu -> recents files reference */
#define GW_REF_MENU_BAR_FILE_MENU_RECENTS_FILES "gw_menu_bar_file_menu_recents_files"
/*! @define	GW_REF_MENU_BAR_FILE_MENU_EXIT	The file menu -> exit reference */
#define GW_REF_MENU_BAR_FILE_MENU_EXIT "gw_menu_bar_file_menu_exit"
/*! @define	GW_REF_MENU_BAR_FILE_MENU	The file menu reference */
#define GW_REF_MENU_BAR_FILE_MENU "gw_menu_bar_file_menu"


GtkWidget * menu_file_create ( GtkWindow *w, GtkAccelGroup *ag, GtkWidget *parent)
{
	GtkWidget *menu_file_header = NULL;
	GtkWidget *menu_file = NULL;
	GtkWidget *menu_file_new = NULL;
	GtkWidget *menu_file_open = NULL;
	GtkWidget *menu_file_save = NULL;
	GtkWidget *menu_file_saveas = NULL;
	GtkWidget *menu_file_close = NULL;
	GtkWidget *menu_file_separator = NULL;
	GtkWidget *menu_file_properties = NULL;
/*	GtkWidget *menu_file_recents_files = NULL;
	GtkWidget *menu_file_recents_files_items = NULL;
*/	GtkWidget *menu_file_exit = NULL;
	GtkAccelGroup *menu_file_ag;
	guint tmp_key;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Menu file header */
	menu_file_header = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_header)->child), _( "_File"));
	gtk_widget_add_accelerator ( menu_file_header, "activate_item", ag, tmp_key, GDK_MOD1_MASK, 0);
	gtk_widget_ref ( menu_file_header);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_HEADER, menu_file_header, (GtkDestroyNotify) gtk_widget_unref);

	menu_file = gtk_menu_new ( );
	gtk_widget_ref ( menu_file);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU, menu_file, (GtkDestroyNotify) gtk_widget_unref);
	gtk_menu_item_set_submenu ( GTK_MENU_ITEM ( menu_file_header), menu_file);
	menu_file_ag = gtk_menu_ensure_uline_accel_group ( GTK_MENU ( menu_file));

	/* Menu file -> new */
	menu_file_new = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_new)->child), _( "_New"));
	gtk_widget_add_accelerator ( menu_file_new, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_new);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_NEW, menu_file_new, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_new);
	gtk_widget_add_accelerator ( menu_file_new, "activate", ag, GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_new), "activate", GTK_SIGNAL_FUNC ( menu_file_new_click), w);

	/* Menu file -> open */
	menu_file_open = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_open)->child), _( "_Open"));
	gtk_widget_add_accelerator ( menu_file_open, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_open);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_OPEN, menu_file_open, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_open);
	gtk_widget_add_accelerator ( menu_file_open, "activate", ag, GDK_o, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_open), "activate", GTK_SIGNAL_FUNC ( menu_file_open_click), w);

	/* Menu file -> save */
	menu_file_save = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_save)->child), _( "_Save"));
	gtk_widget_add_accelerator ( menu_file_save, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_save);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_SAVE, menu_file_save, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_save);
	gtk_widget_add_accelerator ( menu_file_save, "activate", ag, GDK_s, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_save), "activate", GTK_SIGNAL_FUNC ( menu_file_save_click), w);

	/* Menu file -> save as */
	menu_file_saveas = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_saveas)->child), _( "Sav_e as"));
	gtk_widget_add_accelerator ( menu_file_saveas, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_saveas);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_SAVEAS, menu_file_saveas, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_saveas);
	gtk_widget_add_accelerator ( menu_file_saveas, "activate", ag, GDK_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_saveas), "activate", GTK_SIGNAL_FUNC ( menu_file_saveas_click), w);

	/* Menu file -> close */
	menu_file_close = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_close)->child), _( "C_lose"));
	gtk_widget_add_accelerator ( menu_file_close, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_close);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_CLOSE, menu_file_close, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_close);
	gtk_widget_add_accelerator ( menu_file_close, "activate", ag, GDK_f, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_close), "activate",GTK_SIGNAL_FUNC ( menu_file_close_click), w);

	/* Menu file -> separator */
	menu_file_separator = gtk_menu_item_new ( );
	gtk_widget_ref ( menu_file_separator);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_SEPARATOR, menu_file_separator, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_separator);
	gtk_widget_set_sensitive ( menu_file_separator, FALSE);

	/* Menu file -> properties */
	menu_file_properties = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_properties)->child), _( "_Properties"));
	gtk_widget_add_accelerator ( menu_file_properties, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_properties);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_PROPERTIES, menu_file_properties, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_properties);
	gtk_widget_add_accelerator ( menu_file_properties, "activate", ag, GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_properties), "activate", GTK_SIGNAL_FUNC ( menu_file_properties_click), w);

	/* Menu file -> separator */
	menu_file_separator = gtk_menu_item_new ( );
	gtk_widget_ref ( menu_file_separator);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_SEPARATOR, menu_file_separator, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_separator);
	gtk_widget_set_sensitive ( menu_file_separator, FALSE);

	/* Menu file -> recents files */
/*	menu_file_recents_files = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_recents_files)->child), _( "_Recents files"));
	gtk_widget_add_accelerator ( menu_file_recents_files, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_recents_files);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_RECENTS_FILES, menu_file_recents_files, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_recents_files);
	gtk_widget_add_accelerator ( menu_file_recents_files, "activate", ag, GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
*/
/*	menu_file_recents_files_items = gtk_menu_new ( );
	gtk_menu_item_set_submenu ( GTK_MENU_ITEM ( menu_file_recents_files), menu_file_recents_files_items);
*/
	/* Menu file -> separator */
/*	menu_file_separator = gtk_menu_item_new ( );
	gtk_widget_ref ( menu_file_separator);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_SEPARATOR, menu_file_separator, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_separator);
	gtk_widget_set_sensitive ( menu_file_separator, FALSE);
*/
	/* Menu file -> exit */
	menu_file_exit = gtk_menu_item_new_with_label ( "");
	tmp_key = gtk_label_parse_uline ( GTK_LABEL ( GTK_BIN ( menu_file_exit)->child), _( "E_xit"));
	gtk_widget_add_accelerator ( menu_file_exit, "activate_item", menu_file_ag, tmp_key, 0, 0);
	gtk_widget_ref ( menu_file_exit);
	gtk_object_set_data_full ( GTK_OBJECT ( w), GW_REF_MENU_BAR_FILE_MENU_EXIT, menu_file_exit, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( menu_file), menu_file_exit);
	gtk_widget_add_accelerator ( menu_file_exit, "activate", ag, GDK_x, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
	gtk_signal_connect ( GTK_OBJECT ( menu_file_exit), "activate", GTK_SIGNAL_FUNC ( menu_file_exit_click), w);

	return menu_file_header;
}
