/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "MenuFileCallBack.h"

#include <gtk/gtk.h>

#include <string.h> /* strcmp */

#include "../ApplicationManager.h"
#include "../GuiManager.h"

#include "../res/Icons.h"

#include "PropertiesBox.h"
#include "StatusBar.h"
#include "CaptureBox.h"
#include "MailWindowBox.h"
#include "MsgBox.h"
#include "DialogBox.h"
#include "MsgBox.h"
#include "FileSelectionBox.h"
#include "NoteBookManagmentCallBack.h"
#include "NoteBookCatalog.h" /* notebook_catalog_ctree_destroy_all */

#include "../data/CatalogFileInfo.h"
#include "../Catalog.h" /* catalog_save_file */
#include "../ApplicationManager.h"


gboolean menu_file_new_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Checks if a catalog has been modified */
	if ( catalog_file_info_get_ismodified ( gtk_ctree_node_get_row_data ( tree, root)) )
	{
		/* If has been modified, ask to save it before create new catalog */
		gw_dialog_box_create ( w, _( "New file"), g_strconcat ( _( "Would you want save catalog :\n\n"), catalog_file_info_get_title ( gtk_ctree_node_get_row_data ( tree, root)), "\n", NULL), GTK_SIGNAL_FUNC ( menu_file_new_save_file_ok), GTK_SIGNAL_FUNC ( menu_file_new_save_file_no), NULL, NULL);
	}
	else
	{
		/* Else ask the new catalog name */
		capture_box_create ( w, _( "New catalog"), _( "Enter catalog name"), "", GTK_SIGNAL_FUNC ( menu_file_new_file_ok));
	}

	return TRUE;
}


gboolean menu_file_open_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Checks if a catalog has been modified */
	if ( catalog_file_info_get_ismodified ( gtk_ctree_node_get_row_data ( tree, root)) )
	{
		/* If has been modified, ask to save it before open another catalog */
		gw_dialog_box_create ( w, _( "Open file"), g_strconcat ( _( "Would you want save catalog :\n\n"), catalog_file_info_get_title ( gtk_ctree_node_get_row_data ( tree, root)), "\n", NULL), GTK_SIGNAL_FUNC ( menu_file_open_save_file_ok), GTK_SIGNAL_FUNC ( menu_file_open_save_file_no), NULL, NULL);
	}
	else
	{
		/* Else opens file selection box*/
		file_selection_box_create ( _( "Open catalog"), (GtkSignalFunc)menu_file_open_file_ok, (GtkSignalFunc)menu_file_open_file_cancel);
	}

	return TRUE;
}


gboolean menu_file_close_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Checks if a catalog has been modified */
	if ( catalog_file_info_get_ismodified ( gtk_ctree_node_get_row_data ( tree, root)) )
	{
		/* If has been modified, ask to save it before close it */
		gw_dialog_box_create ( w, _( "Close file"), g_strconcat ( _( "Would you want save catalog :\n\n"), catalog_file_info_get_title ( gtk_ctree_node_get_row_data ( tree, root)), "\n", NULL), GTK_SIGNAL_FUNC ( menu_file_close_save_file_ok), GTK_SIGNAL_FUNC ( menu_file_close_save_file_no), NULL, NULL);
	}
	else
	{
		/* Else closes it */
		notebook_catalog_ctree_destroy_all ( );
	}

	return TRUE;
}


gboolean menu_file_send_mail_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		/* Check if the mail options if configured. */
		if ( gw_application_manager_can_send_mail ( ) )
		{
			tree = interface_get_tree ( );
			root = interface_get_tree_root ( );

			/* Checks if a catalog has been modified */
			if ( catalog_file_info_get_ismodified ( gtk_ctree_node_get_row_data ( tree, root)) )
			{
				/* If has been modified, ask to save it before send it by mail */
				gw_dialog_box_create ( w, _( "Send file by mail"), g_strconcat ( _( "Would you want save catalog :\n\n"), catalog_file_info_get_title ( gtk_ctree_node_get_row_data ( tree, root)), "\n", NULL), GTK_SIGNAL_FUNC ( menu_file_send_mail_save_file_ok), GTK_SIGNAL_FUNC ( menu_file_send_mail_save_file_no), NULL, NULL);
			}
			else
			{

				/* Else send it by mail */
				gw_mail_window_box_create ( w, catalog_file_info_get_full_name ( gtk_ctree_node_get_row_data ( tree, root)));
			}

			result = TRUE;
		}
		else
		{
			msg_box_create ( w, _( "Send file by mail"), _( "Please configure mail options in the settings in order to send mails."));
		}
	}

	return result;
}



gboolean menu_file_send_mail_save_file_ok ( GtkWidget *bt, GtkWindow *dg)
{
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;
	struct catalog_file_info *catalog_info = NULL;
	gchar **file = NULL;
	gboolean result = FALSE;
	GtkWindow * window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL)
	{
		tree = interface_get_tree ( );
		root = interface_get_tree_root ( );

		catalog_info = gtk_ctree_node_get_row_data ( tree, root);
		file = g_strsplit ( catalog_file_info_get_full_name ( catalog_info), "/", 2);

		/* Destroys the dialog box. */
		gtk_widget_destroy ( GTK_WIDGET ( dg));

		/* Checks if it's a new catalog (in this case his full name is "./[catalog_full_name]"). */
		if ( strcmp ( file[0], ".") == 0 )
		{
			/* If it's a new catalog, asks a file name. */
			file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_send_mail_saveas_file_selection_ok, NULL);
		}
		else
		{
			window = interface_get_main_window ( );

			/* Else save it directly and send it by mail. */
			menu_file_save_click ( NULL, NULL);

			/* Shows the mail window box. */
			gw_mail_window_box_create ( window, catalog_file_info_get_full_name ( catalog_info));
		}

		g_strfreev ( file);

		result = TRUE;
	}

	return result;
}


gboolean menu_file_send_mail_save_file_no ( GtkWidget *bt, GtkWindow *dg)
{
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;
	GtkWindow *window = NULL;
	struct catalog_file_info *catalog_info = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL )
	{
		tree = interface_get_tree ( );
		root = interface_get_tree_root ( );
		window = interface_get_main_window ( );

		/* Destroys the dialog box. */
		gtk_widget_destroy ( GTK_WIDGET ( dg));

		/* Gets catalog informations */
		catalog_info = gtk_ctree_node_get_row_data ( tree, root);

		/* Shows the mail window box. */
		gw_mail_window_box_create ( window, catalog_file_info_get_full_name ( catalog_info));

		result = TRUE;
	}

	return result;
}


gboolean menu_file_send_mail_saveas_file_selection_ok ( GtkWidget *bt, GtkFileSelection *fs)
{
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;
	GtkWindow *window = NULL;
	struct catalog_file_info *catalog_info = NULL;
	gchar *text[1];
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( fs != NULL )
	{
		tree = interface_get_tree ( );
		root = interface_get_tree_root ( );
		window = interface_get_main_window ( );

		if ( strcmp ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
		{
			/* Updates informations of opened catalog. */
			catalog_info = gtk_ctree_node_get_row_data ( tree, root);
			catalog_file_info_set_title ( catalog_info, g_strdup ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry))));
			catalog_file_info_set_full_name ( catalog_info, g_strdup ( gtk_file_selection_get_filename ( fs)));
			catalog_file_info_set_program_maker ( catalog_info, g_strconcat ( PACKAGE, VERSION, NULL));
			catalog_file_info_set_ismodified ( catalog_info, FALSE);

			/* Writes the new window title. */
			gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_full_name ( catalog_info), NULL));

			text[0] = g_strdup ( catalog_file_info_get_title ( catalog_info));
			gtk_ctree_set_node_info ( tree, root, text[0], 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);

			/* Updates informations of opened catalog. */
			catalog_save_file ( gtk_file_selection_get_filename ( fs), tree, root);

			/* Closes the file selection box. */
			gtk_widget_destroy ( GTK_WIDGET ( fs));

			/* Shows the mail window box. */
			gw_mail_window_box_create ( window, catalog_file_info_get_full_name ( catalog_info));
		}

		result = TRUE;
	}

	return result;
}


gboolean menu_file_properties_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		/* Checks if a catalog is opened */
		if ( root != NULL )
		{
			properties_box_create_catalog ( mi, w, tree, root);

			return TRUE;
		}
		else
		{
			msg_box_create ( w, _( "Catalog properties"), _( "No catalog is opened"));
		}
	}

	return result;
}


gboolean menu_file_save_click ( GtkMenuItem *mi, GtkWindow *w)
{
	struct catalog_file_info *catalog_info = NULL;
	gchar **file;
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Gets catalog informations */
	catalog_info = gtk_ctree_node_get_row_data ( tree, root);

	/* Checks if the catalog has been modified */
	if ( catalog_file_info_get_ismodified ( catalog_info) )
	{
		/* If has been modified save it */
		file = g_strsplit ( catalog_file_info_get_full_name ( catalog_info), "/", 2);

		/* Checks if it's a new catalog (in this case his full name is "./[catalog_name]") */
		if ( strcmp ( file[0], ".") == 0 )
		{
			/* If it's a new catalog, ask a file name */
			file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_saveas_file_selection_ok, NULL);
		}
		else
		{
			/* Else save it directly */
			catalog_save_file ( catalog_file_info_get_full_name ( catalog_info), tree, root);
		}

		g_strfreev ( file);
	}

	return TRUE;
}


gboolean menu_file_saveas_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Checks if the catalog has been modified */
	if ( root != NULL )
	{
		file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_saveas_file_selection_ok, NULL);
	}

	return TRUE;
}


gboolean menu_file_exit_click ( GtkMenuItem *mi, GtkWindow *w)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );
	GtkWindow *window = interface_get_main_window ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Checks if a catalog has been modified */
	if ( catalog_file_info_get_ismodified ( gtk_ctree_node_get_row_data ( tree, root)) )
	{
		/* If has been modified, ask to save it before exit program */
		gw_dialog_box_create ( window, g_strconcat ( _( "Quit "), PACKAGE, NULL), g_strconcat ( _( "Would you want save catalog :\n\n"), catalog_file_info_get_title ( gtk_ctree_node_get_row_data ( tree, root)), "\n", NULL), GTK_SIGNAL_FUNC ( menu_file_exit_save_file_ok), GTK_SIGNAL_FUNC ( menu_file_exit_save_file_no), NULL, NULL);
	}
	else
	{
		/* Else exit program */
		menu_file_exit ();
	}

	return TRUE;
}


gboolean menu_file_open_file_ok ( GtkObject *ob, GtkFileSelection *fs)
{
	struct catalog_file_info *file_info;
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( fs != NULL )
	{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : beginning...\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* Checkes if a catalog has been selected */
		if ( strcmp ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
		{
			gtk_widget_hide ( GTK_WIDGET ( fs));
			status_bar_put_messages ( window, _( "Loading catalog"), _( "Please wait..."));
			interface_update ( );

			file_info = gtk_ctree_node_get_row_data ( tree, root);

			if ( file_info != NULL )
			{
				/* If a catalog is opened, don't save it. Destroys the tree and remove the window title. */
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
				g_print ( "*** GW - %s (%d) :: %s() : a catalog is allready opened\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

				notebook_catalog_ctree_destroy_all ( );
			}

			/* Loads the catalog. */
			gw_application_manager_load_catalog ( gtk_file_selection_get_filename ( fs));
			notebook_managment_refresh_categories ( window);

			gtk_widget_destroy ( GTK_WIDGET( fs));
		}
	}

	return FALSE;
}


gboolean menu_file_open_file_cancel ( GtkObject *ob, GtkFileSelection *fs)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( fs != NULL)
	{
		gtk_widget_destroy ( GTK_WIDGET ( fs));

		return TRUE;
	}

	return FALSE;
}


gboolean menu_file_new_file_ok ( GtkWidget *bt, GtkWindow *w)
{
	struct catalog_file_info *file_info = NULL;
	gchar *text[1];
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	text[0] = capture_box_get_text ( w);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : catalog name = %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, text[0]);
#endif

	if ( strcmp ( text[0], "") != 0 )
	{
		/* Checks if a catalog is allready opened */
		if ( root != NULL )
		{
			/* If is opened, closes it */
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
			g_print ( "*** GW - %s (%d) :: %s() : a catalog is allready opened\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

			notebook_catalog_ctree_destroy_all ( );
		}

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : initializes the catalog data structure\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* Initializes the catalog data structure*/
		file_info = catalog_file_info_new ();
		catalog_file_info_set_title ( file_info, g_strdup ( text[0]));
		catalog_file_info_set_full_name ( file_info, g_strconcat ( "./", g_strdup ( text[0]), NULL));
		catalog_file_info_set_program_maker ( file_info, g_strdup ( "GWhere"));
		catalog_file_info_set_ismodified ( file_info, TRUE);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : writes the new window title\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* Writes the new window title */
		gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_title ( file_info), NULL));

		text[0] = g_strdup ( catalog_file_info_get_title ( file_info));
		root = gtk_ctree_insert_node ( tree, NULL, NULL, text, 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	if ( root == NULL )
	{
		g_print ( "*** GW - %s (%d) :: %s() : new node is NULL\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
	}
#endif

		interface_set_tree_root ( root);
		gtk_ctree_node_set_row_data_full ( tree, root, file_info, (GtkDestroyNotify)catalog_file_info_free);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : selects tree's root node\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* Selects tree's root node */
		gtk_ctree_select ( tree, root);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : refreshes categories list\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* Refreshes categories list */
		notebook_managment_refresh_categories ( window);

		gtk_widget_destroy ( GTK_WIDGET ( w));
	}

	return TRUE;
}


gboolean menu_file_new_save_file_ok ( GtkWidget *bt, GtkWindow *dg)
{
	struct catalog_file_info *file_info = NULL;
	gchar **file;
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );
	GtkWindow *window = interface_get_main_window ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	gtk_widget_destroy ( GTK_WIDGET ( dg));
	file_info = gtk_ctree_node_get_row_data ( tree, root);
	file = g_strsplit ( catalog_file_info_get_full_name ( file_info), "/", 2);

	/* Checks if is a new catalog (in this case his full name is "./[catalog_full_name]"). */
	if ( strcmp ( file[0], ".") == 0 )
	{
		/* If it's a new catalog, asks a file name. */
		file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_new_saveas_file_selection_ok, NULL);
	}
	else
	{
		/* Else save it directly and displays a fill box of new catalog */
		menu_file_save_click ( NULL, NULL);
		capture_box_create ( window, _( "New catalog"), _( "Enter catalog name"), "", GTK_SIGNAL_FUNC ( menu_file_new_file_ok));
	}

	g_strfreev ( file);

	return TRUE;
}


gboolean menu_file_new_save_file_no ( GtkWidget *bt, GtkWindow *dg)
{
	GtkWindow *window = interface_get_main_window ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL )
	{
		gtk_widget_destroy ( GTK_WIDGET ( dg));
		capture_box_create ( window, _("New catalog"), _("Enter catalog name"), "", GTK_SIGNAL_FUNC ( menu_file_new_file_ok));

		return TRUE;
	}

            	return FALSE;
}


gboolean menu_file_new_saveas_file_selection_ok ( GtkWidget *bt, GtkFileSelection *fs)
{
	struct catalog_file_info *file_info = NULL;
	gchar *text[1];
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( strcmp ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
	{
		/* Updates informations of the opened catalog. */
		file_info = gtk_ctree_node_get_row_data ( tree, root);
		catalog_file_info_set_title ( file_info, g_strdup ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry))));
		catalog_file_info_set_full_name ( file_info, g_strdup ( gtk_file_selection_get_filename ( fs)));
		catalog_file_info_set_program_maker ( file_info, g_strconcat ( PACKAGE, VERSION, NULL));
		catalog_file_info_set_description ( file_info, catalog_file_info_get_description ( file_info));
		catalog_file_info_set_ismodified ( file_info, FALSE);

		/* Writes the new window title. */
		gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_full_name ( file_info), NULL));

		text[0] = g_strdup ( catalog_file_info_get_title ( file_info));
		gtk_ctree_set_node_info ( tree, root, text[0], 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);

		/* Saves the catalog. */
		catalog_save_file ( catalog_file_info_get_full_name ( file_info), tree, root);

		/* Close the file selection box and displays a new fill box of new catalog. */
		gtk_widget_destroy ( GTK_WIDGET ( fs));
		capture_box_create ( window, _("New catalog"), _("Enter catalog name"), "", GTK_SIGNAL_FUNC ( menu_file_new_file_ok));
	}

	return TRUE;
}


gboolean menu_file_open_save_file_ok ( GtkWidget *bt, GtkWindow *dg)
{
	struct catalog_file_info *file_info = NULL;
	gchar **file;
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	gtk_widget_destroy ( GTK_WIDGET ( dg));
	file_info = gtk_ctree_node_get_row_data ( tree, root);
	file = g_strsplit ( catalog_file_info_get_full_name ( file_info), "/", 2);

	/* Checks if it's a new catalog (in this case his full name is ".[/catalog_full_name]"). */
	if ( strcmp ( file[0], ".") == 0 )
	{
		/* If it's a new catalog, asks file name. */
		file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_open_saveas_file_selection_ok, NULL);
	}
	else
	{
		/* Else save it directly and displays a file selection box to select the catalog to open. */
		menu_file_save_click ( NULL, NULL);
		file_selection_box_create ( _("Open catalog"), (GtkSignalFunc)menu_file_open_file_ok, (GtkSignalFunc)menu_file_open_file_cancel);
	}

	g_strfreev ( file);

	return TRUE;
}


gboolean menu_file_open_save_file_no ( GtkWidget *bt, GtkWindow *dg)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL )
	{
		gtk_widget_destroy ( GTK_WIDGET ( dg));
		file_selection_box_create ( _("Open catalog"), (GtkSignalFunc)menu_file_open_file_ok, (GtkSignalFunc)menu_file_open_file_cancel);

		return TRUE;
	}

            	return FALSE;
}


gboolean menu_file_open_saveas_file_selection_ok ( GtkWidget *bt, GtkFileSelection *fs)
{
	struct catalog_file_info *file_info = NULL;
	gchar *text[1];
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( strcmp ( gtk_entry_get_text( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
	{
		/* Updates informations of opened catalog. */
		file_info = gtk_ctree_node_get_row_data ( tree, root);
		catalog_file_info_set_title ( file_info, g_strdup ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry))));
		catalog_file_info_set_full_name ( file_info, g_strdup ( gtk_file_selection_get_filename ( fs)));
		catalog_file_info_set_program_maker ( file_info, g_strconcat ( PACKAGE, VERSION, NULL));
		catalog_file_info_set_ismodified ( file_info, FALSE);

		/* Writes the new window title. */
		gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_full_name ( file_info), NULL));

		text[0] = g_strdup ( catalog_file_info_get_title ( file_info));
		gtk_ctree_set_node_info ( tree, root, text[0], 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);

		/* Save the catalog. */
		catalog_save_file ( catalog_file_info_get_full_name ( file_info), tree, root);

		/* Closes the file selection box and displays a new fill box of new catalog. */
		gtk_widget_destroy ( GTK_WIDGET ( fs));
		file_selection_box_create ( _( "Open catalog"), (GtkSignalFunc)menu_file_open_file_ok, (GtkSignalFunc)menu_file_open_file_cancel);
	}

	return TRUE;
}


gboolean menu_file_save_file_ok ( GtkWidget *bt, GtkWindow *dg)
{
	struct catalog_file_info *file_info = NULL;
	gchar **file;
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	gtk_widget_destroy ( GTK_WIDGET ( dg));
	file_info = gtk_ctree_node_get_row_data ( tree, root);
	file = g_strsplit ( catalog_file_info_get_full_name ( file_info), "/", 2);

	/* Checks if it's a new catalog (in this case his full name is "./[catalog_full_name]"). */
	if ( strcmp ( file[0], ".") == 0 )
	{
		/* If it's a new catalog, asks un file name. */
		file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_saveas_file_selection_ok, NULL);
	}
	else
	{
		/* Else save it directly. */
		menu_file_save_click ( NULL, NULL);
	}

	g_strfreev ( file);

	return TRUE;
}


gboolean menu_file_save_file_no ( GtkWidget *bt, GtkWindow *dg)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL )
	{
		gtk_widget_destroy ( GTK_WIDGET ( dg));

		return TRUE;
	}

	return FALSE;
}


gboolean menu_file_saveas_file_selection_ok ( GtkWidget *bt, GtkFileSelection *fs)
{
	struct catalog_file_info *file_info = NULL;
	gchar *text[1];
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( strcmp ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
	{
		/* Updates informations of opened catalog. */
		file_info = gtk_ctree_node_get_row_data ( tree, root);
		catalog_file_info_set_title ( file_info, g_strdup ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry))));
		catalog_file_info_set_full_name ( file_info, g_strdup ( gtk_file_selection_get_filename ( fs)));
		catalog_file_info_set_program_maker ( file_info, g_strconcat ( PACKAGE, VERSION, NULL));
		catalog_file_info_set_ismodified ( file_info, FALSE);

		/* Writes the new window title. */
		gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_full_name ( file_info), NULL));

		text[0] = g_strdup ( catalog_file_info_get_title ( file_info));
		gtk_ctree_set_node_info ( tree, root, text[0], 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);

		/* Save the opened catalog. */
		catalog_save_file ( catalog_file_info_get_full_name ( file_info), tree, root);

		/* Closes the file selection box. */
		gtk_widget_destroy ( GTK_WIDGET ( fs));
	}

	return TRUE;
}


gboolean menu_file_close_save_file_ok ( GtkWidget *bt, GtkWindow *dg)
{
	struct catalog_file_info *file_info = NULL;
	gchar **file;
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	gtk_widget_destroy ( GTK_WIDGET ( dg));
	file_info = gtk_ctree_node_get_row_data ( tree, root);
	file = g_strsplit ( catalog_file_info_get_full_name ( file_info), "/", 2);

	/* Checks if it's a new catalog (in this case his full name is "./[catalog_full_name]"). */
	if ( strcmp ( file[0], ".") == 0 )
	{
		/* If it's a new catalog, asks a file name. */
		file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_close_saveas_file_selection_ok, NULL);
	}
	else
	{
		/* Else save it directly and close it. */
		menu_file_save_click ( NULL, NULL);

		notebook_catalog_ctree_destroy_all ( );
	}

	g_strfreev ( file);

	return TRUE;
}


gboolean menu_file_close_save_file_no ( GtkWidget *bt, GtkWindow *dg)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL )
	{
		notebook_catalog_ctree_destroy_all ( );
		gtk_widget_destroy ( GTK_WIDGET ( dg));

		return TRUE;
	}

	return FALSE;
}


gboolean menu_file_close_saveas_file_selection_ok ( GtkWidget *bt, GtkFileSelection *fs)
{
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( strcmp ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
	{
		/* Updates informations of opened catalog. */
		catalog_save_file ( gtk_file_selection_get_filename ( fs), tree, root);

		/* Closes the file selection box and closes the catalog. */
		gtk_widget_destroy ( GTK_WIDGET ( fs));
		notebook_catalog_ctree_destroy_all ( );
	}

	return TRUE;
}


gboolean menu_file_exit_save_file_ok ( GtkWidget *bt, GtkWindow *dg)
{
	struct catalog_file_info *file_info = NULL;
	gchar **file;
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	gtk_widget_destroy ( GTK_WIDGET ( dg));
	file_info = gtk_ctree_node_get_row_data ( tree, root);
	file = g_strsplit ( catalog_file_info_get_full_name ( file_info), "/", 2);

	/* Checks if it's a new catalog (in this case his full name is "./[catalog_full_name]"). */
	if ( strcmp ( file[0], ".") == 0 )
	{
		/* If it's a new catalog, asks a file name. */
		file_selection_box_create ( _( "Save as catalog"), (GtkSignalFunc)menu_file_exit_saveas_file_selection_ok, NULL);
	}
	else
	{
		/* Else save the catalog directly and exit program. */
		menu_file_save_click ( NULL, NULL);
		menu_file_exit ( );
	}

	g_strfreev ( file);

	return TRUE;
}


gboolean menu_file_exit_save_file_no ( GtkWidget *bt, GtkWindow *dg)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( dg != NULL )
	{
		gtk_widget_destroy ( GTK_WIDGET ( dg));
		menu_file_exit ( );

		return TRUE;
	}

	return FALSE;
}


gboolean menu_file_exit_saveas_file_selection_ok ( GtkWidget *bt, GtkFileSelection *fs)
{
	struct catalog_file_info *file_info = NULL;
	gchar *text[1];
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *tree = interface_get_tree ( );
	GtkCTreeNode *root = interface_get_tree_root ( );


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( strcmp ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry)), "") != 0 )
	{
		/* Updates informations of opened catalog. */
		file_info = gtk_ctree_node_get_row_data ( tree, root);
		catalog_file_info_set_title ( file_info, g_strdup ( gtk_entry_get_text ( GTK_ENTRY ( GTK_FILE_SELECTION ( fs)->selection_entry))));
		catalog_file_info_set_full_name ( file_info, g_strdup ( gtk_file_selection_get_filename ( fs)));
		catalog_file_info_set_program_maker ( file_info, g_strconcat ( PACKAGE, VERSION, NULL));
		catalog_file_info_set_description ( file_info, catalog_file_info_get_description ( file_info));
		catalog_file_info_set_ismodified ( file_info, FALSE);

		/* Writes the new window title. */
		gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_full_name ( file_info), NULL));

		text[0] = g_strdup ( catalog_file_info_get_title ( file_info));
		gtk_ctree_set_node_info ( tree, root, text[0], 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);

		/* Saves the catalog. */
		catalog_save_file ( catalog_file_info_get_full_name ( file_info), tree, root);

		/* Exits the program. */
		gtk_widget_destroy ( GTK_WIDGET ( fs));
		menu_file_exit ( );
	}

	return TRUE;
}


void menu_file_exit ( void)
{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Exit main program */
	notebook_catalog_ctree_destroy_all ( );
	gw_application_manager_exit ( );
	gtk_main_quit ( );
}
