/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/*!
 * @header	MenuPopup.h
 *		GWhere<br>
 *		Copyright (C) 2000  Sbastien LECACHEUR<br>
 *		<br>
 *		This Distributed under the terms of the <a href="http://www.gnu.org/copyleft/gpl.html">GNU General Public Licence</a><br>
 *		<br>
 *		This component was developed for the GWhere project.<br>
 *		<br>
 * @abstract	This is a link component between menu bar and functionalities.
 * @author	Sbastien LECACHEUR <zero@gwhere.org>
 * @version	1.0 2001/08/21
 * @see		support.h
 * @link	GWhere Web Site <http://www.gwhere.org>
 * @link	GTK+ Web Site <http://www.gtk.org>
 * @ref		MenuPopup.c
 * @author	Author <zero@gwhere.org>
 */


#ifndef GW_GUI_MENU_POPUP_H
#define GW_GUI_MENU_POPUP_H


/*!
 * @function	menu_popup_create
 * @abstract	Creates the general popup menu.
 * @discussion	This function creates the general popup menu. This popup menu allows to open, close a catalog, exit program,...
 * @result	the popup menu
 */
GtkMenu * menu_popup_create ( void);


/*!
 * @function	menu_popup_catalog_create
 * @abstract	Creates the catalog popup menu.
 * @discussion	This function creates the catalog popup menu. This popup menu allows to open, close a catalog, exit program,... and view catalog properties...
 * @param	node	the node in the tree where is the catalog root
 * @result	the popup menu
 */
GtkMenu * menu_popup_catalog_create ( GtkCTreeNode *node);


/*!
 * @function	menu_popup_disk_create
 * @abstract	Creates the disk popup menu.
 * @discussion	This function creates the disk popup menu. This popup menu allows to open, close a catalog, exit program,... and view disk properties...
 * @param	node	the node in the tree where is the disk root
 * @result	the popup menu
 */
GtkMenu * menu_popup_disk_create ( GtkCTreeNode *node);


/*!
 * @function	menu_popup_dir_create
 * @abstract	Creates the folder popup menu.
 * @discussion	This function creates the folder popup menu. This popup menu allows to open, close a catalog, exit program,... and view folder properties...
 * @param	node	the node in the tree where is the folder
 * @result	the popup menu
 */
GtkMenu * menu_popup_dir_create ( GtkCTreeNode *node);


/*!
 * @function	menu_popup_file_create
 * @abstract	Creates the file popup menu.
 * @discussion	This function creates the file popup menu. This popup menu allows to open, close a catalog, exit program,... and view file properties...
 * @param	node	the node in the tree where is the dis root
 * @result	the popup menu
 */
GtkMenu * menu_popup_file_create ( gint *row);


#endif
