/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "MenuPopupCallBack.h"

#include <gtk/gtk.h>
#include <string.h> /* strcmp */

#include "NoteBookCatalog.h"

#include "../data/CatalogFileInfo.h"
#include "../data/FileInfo.h"
#include "../data/DirData.h"
#include "../data/DiskInfo.h"
#include "../data/DiskData.h"

#include "../GuiManager.h"
#include "../CatalogStats.h"

#include "CategoriesEditBox.h"
#include "PropertiesBox.h"
#include "CaptureBox.h"
#include "TextBox.h"


extern GtkCList *clist_info;
GtkCTreeNode *node_menu_popup_callback = NULL;
GList *files_menu_popup_callback = NULL;


gboolean menu_popup_catalog_rename ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );
		node_menu_popup_callback = node;

		capture_box_create ( window, _( "Rename catalog"), _( "Enter new catalog name"), catalog_file_info_get_name ( gtk_ctree_node_get_row_data ( tree, node)), GTK_SIGNAL_FUNC ( menu_popup_catalog_rename_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_catalog_rename_ok ( GtkWidget *w, GtkWidget *data)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;
	struct catalog_file_info *catalog_info = NULL;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	gchar *tmp_name, *name, *buf;
	gchar **file;
	gint i = -1;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( data != NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE ( GTK_CLIST ( tree)->selection->data);
		window = interface_get_main_window ( );

		tmp_name = capture_box_get_text ( GTK_WINDOW ( data));
		catalog_info = gtk_ctree_node_get_row_data ( tree, node_menu_popup_callback);


		if ( strcmp ( catalog_file_info_get_title ( catalog_info), tmp_name) != 0 )
		{
			/* If the filled name isn't equal to the catalog name. */

			if ( strcmp ( tmp_name, "") != 0 )
			{
				/* If a name is filled. */

				buf = catalog_file_info_get_full_name ( catalog_info);
				file = g_strsplit ( buf, "/", 0);

				while ( file[++i + 1] != NULL )
				{
					/* Goes to the last item of table in order to change it. */
				}

				file [i] = g_strdup ( tmp_name);

//				catalog_file_info_set_full_name ( catalog_info, g_strjoinv ( "/", file));
//				catalog_file_info_set_title ( catalog_info, g_strdup ( tmp_name));
				catalog_file_info_set_name ( catalog_info, g_strdup ( tmp_name));
				catalog_file_info_set_ismodified ( catalog_info, TRUE);

				gtk_ctree_node_get_pixtext ( tree, node_menu_popup_callback, 0, &name, &spc, &pxmp, &btmp);
				gtk_ctree_node_set_pixtext ( tree, node_menu_popup_callback, 0, g_strdup ( tmp_name), spc, pxmp, btmp);
				gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_full_name ( catalog_info), NULL));

				g_strfreev ( file);

				result = TRUE;
			}
		}

		gtk_widget_destroy ( data);
		node_menu_popup_callback = NULL;

		/* Selects the selected item in the exploration tree to refresh the list of files. */
		gtk_ctree_select ( tree, selection);
	}

	return result;
}


gboolean menu_popup_catalog_add_new_disk ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		/* FIX BUG : Must implement this function!! */
	}

	return result;
}


gboolean menu_popup_catalog_edit_description ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );

		node_menu_popup_callback = node;
		text_box_create ( window, _( "Edit description"), _( "Catalog description"), catalog_file_info_get_description ( gtk_ctree_node_get_row_data ( tree, node)), GTK_SIGNAL_FUNC ( menu_popup_catalog_edit_description_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_catalog_edit_description_ok ( GtkWidget *b, GtkWidget *w )
{
	gboolean result = FALSE;
	struct catalog_file_info *catalog_info = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE( GTK_CLIST ( tree)->selection->data);

		catalog_info = gtk_ctree_node_get_row_data ( tree, node_menu_popup_callback);

		if ( catalog_file_info_get_description ( catalog_info) != NULL )
		{
			if ( strcmp ( text_box_get_text ( w), catalog_file_info_get_description ( catalog_info)) != 0 )
			{
				/* If the filled description isn't equals to the catalog description. */
				if ( strcmp ( text_box_get_text ( w), "") != 0 )
				{
					catalog_file_info_set_description ( catalog_info, g_strdup ( text_box_get_text ( w)));

					/* Indicates that catalog have been modified. */
					catalog_file_info_set_ismodified ( catalog_info, TRUE);
				}
			}
		}
		else
		{
			if ( strcmp ( text_box_get_text ( w), "") != 0 )
			{
				catalog_file_info_set_description ( catalog_info, g_strdup ( text_box_get_text ( w)));

				/* Indicates that catalog have been modified. */
				catalog_file_info_set_ismodified ( catalog_info, TRUE);
			}
		}

		gtk_widget_destroy ( w);

		/* Selects the selected item in the exploration tree to refresh the list of files. */
		gtk_ctree_select ( tree, selection);

		node_menu_popup_callback = NULL;

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_catalog_edit_categories ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	struct catalog_file_info *catalog_info = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		catalog_info = gtk_ctree_node_get_row_data ( tree, node);
		window = interface_get_main_window ( );

		categories_edit_box_create ( window, catalog_info);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_catalog_properties ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkWindow *window = NULL;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		window = interface_get_main_window ( );
		tree = interface_get_tree ( );

		properties_box_create_catalog ( m, window, tree, node);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_move_to ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );

		if ( !GTK_CTREE_ROW ( node)->expanded )
		{
			if ( node != NULL )
			{
				gtk_ctree_expand ( tree, node);
			}
		}

		gtk_ctree_select ( GTK_CTREE ( tree), node);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_rename ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );

		node_menu_popup_callback = node;
		capture_box_create ( window, _( "Rename disk"), _( "Enter new disk name"), disk_info_get_name ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, node))), GTK_SIGNAL_FUNC ( menu_popup_disk_rename_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_rename_ok ( GtkWidget *w, GtkWidget *data)
{
	gboolean result = FALSE;
	struct disk_info *disk_info = NULL;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	gchar *tmp_name, *disk_name, *name;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( data != NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE ( GTK_CLIST ( tree)->selection->data);

		tmp_name = capture_box_get_text ( GTK_WINDOW ( data));
		disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, node_menu_popup_callback));

		if ( strcmp ( disk_info_get_name ( disk_info), tmp_name) != 0 )
		{
			/* Checks if the filled name isn't equals to the disk name. */
			disk_name = disk_get_name ( tmp_name);

			if ( strcmp ( disk_name, "") != 0 )
			{
				/* If a name has been filled. */
				gtk_ctree_node_get_pixtext ( tree, node_menu_popup_callback, 0, &name, &spc, &pxmp, &btmp);
				gtk_ctree_node_set_pixtext ( tree, node_menu_popup_callback, 0, disk_name, spc, pxmp, btmp);
				disk_info_set_name ( disk_info, g_strdup ( disk_name));
				g_free ( disk_name);

				/* Indicates that catalog has been modified. */
				catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, GTK_CTREE_ROW ( node_menu_popup_callback)->parent), TRUE);

				/* Selects the selected item in the exploration tree to refresh the list of files. */
				gtk_ctree_select ( tree, selection);
			}
		}

		gtk_widget_destroy ( data);
		node_menu_popup_callback = NULL;

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_delete ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkWindow *w = NULL;
	GtkCList *clist_info = NULL;
	GtkCList *clist = NULL;
	struct catalog_file_info *file_info = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *parent = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		parent = GTK_CTREE_ROW (node)->parent;
		w = interface_get_main_window ( );
		clist_info = notebook_catalog_get_clist_explorer ( w);
		tree = interface_get_tree ( );

		clist = GTK_CLIST ( tree);
		gtk_clist_freeze ( clist);
		gtk_clist_freeze ( clist_info);
		gtk_clist_clear ( clist_info);
		gtk_ctree_remove_node ( tree, node);
		gtk_clist_thaw ( clist);
		gtk_clist_thaw ( clist_info);
		gtk_ctree_select( GTK_CTREE ( tree), parent);


		file_info = gtk_ctree_node_get_row_data ( tree, parent);
		catalog_file_info_set_ismodified ( file_info, TRUE);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_change_volume_label ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );

		node_menu_popup_callback = node;
		capture_box_create ( window, _( "Change volume label"), _( "Enter new disk volume label"), disk_info_get_volume ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, node))), GTK_SIGNAL_FUNC ( menu_popup_disk_change_volume_label_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_change_volume_label_ok ( GtkWidget *w, GtkWidget *data)
{
	gboolean result = FALSE;
	struct disk_info *disk_info = NULL;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	gchar *tmp_name, *name;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( data != NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE ( GTK_CLIST ( tree)->selection->data);

		tmp_name = capture_box_get_text ( GTK_WINDOW ( data));
		disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, node_menu_popup_callback));


		if ( strcmp ( disk_info_get_volume ( disk_info), tmp_name) != 0 )
		{
			/* Checks if the filled volume name isn't equals to the disk volume name. */
			if ( strcmp ( tmp_name, "") != 0 )
			{
				/* If a volume name has been filled. */
				gtk_ctree_node_get_pixtext ( tree, node_menu_popup_callback, 0, &name, &spc, &pxmp, &btmp);
				gtk_ctree_node_set_pixtext ( tree, node_menu_popup_callback, 0, tmp_name, spc, pxmp, btmp);
				disk_info_set_volume ( disk_info, g_strdup ( tmp_name));

				/* Indicates that catalog has been modified. */
				catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, GTK_CTREE_ROW ( node_menu_popup_callback)->parent), TRUE);

				/* Selects the selected item in the exploration tree to refresh the list of files. */
				gtk_ctree_select ( tree, selection);
			}
		}

		gtk_widget_destroy ( data);
		node_menu_popup_callback = NULL;

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_edit_description ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );

		node_menu_popup_callback = node;
		text_box_create ( window, _( "Edit description"), _( "Disk description"), disk_info_get_description ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, node))), GTK_SIGNAL_FUNC ( menu_popup_disk_edit_description_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_edit_description_ok ( GtkWidget *b, GtkWidget *w )
{
	gboolean result = FALSE;
	struct disk_info *disk_info = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE( GTK_CLIST( tree)->selection->data);

		disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, node_menu_popup_callback));

		if ( disk_info_get_description ( disk_info) != NULL )
		{
			if ( strcmp ( text_box_get_text ( w), disk_info_get_description ( disk_info)) != 0 )
			{
				/* Checks if the filled description isn't equals to the disk description. */
				if ( strcmp ( text_box_get_text ( w), "") != 0 )
				{
					disk_info_set_description ( disk_info, g_strdup ( text_box_get_text ( w)));

					/* Indicates that the catalog has been modified. */
					catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, GTK_CTREE_ROW ( node_menu_popup_callback)->parent), TRUE);
				}
			}
		}
		else
		{
			if ( strcmp ( text_box_get_text ( w), "") != 0 )
			{
				disk_info_set_description ( disk_info, g_strdup ( text_box_get_text ( w)));

				/* Indicates that the catalog has been modified. */
				catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, GTK_CTREE_ROW ( node_menu_popup_callback)->parent), TRUE);
			}
		}

		gtk_widget_destroy ( w);

		/* Selects the selected item in the exploration tree to refresh the list of files. */
		gtk_ctree_select ( tree, selection);

		node_menu_popup_callback = NULL;

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_disk_properties ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkWindow *window = NULL;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		window = interface_get_main_window ( );
		tree = interface_get_tree ( );

		properties_box_create_disk ( m, window, tree, node);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_folder_move_to ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );

		if ( !GTK_CTREE_ROW ( node)->expanded )
		{
			if ( node != NULL )
			{
				gtk_ctree_expand ( tree, node);
			}
		}

		gtk_ctree_select ( GTK_CTREE ( tree), node);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_folder_edit_description ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );

		node_menu_popup_callback = node;
		text_box_create ( window, _( "Edit description"), _( "Folder description"), file_info_get_description ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, node))), GTK_SIGNAL_FUNC ( menu_popup_folder_edit_description_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_folder_edit_description_ok ( GtkWidget *b, GtkWidget *w )
{
	gboolean result = FALSE;
	struct file_info *dir_info = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE( GTK_CLIST ( tree)->selection->data);

		dir_info = dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, node_menu_popup_callback));

		if ( file_info_get_description ( dir_info) != NULL )
		{
			if ( strcmp ( text_box_get_text ( w), file_info_get_description ( dir_info)) != 0 )
			{
				/* Checks if the filled description isn't equals to the folder description. */
				if ( strcmp ( text_box_get_text ( w), "") != 0 )
				{
					file_info_set_description ( dir_info, g_strdup ( text_box_get_text ( w)));

					/* Indicates that catalog has been modified. */
					catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, folder_get_catalog ( tree, node_menu_popup_callback)), TRUE);
				}
			}
		}
		else
		{
			if ( strcmp ( text_box_get_text ( w), "") != 0 )
			{
				file_info_set_description ( dir_info, g_strdup ( text_box_get_text ( w)));

				/* Indicates that catalog has been modified. */
				catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, folder_get_catalog ( tree, node_menu_popup_callback)), TRUE);
			}
		}

		gtk_widget_destroy ( w);

		/* Selects the selected item in the exploration tree to refresh the list of files. */
		gtk_ctree_select ( tree, selection);

		node_menu_popup_callback = NULL;

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_folder_remove ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkWindow *w = NULL;
	GtkCList *clist_info = NULL;
	GtkCList *clist;
	struct catalog_file_info *file_info = NULL;
	GtkCTreeNode *tmp, *parent = NULL;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		parent = GTK_CTREE_ROW (node)->parent;
		w = interface_get_main_window ( );
		clist_info = notebook_catalog_get_clist_explorer ( w);
		tree = interface_get_tree ( );

		/* Updates the folders size which contains the folder to delete. */
		tmp = parent;

		while ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( tmp)->parent)->parent != NULL )
		{
			file_info_set_size ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp)), file_info_get_size ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp))) - file_info_get_size ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, node))));
			tmp = GTK_CTREE_ROW ( tmp)->parent;
		}

		/* Updates the disk size which contains the folder to delete. */
		disk_info_set_full ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp)), disk_info_get_full ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp))) - file_info_get_size ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, node))));

		clist = GTK_CLIST ( tree);
		gtk_clist_freeze ( clist);
		gtk_clist_freeze ( clist_info);
		gtk_clist_clear ( clist_info);
		gtk_ctree_remove_node ( tree, node);
		gtk_clist_thaw ( clist);
		gtk_clist_thaw ( clist_info);
		gtk_ctree_select( GTK_CTREE ( tree), parent);

		file_info = gtk_ctree_node_get_row_data ( tree, GTK_CTREE_ROW ( tmp)->parent);

		/* Indicates that catalog has been modified. */
		catalog_file_info_set_ismodified ( file_info, TRUE);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_folder_properties ( GtkMenuItem *m, GtkCTreeNode *node)
{
	gboolean result = FALSE;
	GtkWindow *window = NULL;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node != NULL )
	{
		window = interface_get_main_window ( );
		tree = interface_get_tree ( );

		properties_box_create_folder ( m, window, tree, node);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_file_edit_description ( GtkMenuItem *m, gint *row)
{
	gboolean result = FALSE;
	GtkWindow *w = NULL;
	GtkCList *clist_info = NULL;
	GList *child = NULL;
	GtkCTreeNode *selection;
	gchar *name;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	GtkCTree *tree = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( row != NULL )
	{
		tree = interface_get_tree ( );
		window = interface_get_main_window ( );

		w = interface_get_main_window ( );
		clist_info = notebook_catalog_get_clist_explorer ( w);

		/* Gets the file name. */
		gtk_clist_get_pixtext ( GTK_CLIST ( clist_info), *row, 0, &name, &spc, &btmp, &pxmp);
		g_free ( row);

		/* Gets the parent folder. */
		selection = GTK_CTREE_NODE( GTK_CLIST( tree)->selection->data);

		/* Gets file informations. */
		files_menu_popup_callback = child = dir_data_get_child_named ( gtk_ctree_node_get_row_data ( tree, selection), name);

		node_menu_popup_callback = selection;
		text_box_create ( window, _( "Edit description"), _( "File description"), file_info_get_description ( child->data), GTK_SIGNAL_FUNC ( menu_popup_file_edit_description_ok));

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_file_edit_description_ok ( GtkWidget *b, GtkWidget *w )
{
	gboolean result = FALSE;
	struct file_info *file_info = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *selection = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w!= NULL )
	{
		tree = interface_get_tree ( );
		selection = GTK_CTREE_NODE( GTK_CLIST ( tree)->selection->data);

		file_info = files_menu_popup_callback->data;

		if ( file_info_get_description ( file_info) != NULL )
		{
			if ( strcmp ( text_box_get_text ( w), file_info_get_description ( file_info)) != 0 )
			{
				/* Checks of the filled description isn't equals to the file description. */
				if ( strcmp ( text_box_get_text ( w), "") != 0 )
				{
					file_info_set_description ( file_info, g_strdup ( text_box_get_text ( w)));

					/* Indicates that catalog has been modified. */
					catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, folder_get_catalog ( tree, node_menu_popup_callback)), TRUE);
				}
			}
		}
		else
		{
			if ( strcmp ( text_box_get_text ( w), "") != 0 )
			{
				file_info_set_description ( file_info, g_strdup ( text_box_get_text ( w)));

				/* Indicates that catalog has been modified. */
				catalog_file_info_set_ismodified ( gtk_ctree_node_get_row_data ( tree, folder_get_catalog ( tree, node_menu_popup_callback)), TRUE);
			}
		}

		gtk_widget_destroy ( w);

		/* Selects the selected item in the exploration tree to refresh the list of files. */
		gtk_ctree_select ( tree, selection);

		node_menu_popup_callback = NULL;
		files_menu_popup_callback = NULL;

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_file_remove ( GtkMenuItem *m, gint *row)
{
	gboolean result = FALSE;
	GtkWindow *w = NULL;
	GtkCList *clist_info = NULL;
	GList *child = NULL;
	struct catalog_file_info *file_info = NULL;
	GtkCTreeNode *selection, *parent, *tmp;
	gchar *name = NULL;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( row != NULL )
	{
		tree = interface_get_tree ( );
		w = interface_get_main_window ( );
		clist_info = notebook_catalog_get_clist_explorer ( w);

		/* Gets the file name to delete. */
		gtk_clist_get_pixtext ( GTK_CLIST ( clist_info), *row, 0, &name, &spc, &btmp, &pxmp);
		g_free ( row);

		/* Gets the parent folder. */
		parent = selection = GTK_CTREE_NODE( GTK_CLIST( tree)->selection->data);

		/* Gets the file informations */
		child = dir_data_get_child_named ( gtk_ctree_node_get_row_data ( tree, selection), name);

		/* Updates the folders size which containsthe file to delete. */
		tmp = parent;

		while ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( tmp)->parent)->parent != NULL )
		{
			file_info_set_size ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp)), file_info_get_size ( dir_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp))) - file_info_get_size ( child->data));
			tmp = GTK_CTREE_ROW ( tmp)->parent;
		}

		/* Updates the disk size which contains the file to delete. */
		disk_info_set_full ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp)), disk_info_get_full ( disk_data_get_info ( gtk_ctree_node_get_row_data ( tree, tmp))) - file_info_get_size ( child->data));

		/* Removes the file from catalog. */
		file_info_free ( child->data);
		child = g_list_remove ( child, child->data);

		/* Selects the selected item in the exploration tree to refresh the list of files. */
		gtk_ctree_select ( GTK_CTREE ( tree), selection);


		/*file_info = gtk_object_get_data ( GTK_OBJECT ( window), "window");*/
		file_info = gtk_ctree_node_get_row_data ( tree, GTK_CTREE_ROW ( tmp)->parent);
		catalog_file_info_set_ismodified ( file_info, TRUE);

		result = TRUE;
	}

	return result;
}


gboolean menu_popup_file_properties ( GtkMenuItem *m, gint *row)
{
	gboolean result = FALSE;
	GtkWindow *w = NULL;
	GtkCList *clist_info = NULL;
	GList *child = NULL;
	GtkCTreeNode *selection;
	gchar *name = NULL;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	GtkWindow *window = NULL;
	GtkCTree *tree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( row != NULL )
	{
		window = interface_get_main_window ( );
		tree = interface_get_tree ( );

		w = interface_get_main_window ( );
		clist_info = notebook_catalog_get_clist_explorer ( w);

		/* Gets the file name to delete. */
		gtk_clist_get_pixtext ( GTK_CLIST ( clist_info), *row, 0, &name, &spc, &btmp, &pxmp);
		g_free ( row);

		/* Gets the parent folder. */
		selection = GTK_CTREE_NODE ( GTK_CLIST ( tree)->selection->data);

		/* Gets file informations. */
		child = dir_data_get_child_named ( gtk_ctree_node_get_row_data ( tree, selection), name);

		properties_box_create_file ( m, window, tree, selection, child);

		result = TRUE;
	}

	return result;
}
