/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "NoteBookCatalog.h"

#include <gtk/gtk.h>
#include <string.h> /* strcmp */

#include "../GuiManager.h"

#include "NoteBookCatalogCallBack.h"
#include "MenuPopup.h"
#include "NoteBookSearch.h" /* notebook_search_get_clist_search */
#include "NoteBookManagmentCallBack.h" /* notebook_managment_refresh_categories */
#include "StatusBar.h" /* status_bar_put_messages */

#include "GWReferences.h"


/*! @define	GW_REF_NOTEBOOK_CATALOG_PANE	The catalog pane */
#define GW_REF_NOTEBOOK_CATALOG_PANE "gw_ref_notebook_catalog_pane"
/*! @define	GW_REF_NOTEBOOK_CATALOG_ARCHIVE_LABEL	The the archive label in the scrollpane */
#define GW_REF_NOTEBOOK_CATALOG_ARCHIVE_LABEL "gw_ref_notebook_catalog_archive_label"
/*! @define	GW_REF_NOTEBOOK_CATALOG_TREE_SCROLLPANE	The tree scrollpane */
#define GW_REF_NOTEBOOK_CATALOG_TREE_SCROLLPANE "gw_ref_notebook_catalog_tree_scrollpane"
/*! @define	GW_REF_NOTEBOOK_CATALOG_LIST_SCROLLPANE	The list scrollpane */
#define GW_REF_NOTEBOOK_CATALOG_LIST_SCROLLPANE "gw_ref_notebook_catalog_list_scrollpane"


GtkWidget * notebook_catalog_create_and_add ( GtkWindow * window, GtkNotebook * parent)
{
	GtkCList *clist_info = NULL;
	GtkWidget *LblCatalogue, *HpanCatalogue;
	GtkWidget *LblArchive;
	GtkWidget *ScrWinCTree, *ScrWinInfoCatalogue;
	GtkWidget *CTreeCatalog, *CListInfo;
	GtkWidget *LblInfoNom, *LblInfoDroits, *LblInfoProprietaire, *LblInfoGroupe, *LblInfoTaille, *LblInfoInoeud, *LblInfoDate, *LblInfoCategory, *LblInfoDescription;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Horizontal pane for the first notebook pane.*/
	HpanCatalogue = gtk_hpaned_new ();
	gtk_widget_ref ( HpanCatalogue);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_CATALOG_PANE, HpanCatalogue,(GtkDestroyNotify) gtk_widget_unref);
	gtk_container_add ( GTK_CONTAINER ( parent), HpanCatalogue);
	gtk_paned_set_position ( GTK_PANED ( HpanCatalogue), 205);

	/* Catalog tree explorer */
	ScrWinCTree = gtk_scrolled_window_new ( NULL, NULL);
	gtk_widget_ref ( ScrWinCTree);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_CATALOG_TREE_SCROLLPANE, ScrWinCTree,(GtkDestroyNotify) gtk_widget_unref);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW ( ScrWinCTree), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_container_add ( GTK_CONTAINER ( HpanCatalogue), ScrWinCTree);

	CTreeCatalog = gtk_ctree_new ( 1, 0);
	gtk_widget_ref ( CTreeCatalog);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_TREE, CTreeCatalog, (GtkDestroyNotify) gtk_widget_unref);
	gtk_signal_connect ( GTK_OBJECT ( CTreeCatalog), "button_press_event", GTK_SIGNAL_FUNC ( notebook_catalog_ctree_button_press_event), NULL);
	gtk_signal_connect ( GTK_OBJECT ( CTreeCatalog), "tree_select_row", GTK_SIGNAL_FUNC ( notebook_catalog_ctree_select_row), NULL);
	gtk_clist_set_selection_mode ( GTK_CLIST ( CTreeCatalog), GTK_SELECTION_BROWSE);
	gtk_container_add ( GTK_CONTAINER ( ScrWinCTree), CTreeCatalog);
	gtk_clist_set_column_width ( GTK_CLIST ( CTreeCatalog), 0, 80);
	gtk_clist_set_column_auto_resize ( GTK_CLIST ( CTreeCatalog), 0, TRUE);
	gtk_clist_column_titles_show ( GTK_CLIST ( CTreeCatalog));

	LblArchive = gtk_label_new ( _( "Archive"));
	gtk_widget_ref ( LblArchive);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_CATALOG_ARCHIVE_LABEL, LblArchive,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( GTK_CLIST ( CTreeCatalog), 0, LblArchive);

	/* List of catalog explorer. */
	ScrWinInfoCatalogue = gtk_scrolled_window_new ( NULL, NULL);
	gtk_widget_ref ( ScrWinInfoCatalogue);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_CATALOG_LIST_SCROLLPANE, ScrWinInfoCatalogue,(GtkDestroyNotify) gtk_widget_unref);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW ( ScrWinInfoCatalogue), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_container_add ( GTK_CONTAINER ( HpanCatalogue), ScrWinInfoCatalogue);

	CListInfo = gtk_clist_new ( 9);
	clist_info = GTK_CLIST ( CListInfo);
	gtk_widget_ref ( CListInfo);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_CLIST_EXPLORER, CListInfo,(GtkDestroyNotify) gtk_widget_unref);
	gtk_signal_connect ( GTK_OBJECT ( CListInfo), "button_press_event", GTK_SIGNAL_FUNC ( notebook_catalog_clist_button_press_event), NULL);
	gtk_clist_set_selection_mode ( clist_info, GTK_SELECTION_BROWSE);
	gtk_container_add ( GTK_CONTAINER ( ScrWinInfoCatalogue), CListInfo);

	gtk_clist_set_column_width ( clist_info, 0, 320);
	gtk_clist_set_column_width ( clist_info, 1, 65);
	gtk_clist_set_column_width ( clist_info, 2, 40);
	gtk_clist_set_column_width ( clist_info, 3, 40);
	gtk_clist_set_column_width ( clist_info, 4, 60);
	gtk_clist_set_column_width ( clist_info, 5, 35);
	gtk_clist_set_column_width ( clist_info, 6, 60);
	gtk_clist_set_column_width ( clist_info, 7, 60);
	gtk_clist_column_titles_show ( clist_info);
	gtk_clist_set_column_justification ( clist_info, 1, GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification ( clist_info, 2, GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification ( clist_info, 3, GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification ( clist_info, 4, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification ( clist_info, 5, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification ( clist_info, 6, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification ( clist_info, 7, GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification ( clist_info, 8, GTK_JUSTIFY_LEFT);
//	gtk_clist_set_sort_column( clist_info, 0);
//	gtk_clist_set_sort_type( clist_info, GTK_SORT_ASCENDING);
//	gtk_clist_set_auto_sort( clist_info, TRUE);

	LblInfoNom = gtk_label_new ( _("Name"));
	gtk_widget_ref ( LblInfoNom);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Nom", LblInfoNom,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 0, LblInfoNom);

	LblInfoDroits = gtk_label_new ( _("Rights"));
	gtk_widget_ref ( LblInfoDroits);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Droits", LblInfoDroits,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 1, LblInfoDroits);

	LblInfoProprietaire = gtk_label_new ( _("Owner"));
	gtk_widget_ref ( LblInfoProprietaire);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Propritaire", LblInfoProprietaire,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 2, LblInfoProprietaire);

	LblInfoGroupe = gtk_label_new ( _("Group"));
	gtk_widget_ref ( LblInfoGroupe);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Groupe", LblInfoGroupe,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 3, LblInfoGroupe);

	LblInfoTaille = gtk_label_new ( _("Size"));
	gtk_widget_ref ( LblInfoTaille);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Taille", LblInfoTaille,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 4, LblInfoTaille);

	LblInfoInoeud = gtk_label_new ( _("Inode"));
	gtk_widget_ref ( LblInfoInoeud);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "I-noeud", LblInfoInoeud,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 5, LblInfoInoeud);

	LblInfoDate = gtk_label_new ( _("Date"));
	gtk_widget_ref ( LblInfoDate);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Date", LblInfoDate,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 6, LblInfoDate);

	LblInfoCategory = gtk_label_new ( _("Category"));
	gtk_widget_ref ( LblInfoCategory);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Category", LblInfoCategory,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 7, LblInfoCategory);

	LblInfoDescription = gtk_label_new ( _("Description"));
	gtk_widget_ref ( LblInfoDescription);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "Description", LblInfoDescription,(GtkDestroyNotify) gtk_widget_unref);
	gtk_clist_set_column_widget ( clist_info, 8, LblInfoDescription);

	LblCatalogue = gtk_label_new ( _("Catalog"));
	gtk_widget_ref ( LblCatalogue);
	gtk_object_set_data_full ( GTK_OBJECT ( window), "LblCatalogue", LblCatalogue,(GtkDestroyNotify) gtk_widget_unref);
	gtk_notebook_set_tab_label ( parent, gtk_notebook_get_nth_page ( parent, 0), LblCatalogue);

	return HpanCatalogue;
}


GtkCList * notebook_catalog_get_clist_explorer ( GtkWindow *w)
{
	GtkCList *clist_explorer = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		clist_explorer = GTK_CLIST ( gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_CLIST_EXPLORER));
	}

	return clist_explorer;
}


gint notebook_catalog_clist_explorer_set_column_title_file ( void)
{
	GtkWindow *w = NULL;
	GtkCList *clist = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	w = interface_get_main_window ( );
	clist = notebook_catalog_get_clist_explorer ( w);

	if ( clist != NULL )
	{
		/* Sets columns width. */
		gtk_clist_set_column_width ( clist, 0, 320);
//		gtk_clist_set_column_width ( clist, 1, 65);
		gtk_clist_set_column_width ( clist, 2, 40);
		gtk_clist_set_column_width ( clist, 3, 40);
		gtk_clist_set_column_width ( clist, 4, 60);
		gtk_clist_set_column_width ( clist, 5, 35);
//		gtk_clist_set_column_width ( clist, 6, 60);
//		gtk_clist_set_column_width ( clist_info, 7, 60);
//		gtk_clist_set_column_justification ( clist, 1, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 2, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 3, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 4, GTK_JUSTIFY_RIGHT);
//		gtk_clist_set_column_justification ( clist, 5, GTK_JUSTIFY_RIGHT);
//		gtk_clist_set_column_justification ( clist, 6, GTK_JUSTIFY_RIGHT);
//		gtk_clist_set_column_justification ( clist, 7, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 8, GTK_JUSTIFY_LEFT);

		/* Sets columns title. */
//		gtk_clist_set_column_title ( clist, 0, _( "Name"));
		gtk_clist_set_column_title ( clist, 1, _( "Rights"));
		gtk_clist_set_column_title ( clist, 2, _( "Owner"));
		gtk_clist_set_column_title ( clist, 3, _( "Group"));
		gtk_clist_set_column_title ( clist, 4, _( "Size"));
		gtk_clist_set_column_title ( clist, 5, _( "Inode"));
//		gtk_clist_set_column_title ( clist, 6, _( "Date"));
//		gtk_clist_set_column_title ( clist, 7, _( "Category"));
//		gtk_clist_set_column_title ( clist, 8, _( "Description"));

		return 0;
	}

	return -1;
}


gint notebook_catalog_clist_explorer_set_column_title_disk ( void)
{
	GtkWindow *w = NULL;
	GtkCList *clist = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	w = interface_get_main_window ( );
	clist = notebook_catalog_get_clist_explorer ( w);

	if ( clist != NULL )
	{
		/* Sets columns width. */
		gtk_clist_set_column_width ( clist, 0, 200);
//		gtk_clist_set_column_width ( clist, 1, 65);
		gtk_clist_set_column_width ( clist, 2, 120);
		gtk_clist_set_column_width ( clist, 3, 80);
		gtk_clist_set_column_width ( clist, 4, 60);
		gtk_clist_set_column_width ( clist, 5, 65);
//		gtk_clist_set_column_width ( clist, 6, 60);
//		gtk_clist_set_column_width ( clist, 7, 60);
//		gtk_clist_set_column_justification ( clist, 1, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 2, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 3, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 4, GTK_JUSTIFY_RIGHT);
//		gtk_clist_set_column_justification ( clist, 5, GTK_JUSTIFY_RIGHT);
//		gtk_clist_set_column_justification ( clist, 6, GTK_JUSTIFY_RIGHT);
//		gtk_clist_set_column_justification ( clist, 7, GTK_JUSTIFY_CENTER);
//		gtk_clist_set_column_justification ( clist, 8, GTK_JUSTIFY_LEFT);

		/* Sets columns title. */
//		gtk_clist_set_column_title ( clist, 0, _( "Name"));
		gtk_clist_set_column_title ( clist, 1, _( "Archive N#"));
		gtk_clist_set_column_title ( clist, 2, _( "Type"));
		gtk_clist_set_column_title ( clist, 3, _( "Volume Name"));
		gtk_clist_set_column_title ( clist, 4, _( "Total Size"));
		gtk_clist_set_column_title ( clist, 5, _( "Free Space"));
//		gtk_clist_set_column_title ( clist, 6, _( "Date"));
//		gtk_clist_set_column_title ( clist, 7, _( "Category"));
//		gtk_clist_set_column_title ( clist, 8, _( "Description"));

		return 0;
	}

	return -1;
}


GtkCTreeNode * notebook_catalog_ctree_node_get_child  ( GtkCTreeNode *node, gchar *name)
{
	GtkCTreeNode *child = NULL;
	gchar *txt;
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	GtkCTree *ctree = interface_get_tree ( );


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW ( node)->children;

	while ( child != NULL )
	{
		if ( (gtk_ctree_node_get_pixtext ( GTK_CTREE ( ctree), child, 0, &txt, &spc, &pxmp, &btmp) && strcmp ( txt, name) == 0) )
		{
			return child;
		}

		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return NULL;
}


gint notebook_catalog_ctree_node_get_num_children ( GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	gint n = 0;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	child = GTK_CTREE_ROW ( node)->children;

	while ( child != NULL )
	{
		n++;
		child = GTK_CTREE_ROW ( child)->sibling;
	}

	return n;
}


gint notebook_catalog_ctree_destroy_all ( void)
{
	/* Destroys all explorer tree :'( */
	GtkWindow *w = NULL;
	GtkCList *clist_info = NULL;
	GtkCList *clist_result = NULL;
	GtkCList *clist;
	GtkWindow *window = interface_get_main_window ( );
	GtkCTree *ctree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	w = interface_get_main_window ( );
	clist_info = notebook_catalog_get_clist_explorer ( w);
	clist_result = notebook_search_get_clist_search ( w);
	ctree = interface_get_tree ( );
	root = interface_get_tree_root ( );

	status_bar_put_messages ( w, _( "Closing catalog"), _( "Please wait..."));
	interface_update ( );

	clist = GTK_CLIST ( ctree);
	gtk_clist_freeze ( clist);
	gtk_clist_freeze ( clist_info);
	gtk_clist_freeze ( clist_result);
	gtk_clist_clear ( clist_info);
	gtk_clist_clear ( clist_result);
	gtk_ctree_remove_node ( ctree, root);
	root = NULL;
	interface_set_tree_root ( root);
	status_bar_put_messages ( window, "", "");
	gtk_clist_thaw ( clist);
	gtk_clist_thaw ( clist_result);
	gtk_clist_thaw ( clist_info);

	gtk_window_set_title ( window, PACKAGE);
	notebook_managment_refresh_categories ( window);

	return 0;
}
