/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "NoteBookCatalogCallBack.h"

#include <gtk/gtk.h>

#include <string.h> /* strcmp */

#include "../tools/GWString.h"

#include "../data/FileInfo.h"
#include "../data/DiskInfo.h"
#include "../data/DiskData.h"
#include "../data/DirData.h"
#include "../data/CatalogFileInfo.h"

#include "../res/Icons.h"

#include "../GuiManager.h"

#include "StatusBar.h"
#include "StatusBarCallBack.h"
#include "NoteBookCatalog.h"
#include "NoteBookSearch.h"
#include "NoteBookManagmentCallBack.h"
#include "MenuPopup.h"

#include "../Catalog.h"


gboolean notebook_catalog_clist_button_press_event ( GtkWidget *w, GdkEventButton *event)
{
	GtkCList *clist_info = NULL;
	GtkCTreeNode *selection = NULL, *node = NULL, *child = NULL;
	struct disk_info *disk_info = NULL;
	struct file_info *file_info = NULL;
	GdkEventButton *bev = ( GdkEventButton *)event;
	gint row,col;
	gint *n;
	GtkWindow *window = NULL;
	GtkCTree *ctree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Gets all needs references. */
	window = interface_get_main_window ( );
	ctree = interface_get_tree ( );
	root = interface_get_tree_root ( );
	clist_info = notebook_catalog_get_clist_explorer ( window);

	if ( root == NULL )
	{
		/* If no one opened catalog. */
		return FALSE;
	}

	/* Ignores clicks on scrollbars. */
	if ( event->window != GTK_CLIST ( clist_info)->clist_window )
	{
		return TRUE;
	}

	/* Gets the selected item in the tree. */
	selection = GTK_CTREE_NODE ( GTK_CLIST( ctree)->selection->data);

	/* Gets the selected item in the list. */
	gtk_clist_get_selection_info ( clist_info, event->x, event->y, &row, &col);

	/* Stops the signal during the process. */
	gtk_signal_emit_stop_by_name ( GTK_OBJECT ( clist_info), "button_press_event");

	/* Checks if it's a double click. */
	if ( event->type == GDK_2BUTTON_PRESS )
	{
		/* Si c'est un double clique */
		if ( event->button == 1 )
		{
			/* Si c'est un double click gauche */

			/* Empcher que le double-click ne gnre un signal en trop */
			gdk_event_get();

			if ( GTK_CTREE_ROW ( selection)->parent == NULL )
			{
				/* L'utilisateur est dans le catalogue : il a donc cliqu sur un disk */

				/* On rcupre les infos attaches  la ligne */
				if ( (disk_info = (struct disk_info*)gtk_clist_get_row_data( clist_info, row)) == NULL )
				{
					/* Si l'utilisateur a cliqu dans le vide */

					return TRUE;
				}
				else
				{
					/* Sinon l'utilisateur a cliqu sur un disk */

					if (!GTK_CTREE_ROW( selection)->expanded)
					{
						gtk_ctree_expand ( ctree, selection);
					}

					/* On recherche dans l'arbre ctree o se trouve le disk slectionn */
					child = GTK_CTREE_ROW ( selection)->children;
					while ( child != NULL )
					{
						if ( strcmp( disk_info_get_name ( disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, child))), disk_info_get_name ( disk_info)) == 0 )
						{
							gtk_ctree_select ( ctree, child);

							return TRUE;
						}

						child = GTK_CTREE_NODE_NEXT( child);
					}

					return TRUE;
				}
			}
			else
			{
				/* Sinon il a cliqu sur un rpertoire ou un fichier */

				/* On rcupre fes infos attaches  la ligne */
				if ( (file_info = (struct file_info*)gtk_clist_get_row_data( clist_info, row)) == NULL )
				{
					/* Si l'utilisateur a cliqu dans le vide */

					return TRUE;
				}
				else
				{
					/* Si l'utilisateur a cliqu sur un lment */

					if ( file_info_get_isdirectory ( file_info) )
					{
						/* Si c'est un rpertoire */

						if ( strcmp( file_info_get_name ( file_info), "..") == 0 )
						{
							/* Si c'est le rpertoire parent */

							child = GTK_CTREE_ROW( selection)->parent;
							if ( child != root )
							{
								gtk_ctree_select( ctree, child);
							}

							return TRUE;
						}
						else
						{
							/* Sinon c'est un rpertoire quelconque */

							if (!GTK_CTREE_ROW( selection)->expanded)
							{
								gtk_ctree_expand( ctree, selection);
							}

							/* On recherche dans l'arbre ctree o se trouve le rpertoire slectionn */
							child = GTK_CTREE_ROW( selection)->children;
							while ( child != NULL )
							{
								if ( strcmp( file_info_get_name ( dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree, child))), file_info_get_name ( file_info)) == 0 )
								{
									gtk_ctree_select( ctree, child);

									return TRUE;
								}

								child = GTK_CTREE_NODE_NEXT( child);
							}

							return TRUE;
						}
					}
					else
					{
						/* Si c'est un fichier */

						status_bar_put_msg_file ( window,
									file_info_get_name ( file_info),
									file_info_get_size ( file_info),
									file_info_get_cdate_to_gchar ( file_info),
									1);

						return TRUE;
					}
				}
			}
		}
	}
	else { if ( event->type == GDK_BUTTON_PRESS )
	{
		/* Si il y a un seul click */
		if ( event->button == 1 )
		{
			/* Si c'est un click gauche : on met  jour la barre des status*/


			if ( GTK_CTREE_ROW ( selection)->parent == NULL )
			{
				/* L'utilisateur est dans le catalogue : il a donc cliqu sur un disk */

				/* On rcupre les infos attaches  la ligne */
				if ( (disk_info = (struct disk_info*)gtk_clist_get_row_data( clist_info, row)) == NULL )
				{
					/* Si l'utilisateur a cliqu dans le vide */

					return TRUE;
				}
				else
				{
					/* Sinon l'utilisateur a cliqu sur un disk */

					/* On recherche dans l'arbre d'exploration le disk sur lequel il a cliqu */
					node = notebook_catalog_ctree_node_get_child ( GTK_CTREE_NODE( GTK_CLIST( ctree)->selection->data), disk_info_get_name ( disk_info));
					disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node));

					/* On met a jour la barre des status */
					status_bar_put_msg_disk ( window,
								disk_info_get_name ( disk_info),
								disk_info_get_type ( disk_info),
								disk_info_get_full ( disk_info),
								disk_info_get_free ( disk_info),
								0);

					return TRUE;
				}
			}
			else
			{
				/* L'utilisateur a cliqu sur un rpertoire ou un fichier */

				/* On rcupre les infos attaches  la ligne */
				if ( (file_info = (struct file_info*)gtk_clist_get_row_data( GTK_CLIST( clist_info), row)) == NULL )
				{
					/* Si l'utilisateur a cliqu dans le vide */

					return TRUE;
				}
				else
				{
					/* Si l'utilisateur a cliqu sur un lment */

					if ( file_info_get_isdirectory ( file_info) )
					{
						/* Si c'est un rpertoire */

						if ( strcmp( file_info_get_name ( file_info), "..") == 0 )
						{
							/* Si c'est le rpertoire parent */

							/* On met  jour la barre des status */
							status_bar_put_msg_file ( window,
										file_info_get_name ( file_info),
										file_info_get_size ( file_info),
										file_info_get_cdate_to_gchar ( file_info),
										1);

							return TRUE;
						}
						else
						{
							/* Si c'est un rpertoire quelconque qui est slectionn */

							/* On recherche dans l'arbre d'exploration le rpertoire sur lequel il a cliqu */
							node = notebook_catalog_ctree_node_get_child ( GTK_CTREE_NODE( GTK_CLIST( ctree)->selection->data), file_info_get_name ( file_info));
							file_info = dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node));

							/* On met  jour la barre des status */
							status_bar_put_msg_dir ( window,
										file_info_get_name ( file_info),
										file_info_get_size ( file_info),
										dir_data_get_num_file ( gtk_ctree_node_get_row_data ( ctree, node)) - 1,
										notebook_catalog_ctree_node_get_num_children ( node),
										0);

							return TRUE;
						}
					}
					else
					{
						/* Si c'est un fichier qui est slectionn */

						/* On met  jour la barre des status */
						status_bar_put_msg_file ( window,
									file_info_get_name ( file_info),
									file_info_get_size ( file_info),
									file_info_get_cdate_to_gchar ( file_info), 1);

						return TRUE;
					}
				}
			}
		}
		else { if ( event->button == 3 )
		{
			/* Si c'est un click droit : on affiche un menu popup*/


			if ( GTK_CTREE_ROW ( selection)->parent == NULL )
			{
				/* L'utilisateur est dans le catalogue : il a donc cliqu sur un disk */

				/* On rcupre les infos attaches  la ligne */
				if ( (disk_info = (struct disk_info*)gtk_clist_get_row_data( clist_info, row)) == NULL )
				{
					/* Si l'utilisateur a cliqu dans le vide */

					return TRUE;
				}
				else
				{
					/* Sinon l'utilisateur a cliqu sur un disk */

					/* On recherche dans l'arbre d'exploration le disk sur lequel l'utilisateur a cliqu */
					node = notebook_catalog_ctree_node_get_child ( GTK_CTREE_NODE( GTK_CLIST( ctree)->selection->data), disk_info_get_name ( disk_info));

					/* On affiche le menu popup */
					gtk_menu_popup ( menu_popup_disk_create ( node), NULL, NULL, NULL, NULL, bev->button, bev->time);

					return TRUE;
				}
			}
			else
			{
				/* L'utilisateur a cliqu sur un rpertoire ou un fichier */

				/* On rcupre les infos attaches  la ligne */
				if ( (file_info = (struct file_info*)gtk_clist_get_row_data( GTK_CLIST( clist_info), row)) == NULL )
				{
					/* Si l'utilisateur a cliqu dans le vide */

					return TRUE;
				}
				else
				{
					/* Sinon l'utilisateur a cliqu sur un lment */

					if ( file_info_get_isdirectory ( file_info) )
					{
						/* Si c'est un rpertoire */

						if ( strcmp( file_info_get_name ( file_info), "..") == 0 )
						{
							/* Si c'est le rpertoire parent */

							return TRUE;
						}
						else
						{
							/* Sinon c'est un rpertoire quelconque qui est slectionn */

							/* On recherche dans l'arbre d'exploration le rpertoire sur lequel l'utilisateur a cliqu */
							node = notebook_catalog_ctree_node_get_child ( GTK_CTREE_NODE( GTK_CLIST( ctree)->selection->data), file_info_get_name ( file_info));

							/* On affiche le menu popup */
							gtk_menu_popup ( menu_popup_dir_create ( node), NULL, NULL, NULL, NULL, bev->button, bev->time);

							return TRUE;
						}
					}
					else
					{
						/* Si c'est un fichier qui est slectionn */

						/* On rcupre le numro de la ligne sur lequel l'utilisateur a cliqu */
						n = (gint*)g_malloc(sizeof(gint));
						*n = row;

						/* On affiche le menu popup */
						gtk_menu_popup ( menu_popup_file_create ( n), NULL, NULL, NULL, NULL, bev->button, bev->time);

						return TRUE;
					}
				}
			}
		}}
	}}

	return FALSE;
}


gboolean notebook_catalog_ctree_button_press_event ( GtkWidget *w, GdkEventButton *event)
{
	GtkCList *clist;
	GtkCTreeNode *node;
	gint row, col;
	GdkEventButton *bev = NULL;
	GtkCTree *ctree = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	bev = ( GdkEventButton *)event;
	ctree = interface_get_tree ( );
	clist = GTK_CLIST ( w);

	/* Ignores clicks on scrollbars. */
	if ( event->window != GTK_CLIST ( clist)->clist_window )
	{
		return TRUE;
	}

	gtk_clist_get_selection_info ( GTK_CLIST ( clist), event->x, event->y, &row, &col);
	node = GTK_CTREE_NODE ( g_list_nth ( clist->row_list, row));

	/* Stops the signal during the process. */
	gtk_signal_emit_stop_by_name ( GTK_OBJECT ( ctree), "button_press_event");

	/* Checks if it's a double click. */
	if ( event->type == GDK_2BUTTON_PRESS )
	{
		/* Checks if it's a left click. */
		if ( event->button == 1 )
		{
			/* Prevents from double click to generate too many signals. */
			gdk_event_get ( );

			/* Checks if user didn't click in the empty space. */
			if ( node != NULL )
			{
				/* Opens or closes the node. */
				if ( GTK_CTREE_ROW ( node)->expanded )
				{
					gtk_ctree_collapse ( ctree, node);
				}
				else
				{
					gtk_ctree_expand ( ctree, node);
				}

				gtk_ctree_select ( ctree, node);
			}

			return TRUE;
		}
	}
	/* Checks if it's a single click. */
	else { if ( event->type == GDK_BUTTON_PRESS )
	{
		/* Checks if it's a left click. */
		if ( event->button == 1 )
		{
			if ( node != NULL )
			{
				/* If user didn't click in the empty space : select the row. */
				gtk_ctree_select ( ctree, node);
			}

			return TRUE;
		}
		/* Checks if it's a left click. */
		else { if ( event->button == 3 )
		{
			/* Displays the popup menu. */
			if ( node == NULL )
			{
				/* If user clicked in the empty space. */
				gtk_menu_popup ( menu_popup_create ( ), NULL, NULL, NULL, NULL, bev->button, bev->time);

				return TRUE;
			}
			else
			{
				/* Selects the item for more user visibility. */
				gtk_ctree_select ( ctree, node);

				if ( GTK_CTREE_ROW ( node)->parent == NULL )
				{
					/* If user clicked on a catalog. */
					gtk_menu_popup ( menu_popup_catalog_create ( node), NULL, NULL, NULL, NULL, bev->button, bev->time);

					return TRUE;
				}
				else { if ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( node)->parent)->parent == NULL )
				{
					/* If user clicked on a disk. */
					gtk_menu_popup ( menu_popup_disk_create ( node), NULL, NULL, NULL, NULL, bev->button, bev->time);

					return TRUE;
				}
				else
				{
					/* If user clicked on a folder. */
					gtk_menu_popup ( menu_popup_dir_create ( node), NULL, NULL, NULL, NULL, bev->button, bev->time);

					return TRUE;
				}}
			}

			return TRUE;
		}}
	}}

	return FALSE;
}


gboolean notebook_catalog_ctree_select_row ( GtkCTree *ctree, GtkCTreeNode *node, gint column, gpointer g)
{
	GtkCList *clist_info = NULL;
	struct disk_info *disk_info = NULL;
	gchar *info[9];
	gint i, row, compt = 0, compt_file = 0;
	long double size = 0;
	GtkCTreeNode *tmp = NULL;
	struct file_info *file_info = NULL;
	struct dir_data *dir_data = NULL;
	struct disk_data *disk_data = NULL;
	GList *files_data = NULL;
	GtkWindow *window = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	window = interface_get_main_window ( );
	root = interface_get_tree_root ( );
	clist_info = notebook_catalog_get_clist_explorer ( window);

	/* Stop the signal during the process. */
	gtk_signal_emit_stop_by_name ( GTK_OBJECT ( ctree), "tree_select_row");

	/* Freezes informations table */
	gtk_clist_freeze ( clist_info);
	gtk_clist_clear ( clist_info);


	/* Updates list */
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : updates list\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( node == root )
	{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : catalog selected\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* If a catalog is selected, must to display disk list */
		notebook_catalog_clist_explorer_set_column_title_disk ( );
		node = GTK_CTREE_ROW ( GTK_CTREE_NODE ( GTK_CLIST ( ctree)->selection->data))->children;

		while ( node != NULL )
		{
			compt++;
			disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node));
			size = size + disk_info_get_full ( disk_info);
			info[0] = g_strdup ( disk_info_get_name ( disk_info));
			info[1] = g_strdup_printf ( "%d", disk_info_get_num ( disk_info));
			info[2] = g_strdup ( disk_info_get_type ( disk_info));
			info[3] = g_strdup ( disk_info_get_volume ( disk_info));
			info[4] = octet2char ( disk_info_get_full ( disk_info));
			info[5] = octet2char ( disk_info_get_free ( disk_info));
			info[6] = g_strdup ( disk_info_get_date_to_gchar ( disk_info));
			info[7] = g_strdup ( disk_info_get_category_name ( disk_info));
			info[8] = g_strdup ( disk_info_get_description ( disk_info));

			row = gtk_clist_append ( clist_info, info);

			/* Note that doesn't give the data destroy function, because that will destroy the database. */
			gtk_clist_set_row_data_full ( clist_info, row, disk_info, NULL);
			gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, disk_info_get_pixmap ( disk_info), disk_info_get_bitmap ( disk_info));

			/* Frees memory used by info. */
			for ( i = 0; i < 9; i++)
			{
				g_free ( info[i]);
			}

			node = GTK_CTREE_ROW ( node)->sibling;
		}

		/* Displays in the status bar the informations about the selected item. */
		status_bar_put_msg_catalog ( window, ctree, root, compt, size);

		/* Updates the list of disk. */
		gtk_clist_thaw ( clist_info);

		return TRUE;
	}
	else { if ( GTK_CTREE_ROW ( node)->parent == root )
	{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : disk selected\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* If a disk is selected, must to display folders and files list */
		notebook_catalog_clist_explorer_set_column_title_file ( );
		tmp = node;

		/* Folders displaying. */
		node = GTK_CTREE_ROW( node)->children;
		while ( node != NULL )
		{
			compt++;
			file_info = dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node));
			size = size + file_info_get_size ( file_info);
			info[0] = g_strdup( file_info_get_name ( file_info));
			info[1] = g_strdup ( file_info_get_rights ( file_info));
			if ( strlen ( file_info_get_owner ( file_info)) != 0 )
			{
				info[2] = g_strdup ( file_info_get_owner ( file_info));
			}
			else
			{
				info[2] = g_strdup ( _ ( "unknown owner"));
			}
			if ( strlen ( file_info_get_group ( file_info)) != 0 )
			{
				info[3] = g_strdup ( file_info_get_group ( file_info));
			}
			else
			{
				info[3] = g_strdup ( _( "unknown group"));
			}
			info[4] = octet2char ( file_info_get_size ( file_info));
			info[5] = g_strdup_printf ( "%ld", file_info_get_inode ( file_info));
			info[6] = g_strdup ( file_info_get_cdate_to_gchar ( file_info));
			info[7] = g_strdup ( file_info_get_category_name ( file_info));
			info[8] = g_strdup ( file_info_get_description ( file_info));

			row = gtk_clist_append( clist_info, info);

			/* Note that doesn't give the data destroy function, because that will destroy the database. */
			gtk_clist_set_row_data_full ( clist_info, row, file_info, NULL);
			gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, PixFolder, MskFolder);

			/* Frees memory used by info. */
			for ( i = 0; i < 9; i++)
			{
				g_free ( info[i]);
			}

			node = GTK_CTREE_ROW ( node)->sibling;
		}

		/* Files displaying. */
		node = tmp;
		disk_data = gtk_ctree_node_get_row_data ( ctree, node);
		files_data = g_list_first ( disk_data_get_child ( disk_data));
		files_data = g_list_next ( files_data);

		/* Remind me why I do that please?? :-( */
		if ( ( GTK_CTREE_ROW ( node)->parent) == root )
		{
			files_data = g_list_next ( files_data);
		}

		while ( files_data != g_list_first (files_data))
		{
			compt++;
			compt_file++;
			size = size + file_info_get_size ( files_data->data);
			info[0] = g_strdup ( file_info_get_name ( files_data->data));
			info[1] = g_strdup ( file_info_get_rights ( files_data->data));
			if ( strlen ( file_info_get_owner ( files_data->data)) != 0 )
			{
				info[2] = g_strdup ( file_info_get_owner ( files_data->data));
			}
			else
			{
				info[2] = g_strdup ( _ ( "unknown owner"));
			}
			if ( strlen ( file_info_get_group ( files_data->data)) != 0 )
			{
				info[3] = g_strdup ( file_info_get_group ( files_data->data));
			}
			else
			{
				info[3] = g_strdup ( _( "unknown group"));
			}
			info[4] = octet2char ( file_info_get_size ( files_data->data));
			info[5] = g_strdup_printf ( "%ld", file_info_get_inode ( files_data->data));
			info[6] = g_strdup ( file_info_get_cdate_to_gchar ( files_data->data));
			info[7] = g_strdup ( file_info_get_category_name ( files_data->data));
			info[8] = g_strdup ( file_info_get_description ( files_data->data));

			if ( strncmp ( info[0], "..", 2) == 0 )
			{
				row = gtk_clist_prepend ( clist_info, info);

				/* Note that doesn't give the data destroy function, because that will destroy the database. */
				gtk_clist_set_row_data_full ( clist_info, row, (struct file_info*)files_data->data, NULL);
				gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, PixParentFolder, MskParentFolder);
			}
			else
			{
				row = gtk_clist_append ( clist_info, info);

				/* Note that doesn't give the data destroy function, because that will destroy the database. */
				gtk_clist_set_row_data_full ( clist_info, row, (struct file_info*)files_data->data, NULL);
				gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, PixFile, MskFile);
			}

			files_data = g_list_next ( files_data);

			/* Frees memory used by info. */
			for ( i = 0; i < 9; i++)
			{
				g_free ( info[i]);
			}
		}

		/* Displays in the status bar the informations about the selected item. */
		status_bar_put_msg_disk ( window,
					disk_info_get_name ( disk_data_get_info ( disk_data)),
					disk_info_get_type ( disk_data_get_info ( disk_data)),
					size,
					0,
					compt - 1);

		/* Updates the list of folders and files. */
		gtk_clist_thaw( clist_info);

		return TRUE;
	}
	else
	{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : folder selected\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		/* If a folder is selected, must to display folders and files list */
		notebook_catalog_clist_explorer_set_column_title_file ( );
		tmp = node;

		/* Displays folders. */
		node = GTK_CTREE_ROW ( node)->children;
		while ( node != NULL )
		{
			compt++;
			file_info = dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree, node));
			size = size + file_info_get_size ( file_info);
			info[0] = g_strdup( file_info_get_name ( file_info));
			info[1] = g_strdup ( file_info_get_rights ( file_info));
			if ( strlen ( file_info_get_owner ( file_info)) != 0 )
			{
				info[2] = g_strdup ( file_info_get_owner ( file_info));
			}
			else
			{
				info[2] = g_strdup ( _ ( "unknown owner"));
			}
			if ( strlen ( file_info_get_group ( file_info)) != 0 )
			{
				info[3] = g_strdup ( file_info_get_group ( file_info));
			}
			else
			{
				info[3] = g_strdup ( _( "unknown group"));
			}
			info[4] = octet2char ( file_info_get_size ( file_info));
			info[5] = g_strdup_printf ( "%ld", file_info_get_inode ( file_info));
			info[6] = g_strdup ( file_info_get_cdate_to_gchar ( file_info));
			info[7] = g_strdup ( file_info_get_category_name ( file_info));
			info[8] = g_strdup ( file_info_get_description ( file_info));

			row = gtk_clist_append ( clist_info, info);

			/* Note that doesn't give the data destroy function, because that will destroy the database. */
 			gtk_clist_set_row_data_full ( clist_info, row, file_info, NULL);
			gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, PixFolder, MskFolder);

			/* Frees memory used by info. */
			for ( i = 0; i < 9; i++)
			{
				g_free ( info[i]);
			}

			node = GTK_CTREE_ROW ( node)->sibling;
		}

		/* Displays files */
		node = tmp;
		dir_data = gtk_ctree_node_get_row_data ( ctree, node);
		files_data = g_list_first ( dir_data_get_child ( dir_data));
		files_data = g_list_next ( files_data);

		/* Remind me why I do that please?? :-( */
		if ( ( GTK_CTREE_ROW ( node)->parent) == root )
		{
			files_data = g_list_next ( files_data);
		}

		while ( files_data != g_list_first ( files_data))
		{
			compt++;
			compt_file++;
			size = size + file_info_get_size ( files_data->data);
			info[0] = g_strdup ( file_info_get_name ( files_data->data));
			info[1] = g_strdup ( file_info_get_rights ( files_data->data));
			if ( strlen ( file_info_get_owner ( files_data->data)) != 0 )
			{
				info[2] = g_strdup ( file_info_get_owner ( files_data->data));
			}
			else
			{
				info[2] = g_strdup ( _ ( "unknown owner"));
			}
			if ( strlen ( file_info_get_group ( files_data->data)) != 0 )
			{
				info[3] = g_strdup ( file_info_get_group ( files_data->data));
			}
			else
			{
				info[3] = g_strdup ( _( "unknown group"));
			}
			info[5] = g_strdup_printf ( "%ld", file_info_get_inode ( files_data->data));
			info[4] = octet2char ( file_info_get_size ( files_data->data));
			info[6] = g_strdup ( file_info_get_cdate_to_gchar ( files_data->data));
			info[7] = g_strdup ( file_info_get_category_name ( files_data->data));
			info[8] = g_strdup ( file_info_get_description ( files_data->data));

			if ( strncmp ( info[0], "..", 2) == 0 )
			{
				row = gtk_clist_prepend ( clist_info, info);

				/* Note that doesn't give the data destroy function, because that will destroy the database. */
				gtk_clist_set_row_data_full ( clist_info, row, (struct file_info*)files_data->data, NULL);
				gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, PixParentFolder, MskParentFolder);
			}
			else
			{
				row = gtk_clist_append ( clist_info, info);

				/* Note that doesn't give the data destroy function, because that will destroy the database. */
				gtk_clist_set_row_data_full ( clist_info, row, (struct file_info*)files_data->data, NULL);
				gtk_clist_set_pixtext ( clist_info, row, 0, info[0], 5, PixFile, MskFile);
			}

			files_data = g_list_next ( files_data);

			/* Frees memory used by info. */
			for ( i = 0; i < 9; i++)
			{
				g_free ( info[i]);
			}
		}

		/* Displays in the status bar the informations about the selected item. */
		status_bar_put_msg_dir ( window,
					file_info_get_name ( dir_data_get_info ( dir_data)),
					size,
					compt_file - 1,
					compt - compt_file,
					compt - 1);

		/* Updates the list of folders and files. */
		gtk_clist_thaw ( clist_info);

		return TRUE;
	}}

	return FALSE;
}


gulong notebook_catalog_ctree_node_get_folder_size ( GtkCTree *ctree, GtkCTreeNode *node)
{
	GtkCTreeNode *child = NULL;
	struct dir_data *dir_data = NULL;
	GList *files = NULL;
	gulong size = 0;
	gulong result = 0;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (ctree != NULL) && (node != NULL ) )
	{
		root = interface_get_tree_root ( );

		/* Lists all subfolders. */
		child = GTK_CTREE_ROW ( node)->children;

		while ( child != NULL )
		{
			/* Recursive call of the function notebook_catalog_ctree_node_get_folder_size. */
			size = size + notebook_catalog_ctree_node_get_folder_size ( ctree, child);
		}

		/* Gets the size of all files which are in the folder. */
		dir_data = gtk_ctree_node_get_row_data ( ctree, node);
		files = g_list_first ( dir_data_get_child ( dir_data));
		files = g_list_next ( files);

		if ( (GTK_CTREE_ROW( node)->parent) == root )
		{
			files = g_list_next ( files);
		}

		while ( files != g_list_first ( files))
		{
			size = size + file_info_get_size ( files->data);
			files = g_list_next ( files);
		}

		result = size;
	}

	return result;
}
