/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "NoteBookManagment.h"

#include <gtk/gtk.h>
#include <string.h> /* strcmp */
#include <stdio.h> /* perror */
#include <sys/types.h>

#include "../tools/GWString.h"

#include "../data/VirtualFileSystemStats.h"

#include "../GuiManager.h"
#include "../Supermount.h"

#include "ComboBoxCategories.h"
#include "NoteBookManagmentCallBack.h"


/*! @define	GW_REF_NOTEBOOK_MANAGMENT_TABBED_PANE_LABEL	The pabbed pane title */
#define GW_REF_NOTEBOOK_MANAGMENT_TABBED_PANE_LABEL "gw_ref_notebook_managment_tabbed_pane_label"

#define REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_ENTRY_DEVICE_DIR "ref_gw_gui_notebook_managment_combo_box_entry_device_dir"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NAME "ref_gw_gui_notebook_managment_entry_device_name"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NUMBER_ARCHIVE "ref_gw_gui_notebook_managment_entry_device_number_archive"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_ENTRY_DEVICE_CATEGORY "ref_gw_gui_notebook_managment_combo_box_entry_device_category"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_DISPLAY_INFO "ref_gw_gui_notebook_managment_check_display_info"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_AUTO_MOUNT "ref_gw_gui_notebook_managment_check_auto_mount"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EJECT "ref_gw_gui_notebook_managment_check_eject"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_CATEGORY_RECURSIVE "ref_gw_gui_notebook_managment_check_category_recursive"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EXPLORE_ARCHIVE "ref_gw_gui_notebook_managment_check_explore_archive"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_MAKE_DESCRIPTION "ref_gw_gui_notebook_managment_check_make_description"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_DEVICE "ref_gw_gui_notebook_managment_combo_box_device"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_CATEGORY "ref_gw_gui_notebook_managment_combo_box_category"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_INFO "ref_gw_gui_notebook_managment_label_display_info_info"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_TYPE "ref_gw_gui_notebook_managment_label_display_info_type"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_MOUNTED "ref_gw_gui_notebook_managment_label_display_info_mounted"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_VOLUME "ref_gw_gui_notebook_managment_label_display_info_volume"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_SERIAL_NUMBER "ref_gw_gui_notebook_managment_label_display_info_serial_number"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_CAPACITY "ref_gw_gui_notebook_managment_label_display_info_capacity"
#define REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_FREE_SPACE "ref_gw_gui_notebook_managment_label_display_info_free_space"


GtkWidget * notebook_managment_create ( GtkWindow * window)
{
	GtkWidget *vb_tabbed_pane = NULL;
	GtkWidget *hb_control;
	GtkWidget *frm_new_disk_control;
	GtkWidget *vb_new_disk_control;
	GtkWidget *hb_disk_selection;
	GtkWidget *lbl_disk_selection;
	GtkWidget *cmb_disk_selection;
	GtkWidget *entry_cmb_disk_selection;
	GtkWidget *hseparator;
	GtkWidget *hb_number_archive;
	GtkWidget *lbl_number_archive;
	GtkWidget *entry_number_archive;
	GtkWidget *hb_disk_name;
	GtkWidget *lbl_disk_name;
	GtkWidget *entry_disk_name;
	GtkHBox *cmb_categories;
	GtkWidget *hb_device_file;
	GtkWidget *lbl_device_file;
	GtkWidget *hb_device_type;
	GtkWidget *lbl_device_type;
	GtkWidget *hb_device_ismounted;
	GtkWidget *lbl_device_ismounted;
	GtkWidget *hb_device_volume_name;
	GtkWidget *lbl_device_volume_name;
	GtkWidget *hb_device_serial_number;
	GtkWidget *lbl_device_serial_number;
	GtkWidget *hb_device_capacity;
	GtkWidget *lbl_device_capacity;
	GtkWidget *hb_device_free_space;
	GtkWidget *lbl_device_free_space;
	GtkWidget *frm_options;
	GtkWidget *vb_options;
	GtkWidget *chk_display_device_info;
	GtkWidget *chk_automount_device;
	GtkWidget *chk_explore_archive_files;
	GtkWidget *chk_auto_eject_device;
	GtkWidget *chk_category_recursively;
	GtkWidget *chk_auto_make_description;
	GtkWidget *bt_start_device_scan;
	GtkTooltips *tooltips_group;
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( window != NULL )
	{
		tree = interface_get_tree ( );
		root = interface_get_tree_root ( );

		/* Inits tooltips group */
		tooltips_group = gtk_tooltips_new ( );
	
		/* Vertical box of the managment tabbed pane. */
		vb_tabbed_pane = gtk_vbox_new ( FALSE, 0);
	
		/* Horizontal box for controls (text field, checks buttons, ...). */
		hb_control = gtk_hbox_new ( TRUE, 0);
		gtk_box_pack_start ( GTK_BOX ( vb_tabbed_pane), hb_control, TRUE, TRUE, 0);
	
		/* Frame of new disk control */
		frm_new_disk_control = gtk_frame_new ( _( "New disk"));
		gtk_container_set_border_width ( GTK_CONTAINER ( frm_new_disk_control), 10);
		gtk_box_pack_start ( GTK_BOX ( hb_control), frm_new_disk_control, FALSE, TRUE, 0);
	
		/* Vertical box for new disk control */
		vb_new_disk_control = gtk_vbox_new ( FALSE, 0);
		gtk_container_add ( GTK_CONTAINER ( frm_new_disk_control), vb_new_disk_control);
	
		/* Horizontal box for disk selection */
		hb_disk_selection = gtk_hbox_new ( FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_disk_selection), 10);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_disk_selection, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_disk_selection, _("Select the disk that will be added to your catalog."), "TipsSelectDisk");
	
		/* Label for disk selection */
		lbl_disk_selection = gtk_label_new ( _( "Select disk : "));
		gtk_box_pack_start ( GTK_BOX ( hb_disk_selection), lbl_disk_selection, FALSE, FALSE, 0);
	
		/* ComboBox to select disk */
		cmb_disk_selection = gtk_combo_new ( );
		gtk_widget_ref ( cmb_disk_selection);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_DEVICE, cmb_disk_selection, (GtkDestroyNotify) gtk_widget_unref);
		gtk_combo_set_use_arrows_always ( GTK_COMBO ( cmb_disk_selection), TRUE);
		gtk_entry_set_editable ( GTK_ENTRY( GTK_COMBO ( cmb_disk_selection)->entry), FALSE);
		gtk_signal_connect ( GTK_OBJECT ( GTK_ENTRY ( GTK_COMBO ( cmb_disk_selection)->entry)), "changed", GTK_SIGNAL_FUNC ( notebook_managment_select_device), GTK_WINDOW ( window));
		gtk_box_pack_start ( GTK_BOX ( hb_disk_selection), cmb_disk_selection, TRUE, TRUE, 0);
	
		/* Entry of ComboBox to select disk */
		entry_cmb_disk_selection = GTK_COMBO ( cmb_disk_selection)->entry;
		gtk_widget_ref ( entry_cmb_disk_selection);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_ENTRY_DEVICE_DIR, entry_cmb_disk_selection, (GtkDestroyNotify) gtk_widget_unref);
	
		/* Horizontal separator */
		hseparator = gtk_hseparator_new ( );
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hseparator, TRUE, TRUE, 0);
	
		/* Horizontal box for number archive */
		hb_number_archive = gtk_hbox_new ( FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_number_archive), 10);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_number_archive, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_number_archive, _( "Enter the number of the new disk. This number can help you with the history of your catalog."), "TipsEnterArchiveNumber");
	
		/* Label for number archive */
		lbl_number_archive = gtk_label_new ( _( "Archive N# : "));
		gtk_box_pack_start ( GTK_BOX ( hb_number_archive), lbl_number_archive, FALSE, FALSE, 0);
	
		/* Entry for number archive */
		entry_number_archive = gtk_entry_new ( );
		gtk_widget_ref ( entry_number_archive);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NUMBER_ARCHIVE, entry_number_archive, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_number_archive), entry_number_archive, TRUE, TRUE, 0);
	
		/* Horizontal box for disk name */
		hb_disk_name = gtk_hbox_new ( FALSE, 0);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_disk_name, TRUE, TRUE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_disk_name), 10);
		gtk_tooltips_set_tip ( tooltips_group, hb_disk_name, _( "Enter the name of the new disk. It's the name will be appear in your catalog."), "TipsEnterDiskName");
	
		/* Label for disk name */
		lbl_disk_name = gtk_label_new ( _( "Name : "));
		gtk_box_pack_start ( GTK_BOX ( hb_disk_name), lbl_disk_name, FALSE, FALSE, 0);
	
		/* Entry for disk name */
		entry_disk_name = gtk_entry_new ( );
		gtk_widget_ref ( entry_disk_name);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NAME, entry_disk_name, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_disk_name), entry_disk_name, TRUE, TRUE, 0);
	
		/* Horizontal separator */
		hseparator = gtk_hseparator_new ( );
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hseparator, TRUE, TRUE, 0);
	
		/* ComboBox for category selection */
		cmb_categories = combo_box_categories_create ( window, _( "Category : "), gtk_ctree_node_get_row_data ( tree, root));
		gtk_widget_ref ( GTK_WIDGET ( cmb_categories));
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_CATEGORY, GTK_WIDGET ( cmb_categories), (GtkDestroyNotify) gtk_widget_unref);
		combo_box_categories_load ( cmb_categories);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), GTK_WIDGET ( cmb_categories), TRUE, TRUE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( cmb_categories), 10);
		gtk_tooltips_set_tip ( tooltips_group, GTK_WIDGET ( cmb_categories), _( "Select the category to apply to the disk which will be added (even if the option is checked)."), "TipSelectCategory");
	
		/* Horizontal separator */
		hseparator = gtk_hseparator_new ( );
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hseparator, TRUE, TRUE, 0);
	
		/* Horizontal box for device file information */
		hb_device_file = gtk_hbox_new ( FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_file), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_file, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_file, _( "Give the device file of the selected disk. The device file is the one of the /etc/fstab file."), "TipsInfo");
	
		/* Device file information label */
		lbl_device_file = gtk_label_new ( _( "Info : "));
		gtk_widget_ref ( lbl_device_file);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_INFO, lbl_device_file, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_file), lbl_device_file, FALSE, FALSE, 0);
	
		/* Horizontal box for device type information */
		hb_device_type = gtk_hbox_new (FALSE, 0);
		gtk_widget_show ( hb_device_type);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_type, TRUE, TRUE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_type), 5);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_type, _( "Give the type of the selected disk. The type is the one of the /etc/fstab file."), "TipsType");
	
		/* Device type information label */
		lbl_device_type = gtk_label_new ( _( "Type : "));
		gtk_widget_ref ( lbl_device_type);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_TYPE, lbl_device_type, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_type), lbl_device_type, FALSE, FALSE, 0);
	
		/* Horizontal box for device mount state information */
		hb_device_ismounted = gtk_hbox_new ( FALSE, 0);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_ismounted, TRUE, TRUE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_ismounted), 5);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_ismounted, _( "Display if the selected disk is mounted or no."), "TipsMounted");
	
		/* Device mount state information label */
		lbl_device_ismounted = gtk_label_new ( _( "Mounted : "));
		gtk_widget_ref ( lbl_device_ismounted);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_MOUNTED, lbl_device_ismounted, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_ismounted), lbl_device_ismounted, FALSE, FALSE, 0);
	
		/* Horizontal box for device volume name information */
		hb_device_volume_name = gtk_hbox_new ( FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_volume_name), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_volume_name, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_volume_name, _( "Give the volume name of the selected disk (if is mounted)."), "TipsVolumeName");
	
		/* Device volume information label */
		lbl_device_volume_name = gtk_label_new ( _( "Volume : "));
		gtk_widget_ref ( lbl_device_volume_name);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_VOLUME, lbl_device_volume_name, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_volume_name), lbl_device_volume_name, FALSE, FALSE, 0);
	
		/* Horizontal box for device serial number information */
		hb_device_serial_number = gtk_hbox_new ( FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_serial_number), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_serial_number, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_serial_number, _( "Give the serial number of the selected disk (if is mounted). This number is a manufacture number."), "TipsSerialNumber");
	
		/* Device serial number information label */
		lbl_device_serial_number = gtk_label_new ( _( "Serial N# : "));
		gtk_widget_ref ( lbl_device_serial_number);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_SERIAL_NUMBER, lbl_device_serial_number, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_serial_number), lbl_device_serial_number, FALSE, FALSE, 0);
	
		/* Horizontal box for device capacity information */
		hb_device_capacity = gtk_hbox_new ( FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_capacity), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_capacity, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_capacity, _( "Give the full space of the selected disk (if is mounted)."), "TipsFullSpace");
	
		/* Device capacity information label */
		lbl_device_capacity = gtk_label_new ( _( "Capacity : "));
		gtk_widget_ref ( lbl_device_capacity);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_CAPACITY, lbl_device_capacity, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_capacity), lbl_device_capacity, FALSE, FALSE, 0);
	
		/* Horizontal box for device free space information */
		hb_device_free_space = gtk_hbox_new (FALSE, 0);
		gtk_container_set_border_width ( GTK_CONTAINER ( hb_device_free_space), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_new_disk_control), hb_device_free_space, TRUE, TRUE, 0);
		gtk_tooltips_set_tip ( tooltips_group, hb_device_free_space, _( "Give the free space of the selected disk (if is mounted)."), "TipsFreeSpace");
	
		/* Device free space information label */
		lbl_device_free_space = gtk_label_new ( _( "Free space : "));
		gtk_widget_ref ( lbl_device_free_space);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_FREE_SPACE, lbl_device_free_space, (GtkDestroyNotify) gtk_widget_unref);
		gtk_box_pack_start ( GTK_BOX ( hb_device_free_space), lbl_device_free_space, FALSE, FALSE, 0);
	
		/* Frame for scan options */
		frm_options = gtk_frame_new ( _( "Options"));
		gtk_container_set_border_width ( GTK_CONTAINER ( frm_options), 10);
		gtk_box_pack_start ( GTK_BOX ( hb_control), frm_options, TRUE, TRUE, 0);
	
		/* Vertical box of the frame options */
		vb_options = gtk_vbox_new ( FALSE, 0);
		gtk_container_add ( GTK_CONTAINER ( frm_options), vb_options);
	
		/* Check button for the displaying of device information */
		chk_display_device_info = gtk_check_button_new_with_label ( _( "Display info"));
		gtk_widget_ref ( chk_display_device_info);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_DISPLAY_INFO, chk_display_device_info, (GtkDestroyNotify) gtk_widget_unref);
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( chk_display_device_info), FALSE);
		gtk_signal_connect ( GTK_OBJECT ( chk_display_device_info), "clicked", GTK_SIGNAL_FUNC ( notebook_managment_refresh_info), window);
		gtk_container_set_border_width ( GTK_CONTAINER ( chk_display_device_info), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_options), chk_display_device_info, FALSE, FALSE, 0);
		gtk_tooltips_set_tip ( tooltips_group, chk_display_device_info, _( "Displays selected device informations as mount point, capacity, free space, label,..."), "TipsChkDisplayInfo");
	
		/* Check button for the automount */
		chk_automount_device = gtk_check_button_new_with_label ( _("Automount"));
		gtk_widget_ref ( chk_automount_device);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_AUTO_MOUNT, chk_automount_device, (GtkDestroyNotify) gtk_widget_unref);
	//	gtk_signal_connect ( GTK_OBJECT ( chk_automount_device), "clicked", GTK_SIGNAL_FUNC (auto_montage), window);
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( chk_automount_device), FALSE);
		gtk_container_set_border_width ( GTK_CONTAINER ( chk_automount_device), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_options), chk_automount_device, FALSE, FALSE, 0);
		gtk_tooltips_set_tip ( tooltips_group, chk_automount_device, _( "Mount the disk if it wasn't mounted. And umount after the disk even if it was mounted before the scan."), "TipsChkAutoMontage");
	
		/* Check button for the exploring of archive files */
		chk_explore_archive_files = gtk_check_button_new_with_label ( _( "Explore archive files (.tar, .gz, .zip, ...)"));
		gtk_widget_ref ( chk_explore_archive_files);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EXPLORE_ARCHIVE, chk_explore_archive_files, (GtkDestroyNotify) gtk_widget_unref);
	//	gtk_signal_connect ( GTK_OBJECT ( chk_explore_archive_files), "clicked", GTK_SIGNAL_FUNC (explorer_archive), window);
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( chk_explore_archive_files), FALSE);
		gtk_container_set_border_width ( GTK_CONTAINER ( chk_explore_archive_files), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_options), chk_explore_archive_files, FALSE, FALSE, 0);
		gtk_tooltips_set_tip ( tooltips_group, chk_explore_archive_files, _( "Not yet implemented!!\nExplore all archives, as it was a directory. Archives are tarred or gzipped files."), "TipsChkExplorerArchive");
	
		/* Check button for the auto eject */
		chk_auto_eject_device = gtk_check_button_new_with_label ( _( "Eject disk if possible"));
		gtk_widget_ref ( chk_auto_eject_device);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EJECT, chk_auto_eject_device, (GtkDestroyNotify) gtk_widget_unref);
	//	gtk_signal_connect ( GTK_OBJECT ( chk_auto_eject_device), "clicked", GTK_SIGNAL_FUNC (ejecter_media), window);
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( chk_auto_eject_device), TRUE);
		gtk_container_set_border_width ( GTK_CONTAINER ( chk_auto_eject_device), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_options), chk_auto_eject_device, FALSE, FALSE, 0);
		gtk_tooltips_set_tip ( tooltips_group, chk_auto_eject_device, _( "Eject the disk if possible (CD-ROM...), even if the tray was closed.  This is enable only if automount is enable!!"), "TipsChkEjecter");
	
		/* Check button for setting category recursively */
		chk_category_recursively = gtk_check_button_new_with_label ( _( "Sets category recursive"));
		gtk_widget_ref ( chk_category_recursively);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_CATEGORY_RECURSIVE, chk_category_recursively, (GtkDestroyNotify) gtk_widget_unref);
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( chk_category_recursively), TRUE);
		gtk_container_set_border_width ( GTK_CONTAINER ( chk_category_recursively), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_options), chk_category_recursively, FALSE, FALSE, 0);
		gtk_tooltips_set_tip ( tooltips_group, chk_category_recursively, _( "Apply selected category to all items in selected disk (disk, folders, files)."), "TipsChkCategoryRecursive");
	
		/* Check button for the auto make description */
		chk_auto_make_description = gtk_check_button_new_with_label ( _( "Auto make description"));
		gtk_widget_ref ( chk_auto_make_description);
		gtk_object_set_data_full ( GTK_OBJECT ( window), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_MAKE_DESCRIPTION, chk_auto_make_description, (GtkDestroyNotify) gtk_widget_unref);
		gtk_container_set_border_width ( GTK_CONTAINER ( chk_auto_make_description), 5);
		gtk_box_pack_start ( GTK_BOX ( vb_options), chk_auto_make_description, FALSE, FALSE, 0);
		gtk_tooltips_set_tip ( tooltips_group, chk_auto_make_description, _( "Not yet implemented!!\nMake auto description of file (file format description, image size for xmp or jpg, etc...)."), "TipsChkMakeDescription");
	
		/* The scan button to start device scanning */
		bt_start_device_scan = gtk_button_new_with_label ( _( "Scan disk"));
		gtk_signal_connect ( GTK_OBJECT ( bt_start_device_scan), "clicked", GTK_SIGNAL_FUNC ( notebook_managment_scan_click), window);
		gtk_container_set_border_width ( GTK_CONTAINER ( bt_start_device_scan), 5);
		gtk_box_pack_start ( GTK_BOX (vb_tabbed_pane), bt_start_device_scan, FALSE, FALSE, 0);
	
		/* Loads the list of devices */
		notebook_managment_load_device_list ( window);
		combo_box_categories_load ( cmb_categories);
	}

	return vb_tabbed_pane;
}


GtkWidget * notebook_managment_create_and_add ( GtkWindow *window, GtkNotebook *parent)
{
	GtkWidget *tabbed_pane = NULL;
	GtkWidget *managment_tabbed_pane_label;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( window != NULL )
	{
		tabbed_pane = notebook_managment_create ( window);

		if ( (parent != NULL) && (tabbed_pane != NULL) )
		{
			gtk_container_add ( GTK_CONTAINER ( parent), tabbed_pane);

			/* Text label of the managment tabbed pane. */
			managment_tabbed_pane_label = gtk_label_new ( _( "Management"));

			/* Store reference to the title of tabbed pane */
			gtk_widget_ref ( managment_tabbed_pane_label);
			gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_MANAGMENT_TABBED_PANE_LABEL, managment_tabbed_pane_label,(GtkDestroyNotify) gtk_widget_unref);
			gtk_notebook_set_tab_label ( parent, gtk_notebook_get_nth_page ( parent, 2), managment_tabbed_pane_label);
		}
	}

	return tabbed_pane;
}


gint notebook_managment_set_device_dir ( GtkWindow *w, gchar *device_dir)
{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return -1;
}


gint notebook_managment_set_device_name ( GtkWindow *w, gchar *device_name)
{
	/* Sets the device name.
	   Returns 0 when it's done.
	   Returns -1 when an error occured. */
	GtkEntry *ent = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL)
	{
		return -1;
	}

	if ( device_name == NULL )
	{
		return -1;
	}

	if ( (ent = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NAME)) == NULL)
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_entry_set_text ( ent, device_name);

	return 0;
}


gint notebook_managment_set_device_number ( GtkWindow *w, gchar *device_number)
{
	/* Sets the device number.
	   Returns 0 when it's done.
	   Returns -1 when an error occured. */
	GtkEntry *ent = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL)
	{
		return -1;
	}

	if ( device_number == NULL )
	{
		return -1;
	}

	if ( (ent = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NUMBER_ARCHIVE)) == NULL)
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_entry_set_text ( ent, device_number);

	return 0;
}


gint notebook_managment_set_device_category ( GtkWindow *w, gchar *device_category)
{
	GtkHBox *cmb;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( device_category == NULL )
	{
		return -1;
	}

	if ( (cmb = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_ENTRY_DEVICE_CATEGORY)) == NULL )
	{
		return -1;
	}

	combo_box_categories_set_selected_category_name ( cmb, device_category);

	return 0;
}


gint notebook_managment_set_option_display_info ( GtkWindow *w, gboolean b)
{
	/* Sets if option display info is activated.
	   Returns O when it's done.
	   Returns -1 when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_DISPLAY_INFO)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_toggle_button_set_active ( chk, TRUE);

	return 0;
}


gint notebook_managment_set_option_automount ( GtkWindow *w, gboolean b)
{
	/* Sets if option automount is activated.
	   Returns O when it's done.
	   Returns -1 when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_AUTO_MOUNT)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_toggle_button_set_active ( chk, TRUE);

	return 0;
}


gint notebook_managment_set_option_eject ( GtkWindow *w, gboolean b)
{
	/* Sets if option eject CD-Rom is activated.
	   Returns O when it's done.
	   Returns -1 when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EJECT)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_toggle_button_set_active ( chk, TRUE);

	return 0;
}


gint notebook_managment_set_option_category_recursive ( GtkWindow *w, gboolean b)
{
	/* Sets if option sets category recursive is activated.
	   Returns O when it's done.
	   Returns -1 when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_CATEGORY_RECURSIVE)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_toggle_button_set_active ( chk, TRUE);

	return 0;
}


gint notebook_managment_set_option_explore_archive ( GtkWindow *w, gboolean b)
{
	/* Sets if option explore archive is activated.
	   Returns O when it's done.
	   Returns -1 when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EXPLORE_ARCHIVE)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_toggle_button_set_active ( chk, TRUE);

	return 0;
}


gint notebook_managment_set_option_make_description ( GtkWindow *w, gboolean b)
{
	/* Sets if option auto make description is activated.
	   Returns O when it's done.
	   Returns -1 when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return -1;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_MAKE_DESCRIPTION)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return -1;
	}

	gtk_toggle_button_set_active ( chk, TRUE);

	return 0;
}


gchar * notebook_managment_get_device_dir ( GtkWindow *w)
{
	/* Gets the device directory.
	   Return NULL when an error occured. */
	GtkEntry *ent = NULL;


	if ( (ent = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_ENTRY_DEVICE_DIR)) == NULL)
	{
		perror ( "gtk_object_get_data");
		return NULL;
	}

#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return gtk_entry_get_text ( ent);
}


gchar * notebook_managment_get_device_name ( GtkWindow *w)
{
	/* Gets the device name.
	   Returns NULL when an error occured. */
	GtkEntry *ent = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (ent = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NAME)) == NULL)
	{
		perror ( "gtk_object_get_data");
		return NULL;
	}

	return gtk_entry_get_text ( ent);
}


gchar * notebook_managment_get_device_number ( GtkWindow *w)
{
	/* Gets the device number archive.
	   Returns NULL when an error occured. */
	GtkEntry *ent = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (ent = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_ENTRY_DEVICE_NUMBER_ARCHIVE)) == NULL)
	{
		perror ( "gtk_object_get_data");
		return NULL;
	}

	return gtk_entry_get_text ( ent);
}


gchar * notebook_managment_get_device_category ( GtkWindow *w)
{
	GtkEntry *ent = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( (ent = gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_ENTRY_DEVICE_CATEGORY)) == NULL)
	{
		perror ( "gtk_object_get_data");
		return NULL;
	}

	return gtk_entry_get_text ( ent);
}


gboolean notebook_managment_get_option_display_info ( GtkWindow *w)
{
	/* Gets if option display info is activated.
	   Returns FALSE when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_DISPLAY_INFO)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return FALSE;
	}

	return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
}


gboolean notebook_managment_get_option_automount ( GtkWindow *w)
{
	/* Gets if option automount is activated.
	   Returns FALSE when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_AUTO_MOUNT)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return FALSE;
	}

	return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
}


gboolean notebook_managment_get_option_eject ( GtkWindow *w)
{
	/* Gets if option eject CD-Rom is activated.
	   Returns FALSE when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EJECT)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return FALSE;
	}

	return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
}


gboolean notebook_managment_get_option_category_recursive ( GtkWindow *w)
{
	/* Gets if option sets category recursive is activated.
	   Returns FALSE when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_CATEGORY_RECURSIVE)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return FALSE;
	}

	return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
}


gboolean notebook_managment_get_option_explore_archive ( GtkWindow *w)
{
	/* Gets if option explore archive is activated.
	   Returns FALSE when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_EXPLORE_ARCHIVE)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return FALSE;
	}

	return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
}


gboolean notebook_managment_get_option_make_description ( GtkWindow *w)
{
	/* Gets if option auto make description is activated.
	   Returns FALSE when an error occured. */
	GtkToggleButton *chk = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return FALSE;
	}

	if ( (chk = gtk_object_get_data (GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_CHECK_MAKE_DESCRIPTION)) == NULL )
	{
		perror ( "gtk_object_get_data");
		return FALSE;
	}

	return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
}


GtkHBox * notebook_managment_get_combo_box_categories ( GtkWindow *w)
{
#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w == NULL )
	{
		return NULL;
	}

	return GTK_HBOX ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_CATEGORY));
}


GtkCombo * notebook_managment_get_combo_box_device ( GtkWindow *w)
{
	GtkCombo *cmb_device = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		cmb_device = GTK_COMBO ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_COMBO_BOX_DEVICE));
	}

	return cmb_device;
}


GtkLabel * notebook_managment_get_label_device_info ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_INFO));
	}

	return label;
}


GtkLabel * notebook_managment_get_label_device_type ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_TYPE));
	}

	return label;
}


GtkLabel * notebook_managment_get_label_device_mounted ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_MOUNTED));
	}

	return label;
}


GtkLabel * notebook_managment_get_label_device_volume ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_VOLUME));
	}

	return label;
}


GtkLabel * notebook_managment_get_label_device_serial_number ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_SERIAL_NUMBER));
	}

	return label;
}


GtkLabel * notebook_managment_get_label_device_capacity ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_CAPACITY));
	}

	return label;
}


GtkLabel * notebook_managment_get_label_device_free_space ( GtkWindow *w)
{
	GtkLabel *label = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		label = GTK_LABEL ( gtk_object_get_data ( GTK_OBJECT ( w), REF_GW_GUI_NOTEBOOK_MANAGMENT_LABEL_DISPLAY_INFO_FREE_SPACE));
	}

	return label;
}
