/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "NoteBookManagmentCallBack.h"

#include <gtk/gtk.h>
#include <string.h> /* strcmp */
/* #include <pthread.h> Not used at time */
#include <sys/types.h>

#include "../data/VirtualFileSystemStats.h"
#include "../data/ScanDiskInfo.h"

#include "../tools/GWString.h"

#include "../GuiManager.h"
#include "../DeviceManager.h"
#include "../Supermount.h"
#include "../CatalogStats.h"
#include "../Catalog.h"

#include "../res/Icons.h"

#include "ComboBoxCategories.h"
#include "NoteBookManagment.h"
#include "CaptureBox.h"
#include "ProgressBarBox.h"
#include "MsgBox.h"


/*! @define	GW_GUI_NOTEBOOK_MANAGMENT_CALLBACK_DEFAULT_DEVICE_NAME	The default device mount point to select */
#define GW_GUI_NOTEBOOK_MANAGMENT_CALLBACK_DEFAULT_DEVICE_NAME "/mnt/cdrom"


gint notebook_managment_load_device_list ( GtkWindow *w)
{
	GList *devices = NULL, *tmp = NULL;
	GtkCombo *cmb = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		devices = Charger_media ( devices);

		if ( devices != NULL )
		{
			cmb = notebook_managment_get_combo_box_device ( w);

			if ( cmb != NULL )
			{
				gtk_combo_set_popdown_strings ( cmb, devices);

				/* Selects default device : mnt/cdrom */
				if ( (tmp = g_list_find_custom ( devices, GW_GUI_NOTEBOOK_MANAGMENT_CALLBACK_DEFAULT_DEVICE_NAME, (GCompareFunc)strcmp)) == NULL)
				{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
					g_print ( "*** GW - %s (%d) :: %s() : select default device = %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, GW_GUI_NOTEBOOK_MANAGMENT_CALLBACK_DEFAULT_DEVICE_NAME);
#endif

					tmp = g_list_first ( devices);
				}
				else
				{
				}

				gtk_entry_set_text ( GTK_ENTRY ( cmb->entry), tmp->data);

				if ( devices != NULL )
				{
					g_list_foreach ( devices, (GFunc)g_free, NULL);
					g_list_free ( devices);
				}

				result = 0;
			}
		}
	}

	return result;
}


gboolean notebook_managment_scan_click ( GtkWidget *bt, GtkWindow *window)
{
	GtkWindow *msg = NULL;
	gint num_archive;
	gchar *name, *disk_name, *dir, *num;
	struct vfs_stats *vfs = NULL;
	struct category *category = NULL;
	gboolean has_been_mounted = FALSE;
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;
	struct scan_disk_info *scan_disk = NULL;
	GtkWindow *progress_bar = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( window != NULL )
	{
		tree = interface_get_tree ( );
		root = interface_get_tree_root ( );

		dir = g_strdup ( notebook_managment_get_device_dir ( window));
		name = g_strdup ( notebook_managment_get_device_name ( window));
		num = g_strdup ( notebook_managment_get_device_number ( window));

		vfs = vfs_stats_new ( );

		if ( vfs != NULL )
		{
			vfs_stats_load ( vfs, dir);

#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
			g_print ( "*** GW - %s (%d) :: %s() : checking if device can be mounted\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

			/* Checks if the device can be mounted */
			if ( (vfs_stats_is_mounted ( vfs) == FALSE) && (notebook_managment_get_option_automount ( window) == FALSE) )
			{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
				g_print ( "*** GW - %s (%d) :: %s() : device is not mounted\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

				msg = msg_box_create ( window, _( "Add disk"), _( "The disk hasn't been added : there are not mounted device"));

				g_free ( dir);
				g_free ( name);
				g_free ( num);

				result = FALSE;
			}
			else { if ( vfs_stats_can_mounted ( vfs) == FALSE )
			{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
				g_print ( "*** GW - %s (%d) :: %s() : cannot mount device\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

				msg = msg_box_create ( window, _( "Add disk"), _( "The disk hasn't been added : cannot mount device"));

				g_free ( dir);
				g_free ( name);
				g_free ( num);

				result = FALSE;
			}
			else
			{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
				g_print ( "*** GW - %s (%d) :: %s() : device is mounted or can be mounted\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

				/* Puts a mapping on the disk name */
				disk_name = disk_get_name ( name);

				/* Gets archive number */
				num_archive = disk_get_num_archive ( num);

				/* Checkes if the catalog is opened */
				if ( root == NULL )
				{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
					g_print ( "*** GW - %s (%d) :: %s() : no catalog is opened\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

					/* If no catalog is opened */
					capture_box_create ( window, _("Add new disk"), _("Enter catalog name"), _("New catalog"), GTK_SIGNAL_FUNC ( capture_box_catalog_name_ok));

					result = FALSE;
				}
				else
				{
					/* If a catalog is opened */
					if ( notebook_managment_get_option_category_recursive ( window) == TRUE )
					{
						category = combo_box_categories_get_selected_category ( notebook_managment_get_combo_box_categories ( window));
					}
					else
					{
						category = catalog_file_info_get_category ( combo_box_categories_get_catalog_info ( notebook_managment_get_combo_box_categories ( window)), _( "none"));
					}

					if ( strcmp ( disk_name, "") == 0 )
					{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
						g_print ( "*** GW - %s (%d) :: %s() : dik name isn't filled\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

						/* If no disk name is filled */
						capture_box_create ( window, _("Add disk"), _("Enter disk name"), disk_get_name ( _("New disk")), GTK_SIGNAL_FUNC ( capture_box_disk_name_ok));

						result = FALSE;
					}
					else
					{
						/* Adds the disk with the disk name filled */
						if ( vfs_stats_is_mounted ( vfs) || ((has_been_mounted = vfs_stats_disk_mount ( vfs)) == TRUE))
						{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
							g_print ( "*** GW - %s (%d) :: %s() : the disk is going to be added\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

							/* If device is allready mounted or if has been mounted */
							scan_disk = scan_disk_info_new ( );
							scan_disk_info_set_vfs ( scan_disk, vfs);
							scan_disk_info_set_disk_name ( scan_disk, g_strdup ( disk_name));
							scan_disk_info_set_disk_num_archive ( scan_disk, num_archive);
							scan_disk_info_set_disk_category ( scan_disk, category);
							scan_disk_info_set_scan_disk_set_current_statment ( scan_disk, &notebook_managment_set_current_statment);

							progress_bar = progress_bar_box_create ( window, _( "Scanning"), _( "Adding disk is working"), _( "Please wait"), vfs_stats_get_total_used_space ( vfs), notebook_managment_stop_scan_disk, scan_disk);
							interface_update ( );

							scan_disk_info_set_statment_controler ( scan_disk, progress_bar);

							if ( catalog_add_disk ( scan_disk, tree, root) != -1 )
							{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
								g_print ( "*** GW - %s (%d) :: %s() : the disk has been correctly added\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

								progress_bar_box_set_text ( progress_bar, _( "The disk has been added"));
								progress_bar_box_set_ok_state ( progress_bar);

								result = TRUE;
							}
							else
							{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
								g_print ( "*** GW - %s (%d) :: %s() : the disk hasn't been correctly added\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

								progress_bar_box_set_text ( progress_bar, scan_disk_info_get_error_msg ( scan_disk));
								progress_bar_box_set_ok_state ( progress_bar);

								result = FALSE;
							}

							if ( has_been_mounted == TRUE )
							{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
								g_print ( "*** GW - %s (%d) :: %s() : try to umount device\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

								/* Umounts only if device didn't mounted before */
								vfs_stats_disk_umount ( vfs);

								/* Eject CD-Rom if option checked */
								if ( notebook_managment_get_option_eject ( window) == TRUE)
								{
									Eject_media ( dir);
								}
							}
						}
						else
						{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
							g_print ( "*** GW - %s (%d) :: %s() : device cannot be mounted, adding canceled\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

							/* The device cannot be mounted */
							msg_box_create ( window, _( "Add disk"), _( "Mount failed\n"));

							result = FALSE;
						}
					}
				}

				/* Clears all field zone */
				notebook_managment_set_device_name ( window, "");
				notebook_managment_set_device_number ( window, "");

				/* Don't free scan_disk because progress_bar_box need it, scan_disk must free
				when progress_bar_box destroyed */
				g_free ( dir);
				g_free ( name);
				g_free ( disk_name);
				g_free ( num);
				vfs_stats_free ( vfs);
			}}
		}
	}

	return result;
}


gint notebook_managment_select_device ( GtkWidget *cmb, GtkWindow *w)
{
	GtkLabel *l;
	gchar *text, *msg;
	gboolean mounted = FALSE;
	gboolean has_been_mounted = FALSE;
	struct vfs_stats *vfs = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( !notebook_managment_get_option_display_info ( w))
	{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : no display device informations\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		l = notebook_managment_get_label_device_info ( w);
		gtk_label_set_text ( l, _( "Info : "));

		l = notebook_managment_get_label_device_type ( w);
		gtk_label_set_text ( l, _( "Type : "));

		l = notebook_managment_get_label_device_mounted ( w);
		gtk_label_set_text ( l, _( "Mounted : "));

		l = notebook_managment_get_label_device_volume ( w);
		gtk_label_set_text ( l, _( "Volume : "));

		l = notebook_managment_get_label_device_serial_number ( w);
		gtk_label_set_text ( l, _( "Serial number : "));

		l = notebook_managment_get_label_device_capacity ( w);
		gtk_label_set_text ( l, _( "Capacity : "));

		l = notebook_managment_get_label_device_free_space ( w);
		gtk_label_set_text ( l, _( "Free space : "));

		result = 0;
	}
	else
	{
#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
		g_print ( "*** GW - %s (%d) :: %s() : device informations are going to be displayed\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		vfs = vfs_stats_new ( );

		if ( vfs != NULL )
		{
			if ( vfs_stats_load ( vfs, gtk_entry_get_text ( GTK_ENTRY ( cmb))) != -1 )
			{
				/* Checks if device is mounted */
				mounted = vfs_stats_is_mounted ( vfs);

				/* If device is not mounted and automount is activated */
				if ( !mounted && notebook_managment_get_option_automount ( w) )
				{
					has_been_mounted = vfs_stats_disk_mount ( vfs);
					mounted = has_been_mounted;
				}

				text = g_strconcat ( _( "Info : "), mounted?vfs_stats_get_name ( vfs):_( "no drive"), NULL);
				l = notebook_managment_get_label_device_info ( w);

				gtk_label_set_text( l, text);

				if ( mounted )
				{
					msg = g_strdup ( vfs_stats_get_type ( vfs));
				}
				else
				{
					msg = g_strdup ( _( "no drive"));
				}

				text = g_strconcat ( _( "Type : "), msg, NULL);
				l = notebook_managment_get_label_device_type ( w);
				gtk_label_set_text ( l, text);

				l = notebook_managment_get_label_device_mounted ( w);
				text = g_strconcat ( _( "Mounted : "), mounted?_( "Yes"):_( "No"), NULL);
				gtk_label_set_text ( l, text);

				if ( mounted )
				{
					if ( ( msg= g_strdup ( vfs_stats_get_label ( vfs)) ) == NULL )
					{
						msg = g_strdup ( _( "No label"));
					}
				}
				else
				{
					msg = g_strdup ( _( "no drive"));
				}

				l = notebook_managment_get_label_device_volume ( w);
				text = g_strconcat( _( "Volume name : "), mounted?msg:_("no drive"), NULL);
				g_free ( msg);
				gtk_label_set_text( l, text);

				l = notebook_managment_get_label_device_serial_number ( w);
				text = g_strconcat( _( "Serial number : "), mounted?vfs_stats_get_serial_number ( vfs):_( "no drive"), NULL);
				gtk_label_set_text( l, text);

				l = notebook_managment_get_label_device_capacity ( w);
				text = g_strconcat( _( "Capacity : "), mounted?octet2char ( vfs_stats_get_total_capacity ( vfs)):_( "no drive"), NULL);
				gtk_label_set_text( l, text);

				l = notebook_managment_get_label_device_free_space ( w);
				text = g_strconcat( _( "Free space : "), mounted?octet2char ( (long double)vfs_stats_get_total_free_space ( vfs)):_( "no drive"), NULL);
				gtk_label_set_text( l, text);

				g_free ( text);

				if ( has_been_mounted )
				{
					vfs_stats_disk_umount ( vfs);
				}

				vfs_stats_free ( vfs);

				result = 0;
			}
		}
	}

	return result;
}


gint notebook_managment_refresh_info ( GtkWidget *chk, GtkWindow *w)
{
	GtkCombo *cmb = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( (cmb = notebook_managment_get_combo_box_device ( w)) != NULL)
		{
			result = notebook_managment_select_device ( GTK_WIDGET ( GTK_OBJECT ( GTK_ENTRY ( GTK_COMBO ( cmb)->entry))), w);
		}
	}

	return result;
}


gint notebook_managment_refresh_categories ( GtkWindow *w)
{
	GtkHBox *cmb;
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		tree = interface_get_tree ( );
		root = interface_get_tree_root ( );

		cmb = notebook_managment_get_combo_box_categories ( w);

		if ( root == NULL || tree == NULL)
		{
			combo_box_categories_set_catalog_info ( cmb, NULL);
		}
		else
		{
			combo_box_categories_set_catalog_info ( cmb, gtk_ctree_node_get_row_data ( tree, root));
		}

		combo_box_categories_load ( cmb);

		result = 0;
	}

	return result;
}


gboolean notebook_managment_stop_scan_disk ( GtkObject *obj, gpointer data)
{
	struct scan_disk_info *scan_disk = NULL;
	GtkWindow *progress_bar_box = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( data != NULL )
	{
		scan_disk = (struct scan_disk_info*)data;
		progress_bar_box = GTK_WINDOW ( scan_disk_info_get_statment_controler ( scan_disk));

		if ( progress_bar_box_get_isok_state ( progress_bar_box))
		{
			gtk_widget_destroy ( GTK_WIDGET ( progress_bar_box));
			scan_disk_info_free ( scan_disk);

			result = TRUE;
		}
		else { if ( progress_bar_box_get_iscancel_state ( progress_bar_box) )
		{
			scan_disk_info_set_stop ( scan_disk, TRUE);
			progress_bar_box_set_text ( progress_bar_box, _( "the adding disk is canceled"));
			progress_bar_box_set_ok_state ( progress_bar_box);

			result = TRUE;
		}
		else
		{
			result = FALSE;
		}}
	}

	return result;
}


gint notebook_managment_set_current_statment ( gpointer w, gfloat current)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		progress_bar_box_add_value ( (GtkWindow*)w, current);
		interface_update ( );

		result =0;
	}

	return result;
}



gint capture_box_catalog_name_ok ( GtkWidget *w, gpointer data)
{
	struct catalog_file_info *file_info = NULL;
	GtkCTreeNode *node = NULL;
	gchar *text[1];
	GtkWindow *window = NULL;
	GtkCTree *tree = NULL;
	GtkCTreeNode *root = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( data != NULL )
	{
		text[0] = capture_box_get_text ( data);

		if ( strcmp ( text[0], "") != 0 )
		{
			/* If catalog name is filled : update catalog informations */
			window = interface_get_main_window ( );
			tree = interface_get_tree ( );
			root = interface_get_tree_root ( );

			if ( (window != NULL) /*&& (tree != NULL) && (root != NULL)*/ )
			{
				file_info = catalog_file_info_new ( );
				catalog_file_info_set_title ( file_info, g_strdup ( text[0]));
				catalog_file_info_set_full_name ( file_info, g_strconcat ( "./", text[0], NULL));
				catalog_file_info_set_program_maker ( file_info, g_strdup ( PACKAGE));
				catalog_file_info_set_ismodified ( file_info, TRUE);

				gtk_window_set_title ( window, g_strconcat ( PACKAGE, " - ", catalog_file_info_get_title ( file_info), NULL));

				/* Create an a new catalog node */
				text[0] = g_strdup ( catalog_file_info_get_title ( file_info));
				root = node = gtk_ctree_insert_node ( tree, NULL, NULL, text, 5, PixCatalog, MskCatalog, PixCatalog, MskCatalog, FALSE, TRUE);
				interface_set_tree_root ( root);
				gtk_ctree_node_set_row_data_full ( tree, root, file_info, (GtkDestroyNotify)catalog_file_info_free);

				notebook_managment_refresh_categories ( window);

				/* Destroys the capture box and relaunches the scan */
				gtk_widget_destroy ( data);
				notebook_managment_scan_click ( NULL,  window);

				result = 0;
			}
		}
		else
		{
			msg_box_create ( window, _( "Add disk"), _( "Please select a catalog name\n"));

			result = -1;
		}
	}

	return result;
}


gint capture_box_disk_name_ok ( GtkWidget *w, gpointer data)
{
	gchar *disk_name;
	GtkWindow *window = NULL;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( data != NULL )
	{
		window = interface_get_main_window ( );

		if ( window != NULL )
		{
			disk_name = g_strdup ( capture_box_get_text ( data));
			notebook_managment_set_device_name ( window, disk_name);

			/* Destroys the capture box and relaunches the scan */
			gtk_widget_destroy ( data);
			notebook_managment_scan_click ( NULL, window);

			result = 0;
		}
	}

	return result;
}
