/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include "NoteBookSearchCallBack.h" /* notebook_search_column_sort notebook_search_search_button_click notebook_search_select_row */

#include "../support.h"
#include "NoteBookSearch.h"


/*! @define	GW_REF_NOTEBOOK_SEARCH_TABBED_PANE_LABEL	The pabbed pane title */
#define GW_REF_NOTEBOOK_SEARCH_TABBED_PANE_LABEL "gw_ref_notebook_search_tabbed_pane_label"
/*! @define	GW_REF_NOTEBOOK_SEARCH_ENTRY		The search text entry */
#define GW_REF_NOTEBOOK_SEARCH_ENTRY "gw_notebook_search_entry"
/*! @define	GW_REF_NOTEBOOK_START_SEARCH_BUTTON		The start search button */
#define GW_REF_NOTEBOOK_START_SEARCH_BUTTON "gw_notebook_start_search_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_CHECK_BUTTON		The use case or uncase sensitive check button */
#define GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_CHECK_BUTTON "gw_ref_notebook_search_use_case_sensitive_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_TOOL_TIPS		The use case or uncase sensitive tooltips */
#define GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_TOOL_TIPS "gw_ref_notebook_search_use_case_sensitive_tool_tips"
/*! @define	GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_CHECK_BUTTON		The use or not regular expression check button */
#define GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_CHECK_BUTTON "gw_ref_notebook_search_use_regular_expression_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_TOOL_TIPS		The use or not regular expression check button */
#define GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_TOOL_TIPS "gw_ref_notebook_search_use_regular_expression_tool_tips"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_CHECK_BUTTON		The match file check button */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_CHECK_BUTTON "gw_ref_notebook_search_match_file_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_TOOL_TIPS		The match file tooltips */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_TOOL_TIPS "gw_ref_notebook_search_match_file_tool_tips"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_CHECK_BUTTON		The match folder check button */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_CHECK_BUTTON "gw_ref_notebook_search_match_folder_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_TOOL_TIPS		The match folder tooltips */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_TOOL_TIPS "gw_ref_notebook_search_match_folder_tool_tips"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_CHECK_BUTTON		The match disk check button */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_CHECK_BUTTON "gw_ref_notebook_search_match_disk_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_TOOL_TIPS		The match disk tooltips */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_TOOL_TIPS "gw_ref_notebook_search_match_disk_tool_tis"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_CHECK_BUTTON		The match category check button */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_CHECK_BUTTON "gw_ref_notebook_search_match_category_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_TOOL_TIPS		The match category tooltips */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_TOOL_TIPS "gw_ref_notebook_search_match_category_tool_tips"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_CHECK_BUTTON		The match description check button */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_CHECK_BUTTON "gw_ref_notebook_search_match_description_check_button"
/*! @define	GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_TOOL_TIPS		The match description tooltips */
#define GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_TOOL_TIPS "gw_ref_notebook_search_match_description_tool_tips"
/*! @define	GW_REF_NOTEBOOK_SEARCH_CLIST_SEARCH		The result list of search */
#define GW_REF_NOTEBOOK_SEARCH_CLIST_SEARCH "gw_ref_notebook_search_clist_search"


GtkWidget * notebook_search_create ( GtkWindow * window)
{
	GtkCList *clist_result = NULL;
	GtkWidget *vb_tabbed_pane;
	GtkWidget *hb_control;
	GtkWidget *lbl_search;
	GtkWidget *entry_search_string;
	GtkWidget *bt_search;
	GtkWidget *chk_bt_case_sensitive;
	GtkWidget *chk_bt_regex;
	GtkWidget *hb_options;
	GtkWidget *chk_match_file;
	GtkWidget *chk_match_folder;
	GtkWidget *chk_match_disk;
	GtkWidget *chk_match_category;
	GtkWidget *chk_match_description;
	GtkWidget *hb_search_result;
	GtkWidget *scrolled_result_list;
	GtkWidget *clist_search_result;
	GtkWidget *lbl_title_col;
	GtkTooltips *tooltips_group;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	/* Inits the accel group for keystroke shortcuts. */
	tooltips_group = gtk_tooltips_new ();

	/* Vertical box of the search tabbed pane. */
	vb_tabbed_pane = gtk_vbox_new ( FALSE, 0);

	/* Horizontal box for controls (checkes boxes, buttons, ...). */
	hb_control = gtk_hbox_new ( FALSE, 10);
	gtk_container_set_border_width ( GTK_CONTAINER ( hb_control), 10);
	gtk_box_pack_start ( GTK_BOX ( vb_tabbed_pane), hb_control, FALSE, TRUE, 0);

	lbl_search = gtk_label_new ( _( "Search : "));
	gtk_box_pack_start ( GTK_BOX ( hb_control), lbl_search, FALSE, FALSE, 0);

	/* Text area to search. */
	entry_search_string = gtk_entry_new ( );

	/* Store reference to the search string */
	gtk_widget_ref ( entry_search_string);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_ENTRY, entry_search_string, (GtkDestroyNotify) gtk_widget_unref);
	gtk_box_pack_start ( GTK_BOX ( hb_control), entry_search_string, TRUE, TRUE, 0);

	/* Start search button. */
	bt_search = gtk_button_new_with_label ( _( "Search"));

	/* Store reference to the start search button. */
	gtk_widget_ref ( bt_search);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_START_SEARCH_BUTTON, bt_search,(GtkDestroyNotify) gtk_widget_unref);
	gtk_tooltips_set_tip ( tooltips_group, bt_search, _("The search is case sensitive and supports character \"*\".\nExample : img*.bmp or *.htm"), "TipsSelectDisk");
	gtk_signal_connect ( GTK_OBJECT ( bt_search), "clicked", GTK_SIGNAL_FUNC ( notebook_search_search_button_click), window);
	gtk_container_set_border_width ( GTK_CONTAINER ( bt_search), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_control), bt_search, FALSE, FALSE, 0);

	/* Case/uncase sensitive check buton. */
	chk_bt_case_sensitive = gtk_check_button_new_with_label ( _( "Case sensitive"));

	/* Store reference to the case or uncase sensitive check button. */
	gtk_widget_ref ( chk_bt_case_sensitive);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_CHECK_BUTTON, chk_bt_case_sensitive, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_bt_case_sensitive), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_control), chk_bt_case_sensitive, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_bt_case_sensitive, _( "The search can be be or not case sensitive.\nIn case sensitive \"File.txt\" doesn't match with \"file.txt\". But in uncase sensitive it matches."), GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_TOOL_TIPS);

	/* Use/not use regular expression check buton. */
	chk_bt_regex = gtk_check_button_new_with_label ( _( "Regular expression"));

	/* Store reference to the us or not regular expression check button. */
	gtk_widget_ref ( chk_bt_regex);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_CHECK_BUTTON, chk_bt_regex, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_bt_regex), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_control), chk_bt_regex, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_bt_regex, _( "Can use regular expression. See documentation for more information."), GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_TOOL_TIPS);

	/* Horizontal box for search options. */
	hb_options = gtk_hbox_new ( FALSE, 10);
	gtk_container_set_border_width ( GTK_CONTAINER ( hb_options), 0);
	gtk_box_pack_start ( GTK_BOX ( vb_tabbed_pane), hb_options, FALSE, TRUE, 0);

	/* Match file check buton. */
	chk_match_file = gtk_check_button_new_with_label ( _( "File"));
	gtk_widget_ref ( chk_match_file);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_CHECK_BUTTON, chk_match_file, (GtkDestroyNotify) gtk_widget_unref);
	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON ( chk_match_file), TRUE);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_match_file), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_options), chk_match_file, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_match_file, _( "The search matches with files name."), GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_TOOL_TIPS);

	/* Match folder check buton. */
	chk_match_folder = gtk_check_button_new_with_label ( _( "Folder"));
	gtk_widget_ref ( chk_match_folder);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_CHECK_BUTTON, chk_match_folder, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_match_folder), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_options), chk_match_folder, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_match_folder, _( "The search matches with folders name."), GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_TOOL_TIPS);

	/* Match disk check buton. */
	chk_match_disk = gtk_check_button_new_with_label ( _( "Disk"));
	gtk_widget_ref ( chk_match_disk);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_CHECK_BUTTON, chk_match_disk, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_match_disk), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_options), chk_match_disk, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_match_disk, _( "The search matches with disks name."), GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_TOOL_TIPS);

	/* Match category check buton. */
	chk_match_category = gtk_check_button_new_with_label ( _( "Category"));
	gtk_widget_ref ( chk_match_category);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_CHECK_BUTTON, chk_match_category, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_match_category), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_options), chk_match_category, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_match_category, _( "The search matches with categories name."), GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_TOOL_TIPS);

	/* Match description check buton. */
	chk_match_description = gtk_check_button_new_with_label ( _( "Description"));
	gtk_widget_ref ( chk_match_description);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_CHECK_BUTTON, chk_match_description, (GtkDestroyNotify) gtk_widget_unref);
	gtk_container_set_border_width ( GTK_CONTAINER ( chk_match_description), 5);
	gtk_box_pack_start ( GTK_BOX ( hb_options), chk_match_description, FALSE, FALSE, 0);
	gtk_tooltips_set_tip ( tooltips_group, chk_match_description, _( "The search matches with descriptions."), GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_TOOL_TIPS);

	/* Horizontal box to display search result. */
	hb_search_result = gtk_hbox_new (FALSE, 0);
	gtk_box_pack_start ( GTK_BOX ( vb_tabbed_pane), hb_search_result, TRUE, TRUE, 0);

	/* Scrolled result list. */
	scrolled_result_list = gtk_scrolled_window_new ( NULL, NULL);
	gtk_box_pack_start ( GTK_BOX ( hb_search_result), scrolled_result_list, TRUE, TRUE, 0);

	/* List of search result. */
	clist_search_result = gtk_clist_new ( 7);
	clist_result = GTK_CLIST ( clist_search_result);

	/* Store reference to the result search list. */
	gtk_widget_ref ( clist_search_result);
	gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_CLIST_SEARCH, clist_search_result,(GtkDestroyNotify) gtk_widget_unref);
	gtk_signal_connect ( GTK_OBJECT ( clist_search_result), "click_column", GTK_SIGNAL_FUNC ( notebook_search_column_sort), NULL);
	gtk_signal_connect ( GTK_OBJECT ( clist_search_result), "select_row", GTK_SIGNAL_FUNC ( notebook_search_select_row), window);
	gtk_container_add ( GTK_CONTAINER ( scrolled_result_list), clist_search_result);
	gtk_clist_set_column_width ( clist_result, 0, 200);
	gtk_clist_set_column_width ( clist_result, 1, 160);
	gtk_clist_set_column_width ( clist_result, 2, 320);
	gtk_clist_set_column_width ( clist_result, 3, 60);
	gtk_clist_set_column_width ( clist_result, 4, 120);
	gtk_clist_set_column_width ( clist_result, 5, 60);
	gtk_clist_set_column_width ( clist_result, 6, 60);
	gtk_clist_column_titles_show ( clist_result);
	gtk_clist_set_column_justification ( clist_result, 1, GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification ( clist_result, 2, GTK_JUSTIFY_LEFT);
	gtk_clist_set_column_justification ( clist_result, 3, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification ( clist_result, 4, GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_justification ( clist_result, 5, GTK_JUSTIFY_LEFT);
	gtk_clist_set_column_justification ( clist_result, 6, GTK_JUSTIFY_LEFT);
/*	gtk_clist_set_sort_column ( clist_result, 0);
	gtk_clist_set_sort_type ( clist_result, GTK_SORT_ASCENDING);
	gtk_clist_set_auto_sort ( clist_result, TRUE);
*/

	lbl_title_col = gtk_label_new ( _( "Name"));
	gtk_clist_set_column_widget ( clist_result, 0, lbl_title_col);

	lbl_title_col = gtk_label_new ( _( "Disk"));
	gtk_clist_set_column_widget ( clist_result, 1, lbl_title_col);

	lbl_title_col = gtk_label_new ( _( "Path"));
	gtk_clist_set_column_widget ( clist_result, 2, lbl_title_col);

	lbl_title_col = gtk_label_new ( _( "Size"));
	gtk_clist_set_column_widget ( clist_result, 3, lbl_title_col);

	lbl_title_col = gtk_label_new ( _( "Date"));
	gtk_clist_set_column_widget ( clist_result, 4, lbl_title_col);

	lbl_title_col = gtk_label_new ( _( "Category"));
	gtk_clist_set_column_widget ( clist_result, 5, lbl_title_col);

	lbl_title_col = gtk_label_new ( _( "Description"));
	gtk_clist_set_column_widget ( clist_result, 6, lbl_title_col);

	return vb_tabbed_pane;
}


GtkWidget * notebook_search_create_and_add ( GtkWindow *window, GtkNotebook *parent)
{
	GtkWidget *tabbed_pane_search = NULL;
	GtkWidget *search_tabbed_pane_label;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( window != NULL )
	{
		tabbed_pane_search = notebook_search_create ( window);

		if ( tabbed_pane_search != NULL )
		{
			gtk_container_add ( GTK_CONTAINER ( parent), tabbed_pane_search);

			/* Text label of the search tabbed pane. */
			search_tabbed_pane_label = gtk_label_new ( _( "Search"));

			/* Store reference to the title of tabbed pane */
			gtk_widget_ref ( search_tabbed_pane_label);
			gtk_object_set_data_full ( GTK_OBJECT ( window), GW_REF_NOTEBOOK_SEARCH_TABBED_PANE_LABEL, search_tabbed_pane_label,(GtkDestroyNotify) gtk_widget_unref);
			gtk_notebook_set_tab_label ( parent, gtk_notebook_get_nth_page ( parent, 1), search_tabbed_pane_label);
		}
	}

	return tabbed_pane_search;
}


gchar * notebook_search_get_search_text ( GtkWindow *w)
{
	GtkEntry *search_entry = NULL;
	gchar *search_text = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		search_entry = gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_ENTRY);

		if ( search_entry != NULL )
		{
			search_text = g_strdup ( gtk_entry_get_text ( search_entry));
		}
	}

	return search_text;
}


GtkCList * notebook_search_get_clist_search ( GtkWindow *w)
{
	GtkCList *clist_search = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		clist_search = GTK_CLIST ( gtk_object_get_data ( GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_CLIST_SEARCH));
	}

	return clist_search;
}


gboolean notebook_search_get_option_case_sensitive ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_USE_CASE_SENSITIVE_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


gboolean notebook_search_get_option_regular_expression ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_USE_REGULAR_EXPRESSION_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


gboolean notebook_search_get_option_match_file ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_MATCH_FILE_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


gboolean notebook_search_get_option_match_folder ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_MATCH_FOLDER_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


gboolean notebook_search_get_option_match_disk ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_MATCH_DISK_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


gboolean notebook_search_get_option_match_category ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_MATCH_CATEGORY_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


gboolean notebook_search_get_option_match_description ( GtkWindow *w)
{
	GtkToggleButton *chk = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		chk = gtk_object_get_data (GTK_OBJECT ( w), GW_REF_NOTEBOOK_SEARCH_MATCH_DESCRIPTION_CHECK_BUTTON);

		if ( chk != NULL )
		{
			result = gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON ( chk));
		}
	}

	return result;
}


struct user_quick_search *notebook_search_get_user_quick_search ( GtkWindow *w)
{
	struct user_quick_search *user_qs = NULL;


#ifdef GW_DEBUG_GUI_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		user_qs = user_quick_search_new ( );

		user_quick_search_set_patern ( user_qs, g_strdup ( notebook_search_get_search_text ( w)));
		user_quick_search_set_case_sensitive ( user_qs, notebook_search_get_option_case_sensitive ( w));
		user_quick_search_set_regex ( user_qs, notebook_search_get_option_regular_expression ( w));
		user_quick_search_set_match_file ( user_qs, notebook_search_get_option_match_file ( w));
		user_quick_search_set_match_folder ( user_qs, notebook_search_get_option_match_folder ( w));
		user_quick_search_set_match_disk ( user_qs, notebook_search_get_option_match_disk ( w));
		user_quick_search_set_match_category ( user_qs, notebook_search_get_option_match_category ( w));
		user_quick_search_set_match_description ( user_qs, notebook_search_get_option_match_description ( w));
	}

	return user_qs;
}
