/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/*!
 * @header	NoteBookSearch.h
 *		GWhere<br>
 *		Copyright (C) 2000  Sbastien LECACHEUR<br><br>
 *		This Distributed under the terms of the <a href="http://www.gnu.org/copyleft/gpl.html">GNU General Public Licence</a><br><br>
 *		This component was developed for the GWhere project.<br><br>
 * @abstract	This is a GUI component for the pane of search engine.
 * @author	Sbastien LECACHEUR <zero@gwhere.org>
 * @version	1.0 2002/02/13
 * @see		support.h
 * @link		GWhere Web Site <http://www.gwhere.org>
 * @link		GTK+ Web Site <http://www.gtk.org>
 * @ref		NoteBookSearch.c
 * @author	Author <zero@gwhere.org>
 */


#ifndef GW_GUI_NOTEBOOK_SEARCH_H
#define GW_GUI_NOTEBOOK_SEARCH_H


#include "../data/UserQuickSearch.h"


/*!
 * @function	notebook_search_create_and_add
 * @abstract	Creates a search engine notebook pane.
 * @discussion	This function creates a search engine notebook pane.
 * @param	w		main window
 * @result	the search engine notebook pane.<br>
 *		Returns <i>NULL</i> when an error occured.<br>
 */
GtkWidget * notebook_search_create ( GtkWindow *w);


/*!
 * @function	notebook_search_create_and_add
 * @abstract	Creates a search engine notebook pane and adds it to the notebook.
 * @discussion	This function creates a search engine notebook pane and adds it to the notebook.
 * @param	w		main window
 * @param	parent		the container
 * @result	the search engine notebook pane.<br>
 *		Returns <i>NULL</i> when an error occured.<br>
 */
GtkWidget * notebook_search_create_and_add ( GtkWindow *w, GtkNotebook * parent);


/*!
 * @function	notebook_search_get_clist_search
 * @abstract	Gets the list of the search engine.
 * @discussion	This function gets the list of search engine.
 * @param	w		refence to the main window
 * @result	the list of search engine.<br>
 *		Returns <i>NULL</i> when an error occured.<br>
 */
GtkCList *notebook_search_get_clist_search ( GtkWindow *w);


/*!
 * @function	notebook_search_get_search_text
 * @abstract	Gets the search string.
 * @discussion	This function gets the search string. This string may be empty.
 * @param	w		refence to the main window
 * @result	the search string of search engine.<br>
 *		Returns <i>NULL</i> when an error occured.<br>
 */
gchar *notebook_search_get_search_text ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_case_sensitive
 * @abstract	Gets if the search is case sensitive.
 * @discussion	This function gets if the search is case sensitive.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search is case sensitive else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_case_sensitive ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_regular_expression
 * @abstract	Gets if the search use regular expression.
 * @discussion	This function gets if the search use regular expression.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search use regular expression else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_regular_expression ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_match_file
 * @abstract	Gets if the search matches with file
 * @discussion	This function gets if the search matches with files name.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search matches with files else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_match_file ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_match_folder
 * @abstract	Gets if the search matches with folder
 * @discussion	This function gets if the search matches with folders name.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search matches with folders else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_match_folder ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_match_disk
 * @abstract	Gets if the search matches with disk
 * @discussion	This function gets if the search matches with disks name.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search matches with disks else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_match_disk ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_match_category
 * @abstract	Gets if the search matches with categories
 * @discussion	This function gets if the search matches with categories name.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search matches with categories else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_match_file ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_match_category
 * @abstract	Gets if the search matches with categories
 * @discussion	This function gets if the search matches with categories.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search matches with categories else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_match_category ( GtkWindow *w);


/*!
 * @function	notebook_search_get_option_match_description
 * @abstract	Gets if the search matches with descriptions
 * @discussion	This function gets if the search matches with descriptions.
 * @param	w		refence to the main window
 * @result	Returns <i>TRUE</i> if the search matches with descriptions else returns <i>FALSE</i>.<br>
 *		Returns <i>FALSE</i> when an error occured.<br>
 */
gboolean notebook_search_get_option_match_description ( GtkWindow *w);


/*!
 * @function	notebook_search_get_user_quick_search
 * @abstract	Gets the user quick search informations
 * @discussion	This function gets the user quick search informations.
 *				<b>Warning :</b> this function allocates a new <i>struct user_quick_search<i>
 *				must free this structure after.
 * @param	w		refence to the main window
 * @result	the user quick search structure.<br>
 *		Returns <i>NULL</i> when an error occured.<br>
 */
struct user_quick_search *notebook_search_get_user_quick_search ( GtkWindow *w);


#endif
