/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "NoteBookSearchCallBack.h"

#include <gtk/gtk.h>

#include "../SearchEngine.h"
#include "../GuiManager.h"

#include "NoteBookSearch.h"
#include "StatusBarCallBack.h"
#include "MsgBox.h"

#include "../data/UserQuickSearch.h"
//#include "../data/EngineQuickSearch.h"


gboolean notebook_search_column_sort ( GtkCList *clist, gint column, gpointer data)
{
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( clist != NULL )
	{
		if (column == clist->sort_column)
		{
			if (clist->sort_type == GTK_SORT_ASCENDING)
			{
				clist->sort_type = GTK_SORT_DESCENDING;
			}
			else
			{
				clist->sort_type = GTK_SORT_ASCENDING;
			}
		}
		else
		{
			gtk_clist_set_sort_column (clist, column);
		}

		gtk_clist_sort (clist);

		result = TRUE;
	}

	return result;
}


gboolean notebook_search_search_button_click ( GtkWidget *bt, GtkWindow *w)
{
	gboolean result = FALSE;
	struct user_quick_search *user_qs = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( interface_get_tree_root ( ) != NULL )
		{
			/* Allocate user_quick_search structure, had to free this structure after */
			user_qs = notebook_search_get_user_quick_search ( w);

			search_engine_search_in_disk ( w, user_qs);

			/* Free the user_quick_search struture. */
			user_quick_search_free ( user_qs);

			result = TRUE;
		}
		else
		{
			msg_box_create ( w, _( "Searching..."), _( "Cannot start searching : no catalog loaded"));

			result = FALSE;
		}
	}

	return result;
}


gboolean notebook_search_select_row ( GtkCList *clist, gint line, gint column, GdkEventButton *event, GtkWindow *w)
{
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( clist != NULL && w != NULL )
	{
		status_bar_set_catalog_search_page ( w, g_list_length ( clist->selection));

		result = TRUE;
	}

	return result;
}
