/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include <stdlib.h>
#include <string.h> /* strlen strcmp */

#include "../tools/GWString.h"
#include "../CatalogStats.h"

#include "../data/Category.h"
#include "../data/DirInfo.h"
#include "../data/DirData.h"
#include "../data/DiskData.h"

#include "PropertiesBox.h"
#include "ComboBoxCategories.h"

#include "../support.h"
#include "PropertiesBoxCallBack.h"


/* I don't like globals variables, but how to take another way? */
extern GtkCTree *ctree_properties;
extern GtkCTreeNode *node_properties;
extern GList *files_properties;


gboolean properties_box_catalog_ok ( GtkWidget *b, GtkWidget *w)
{
	struct catalog_file_info *catalog_info = NULL;
	GtkCTreeNode *selection = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		selection = GTK_CTREE_NODE ( GTK_CLIST ( ctree_properties)->selection->data);

		/* Gets catalog informations */
		catalog_info = gtk_ctree_node_get_row_data ( ctree_properties, node_properties);

		/* Checks if must update catalog name */
		properties_box_catalog_name_check ( w, node_properties, catalog_info);

		/* Checks if must update catalog description */
		properties_box_catalog_description_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		gtk_widget_destroy ( w);

		/* Selectes the selected item in the exploration tree to update files informations */
		gtk_ctree_select ( ctree_properties, selection);

		result = TRUE;
	}

	return result;
}


gboolean properties_box_disk_ok ( GtkWidget *b, GtkWidget *w)
{
	struct disk_info *disk_info = NULL;
	struct catalog_file_info *catalog_info = NULL;
	GtkCTreeNode *selection = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		selection = GTK_CTREE_NODE ( GTK_CLIST ( ctree_properties)->selection->data);

		/* Gets catalog and disk informations */
		catalog_info = gtk_ctree_node_get_row_data ( ctree_properties, GTK_CTREE_ROW ( node_properties)->parent);
		disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree_properties, node_properties));

		/* Checks if must update disk name */
		properties_box_disk_name_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk number archive */
		properties_box_disk_num_archive_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk category */
		properties_box_disk_category_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk description */
		properties_box_disk_description_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update catalog name */
		properties_box_catalog_name_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		/* Checks if must update catalog description */
		properties_box_catalog_description_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		gtk_widget_destroy ( w);

		/* Selectes the selected item in the exploration tree to update files informations */
		gtk_ctree_select ( ctree_properties, selection);

		result = TRUE;
	}

	return result;
}


gboolean properties_box_folder_ok ( GtkWidget *b, GtkWidget *w)
{
	struct file_info *folder_info;
	struct disk_info *disk_info;
	struct catalog_file_info *catalog_info;
	GtkCTreeNode *selection = NULL;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		selection = GTK_CTREE_NODE ( GTK_CLIST ( ctree_properties)->selection->data);

		/* Gets catalog, disk and folder informations */
		folder_info = dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree_properties, node_properties));
		node_properties = folder_get_disk ( ctree_properties, node_properties);
		disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree_properties, node_properties));
		catalog_info = gtk_ctree_node_get_row_data ( ctree_properties, GTK_CTREE_ROW ( node_properties)->parent);

		/* Checks if must update folder category */
		properties_box_folder_category_check ( w, catalog_info, folder_info);

		/* Checks if must update folder description */
		properties_box_folder_description_check ( w, catalog_info, folder_info);

		/* Checks if must update disk name */
		properties_box_disk_name_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk number archive */
		properties_box_disk_num_archive_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk category */
		properties_box_disk_category_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk description */
		properties_box_disk_description_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update catalog name */
		properties_box_catalog_name_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		/* Checks if must update catalog description */
		properties_box_catalog_description_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		gtk_widget_destroy ( w);

		/* Selectes the selected item in the exploration tree to update files informations */
		gtk_ctree_select ( ctree_properties, selection);

		result = TRUE;
	}

	return result;
}


gboolean properties_box_file_ok ( GtkWidget *b, GtkWidget *w)
{
	struct file_info *folder_info = NULL, *file_info = NULL;
	struct disk_info *disk_info = NULL;
	struct catalog_file_info *catalog_info = NULL;
	GtkCTreeNode *selection = NULL;
	GtkCTreeNode *folder_node = node_properties;
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		selection = GTK_CTREE_NODE ( GTK_CLIST ( ctree_properties)->selection->data);

		/* Gets catalog, disk and file informations */
		file_info = (struct file_info*)files_properties->data;
		node_properties = folder_get_disk ( ctree_properties, node_properties);
		disk_info = disk_data_get_info ( gtk_ctree_node_get_row_data ( ctree_properties, node_properties));
		catalog_info = gtk_ctree_node_get_row_data ( ctree_properties, GTK_CTREE_ROW ( node_properties)->parent);

		/* Checks if must update file category */
		properties_box_file_category_check ( w, catalog_info, file_info);

		/* Checks if must update file description */
		properties_box_file_description_check ( w, catalog_info, file_info);

		/* Checks if the file is in the root folder */
		if ( GTK_CTREE_ROW ( GTK_CTREE_ROW ( folder_node)->parent)->parent != NULL )
		{
			/* If it's not in the root folder */

			/* Gets folder informations */
			folder_info = dir_data_get_info ( gtk_ctree_node_get_row_data ( ctree_properties, folder_node));

			/* Checks if must update folder category */
			properties_box_folder_category_check ( w, catalog_info, folder_info);

			/* Checks if must update folder description */
			properties_box_folder_description_check ( w, catalog_info, folder_info);
		}

		/* Checks if must update disk name */
		properties_box_disk_name_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk number archive */
		properties_box_disk_num_archive_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk category */
		properties_box_disk_category_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update disk description */
		properties_box_disk_description_check ( w, node_properties, catalog_info, disk_info);

		/* Checks if must update catalog name */
		properties_box_catalog_name_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		/* Checks if must update catalog description */
		properties_box_catalog_description_check ( w, GTK_CTREE_ROW ( node_properties)->parent, catalog_info);

		gtk_widget_destroy ( w);

		/* Selectes the selected item in the exploration tree to update files informations */
		gtk_ctree_select ( ctree_properties, selection);

		result = TRUE;
	}

	return result;
}


gint properties_box_catalog_name_check ( GtkWidget *w, GtkCTreeNode *node, struct catalog_file_info *catalog_info)
{
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	gchar *name;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( properties_box_get_catalog_name ( w), catalog_file_info_get_name ( catalog_info)) != 0 )
		{
			/* If displayed catalog name is different as catalog name */

			if ( strcmp ( properties_box_get_catalog_name ( w), "") != 0 )
			{
				/* If a name is displayed */
				catalog_file_info_set_name ( catalog_info, g_strdup ( properties_box_get_catalog_name ( w)));

				gtk_ctree_node_get_pixtext ( ctree_properties, node, 0, &name, &spc, &pxmp, &btmp);
				gtk_ctree_node_set_pixtext ( ctree_properties, node, 0, catalog_file_info_get_name ( catalog_info), spc, pxmp, btmp);
			}
			else
			{
				catalog_file_info_set_name ( catalog_info, NULL);

				gtk_ctree_node_get_pixtext ( ctree_properties, node, 0, &name, &spc, &pxmp, &btmp);
				gtk_ctree_node_set_pixtext ( ctree_properties, node, 0, catalog_file_info_get_title ( catalog_info), spc, pxmp, btmp);
			}

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_catalog_description_check ( GtkWidget *w, GtkCTreeNode *node, struct catalog_file_info *catalog_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( properties_box_get_catalog_description ( w), catalog_file_info_get_description ( catalog_info)) != 0 )
		{
			/* If displayed description is different as catalog description */
			if ( strcmp ( properties_box_get_catalog_description ( w), "") != 0 )
			{
				catalog_file_info_set_description ( catalog_info, g_strdup ( properties_box_get_catalog_description ( w)));
			}
			else
			{
				catalog_file_info_set_description ( catalog_info, NULL);
			}

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_disk_name_check ( GtkWidget *w, GtkCTreeNode *node, struct catalog_file_info *catalog_info,struct disk_info *disk_info)
{
	guint8 spc;
	GdkBitmap *btmp;
	GdkPixmap *pxmp;
	gchar *tmp_name, *disk_name, *name;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( (tmp_name = properties_box_get_disk_name ( w)), disk_info_get_name ( disk_info)) != 0 )
		{
			/* If displayed disk name is different as disk name */

			disk_name = disk_get_name ( tmp_name);
			if ( strcmp ( disk_name, "") != 0 )
			{
				gtk_ctree_node_get_pixtext ( ctree_properties, node_properties, 0, &name, &spc, &pxmp, &btmp);
				gtk_ctree_node_set_pixtext ( ctree_properties, node_properties, 0, disk_name, spc, pxmp, btmp);
				disk_info_set_name ( disk_info, g_strdup ( disk_name));

				/* Indicate that's catalog is modified */
				catalog_file_info_set_ismodified ( catalog_info, TRUE);
				g_free ( disk_name);
			}
		}

		result = 0;
	}

	return result;
}


gint properties_box_disk_num_archive_check ( GtkWidget *w, GtkCTreeNode *node, struct catalog_file_info *catalog_info, struct disk_info *disk_info)
{
	gint num;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( ( num = atoi ( properties_box_get_disk_num ( w))) != disk_info_get_num ( disk_info) )
		{
			/* If displayed number archive is different as disk number archive */
			if ( num > 0 )
			{
				if ( !disk_num_is_exist ( num))
				{
					/* If number archive doesn't exists : it's possible to change it */
					disk_info_set_num ( disk_info, num);

					/* Indicate that's catalog is modified */
					catalog_file_info_set_ismodified ( catalog_info, TRUE);
				}
			}
		}

		result = 0;
	}

	return result;
}


gint properties_box_disk_category_check ( GtkWidget *w, GtkCTreeNode *node, struct catalog_file_info *catalog_info, struct disk_info *disk_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( properties_box_get_disk_category ( w) != disk_info_get_category ( disk_info) )
		{
			disk_info_set_category ( disk_info, properties_box_get_disk_category ( w));

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_disk_description_check ( GtkWidget *w, GtkCTreeNode *node, struct catalog_file_info *catalog_info, struct disk_info *disk_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( properties_box_get_disk_description ( w), disk_info_get_description ( disk_info)) != 0 )
		{
			/* If displayed description is different as disk description */
			if ( strcmp ( properties_box_get_disk_description ( w), "") != 0 )
			{
				disk_info_set_description ( disk_info, g_strdup ( properties_box_get_disk_description ( w)));
			}
			else
			{
				disk_info_set_description ( disk_info, NULL);
			}

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_folder_category_check ( GtkWidget *w, struct catalog_file_info *catalog_info, struct file_info *folder_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( properties_box_get_folder_category ( w) != file_info_get_category ( folder_info) )
		{
			file_info_set_category ( folder_info, properties_box_get_folder_category ( w));

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_folder_description_check ( GtkWidget *w, struct catalog_file_info *catalog_info, struct file_info *folder_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( properties_box_get_folder_description ( w), file_info_get_description ( folder_info)) != 0 )
		{
			/* If displayed description is different as folder description */
			if ( strcmp ( properties_box_get_folder_description ( w), "") != 0 )
			{
				file_info_set_description ( folder_info, g_strdup ( properties_box_get_folder_description ( w)));
			}
			else
			{
				file_info_set_description ( folder_info, NULL);
			}

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_file_category_check ( GtkWidget *w, struct catalog_file_info *catalog_info, struct file_info *file_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( properties_box_get_file_category ( w) != file_info_get_category ( file_info) )
		{
			file_info_set_category ( file_info, properties_box_get_file_category ( w));

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}


gint properties_box_file_description_check ( GtkWidget *w, struct catalog_file_info *catalog_info, struct file_info *file_info)
{
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( properties_box_get_file_description ( w), file_info_get_description ( file_info)) != 0 )
		{
			/* If displayed description is different as file description */
			if ( strcmp ( properties_box_get_file_description ( w), "") != 0 )
			{
				file_info_set_description ( file_info, g_strdup ( properties_box_get_file_description ( w)));
			}
			else
			{
				file_info_set_description ( file_info, NULL);
			}

			/* Indicate that's catalog is modified */
			catalog_file_info_set_ismodified ( catalog_info, TRUE);
		}

		result = 0;
	}

	return result;
}
