/*  GWhere
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "SettingsCatalogFile.h"

#include <gtk/gtk.h>
#include <stdlib.h> /* strtol */

#include "../ApplicationManager.h"
#include "SettingsWindowBox.h"


#define GW_PLUGIN_SETTINGS_WINDOW "gw_plugin_settings_window"
#define GW_PLUGIN_SETTINGS_CATALOG_COMPRESSION_LEVEL_ENTRY "gw_plugin_settings_catalog_compression_level_entry"


gint gw_plugin_settings_catalog_file_pane_create ( GtkWindow *settings, GtkContainer *parent, GtkWidget **pane);
gint gw_plugin_settings_catalog_file_pane_load ( GtkWidget *pane);
gint gw_plugin_settings_catalog_file_pane_on_change ( GtkEntry *entry, GtkWidget *pane);
gint gw_plugin_settings_catalog_file_pane_apply ( GtkWidget *pane);


gint gw_plugin_settings_catalog_file_init ( GWSettingsModule **module)
{
	gint result = -1;

	if ( (*module = gw_settings_module_new ( )) != NULL )
	{
		(*module)->name = g_strdup ( _( "Catalog file"));
		(*module)->create = &gw_plugin_settings_catalog_file_pane_create;
		(*module)->load = &gw_plugin_settings_catalog_file_pane_load;
		(*module)->on_change = &gw_plugin_settings_catalog_file_pane_on_change;
		(*module)->apply = &gw_plugin_settings_catalog_file_pane_apply;

		result = 0;
	}

	return result;
}


gint gw_plugin_settings_catalog_file_pane_create ( GtkWindow *settings, GtkContainer *parent, GtkWidget **pane)
{
	gint result = -1;
	GtkWidget *table_pane;
	GtkWidget *lbl_catalog_level_compression;
	GtkAdjustment *adj_catalog_level_compression;
	GtkWidget *spn_catalog_level_compression;
	gfloat level = GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_DEFAULT;
	GtkTooltips *tooltips;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	tooltips = gtk_tooltips_new ( );

	if ( settings != NULL && parent != NULL )
	{
		table_pane = gtk_table_new ( 1, 2, FALSE);
		gtk_widget_ref ( GTK_WIDGET ( settings));
		gtk_object_set_data_full ( GTK_OBJECT ( table_pane), GW_PLUGIN_SETTINGS_WINDOW, settings, (GtkDestroyNotify) gtk_widget_unref);
		gtk_container_set_border_width ( GTK_CONTAINER ( table_pane), 5);
		gtk_table_set_row_spacings ( GTK_TABLE (table_pane), 5);
		gtk_table_set_col_spacings ( GTK_TABLE (table_pane), 5);

		lbl_catalog_level_compression = gtk_label_new ( _( "Catalog compression level :"));
		gtk_table_attach ( GTK_TABLE ( table_pane), lbl_catalog_level_compression, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
		gtk_misc_set_alignment ( GTK_MISC ( lbl_catalog_level_compression), 0, 0.5);

		adj_catalog_level_compression = (GtkAdjustment *) gtk_adjustment_new ( level, GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_MIN, GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_MAX, 1.0, 1.0, 0.0);
		spn_catalog_level_compression = gtk_spin_button_new ( adj_catalog_level_compression, 0, 0);
		gtk_widget_ref ( GTK_WIDGET ( spn_catalog_level_compression));
		gtk_object_set_data_full ( GTK_OBJECT ( table_pane), GW_PLUGIN_SETTINGS_CATALOG_COMPRESSION_LEVEL_ENTRY, spn_catalog_level_compression, (GtkDestroyNotify) gtk_widget_unref);
		gtk_spin_button_set_numeric ( GTK_SPIN_BUTTON ( spn_catalog_level_compression), TRUE);
		gtk_spin_button_set_wrap ( GTK_SPIN_BUTTON ( spn_catalog_level_compression), TRUE);
		gtk_spin_button_set_shadow_type ( GTK_SPIN_BUTTON ( spn_catalog_level_compression), GTK_SHADOW_OUT);
		gtk_tooltips_set_tip ( tooltips, spn_catalog_level_compression, _( "Select the catalog compression level. The default value is 3, it's a good value between hight performance and small size."), NULL);

		gtk_table_attach ( GTK_TABLE ( table_pane), spn_catalog_level_compression, 1, 2, 0, 1, (GtkAttachOptions) ( GTK_FILL), (GtkAttachOptions) (0), 0, 0);

		*pane = table_pane;

		gw_plugin_settings_catalog_file_pane_load ( table_pane);

		/* Connects signals after loading. */
		gtk_signal_connect  ( GTK_OBJECT ( spn_catalog_level_compression), "changed", (GtkSignalFunc)gw_plugin_settings_catalog_file_pane_on_change, table_pane);

		result = 0;
	}

	return result;
}


gint gw_plugin_settings_catalog_file_pane_load ( GtkWidget *pane)
{
	gint result = -1;
	GtkSpinButton *button = NULL;
	GtkAdjustment *adj_catalog_level_compression = NULL;
	gfloat level = GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL_DEFAULT;
	gchar *value = NULL;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( pane != NULL )
	{
#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : loading...\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

		if ( (button = GTK_SPIN_BUTTON ( gtk_object_get_data ( GTK_OBJECT ( pane), GW_PLUGIN_SETTINGS_CATALOG_COMPRESSION_LEVEL_ENTRY))) != NULL)
		{
#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
			g_print ( "*** GW - %s (%d) :: %s() : getting the compression level...\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

			value = gw_application_manager_get_settings ( GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL);

#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
			g_print ( "*** GW - %s (%d) :: %s() : compression level is %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, value);
#endif

			if ( value != NULL )
			{
				level = strtod ( value, NULL);
			}

#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
			g_print ( "*** GW - %s (%d) :: %s() : compression level is %f\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, level);
#endif
		}
		else
		{
#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
			g_print ( "*** GW - %s (%d) :: %s() : warning no spin button!!\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif
		}

		adj_catalog_level_compression = gtk_spin_button_get_adjustment ( button);
		gtk_adjustment_set_value ( adj_catalog_level_compression, level);

		button = NULL;
		value = NULL;

		result = 0;
	}

	return result;
}


gint gw_plugin_settings_catalog_file_pane_on_change ( GtkEntry *entry, GtkWidget *pane)
{
	gint result = -1;
	GtkWindow *settings = NULL;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( pane != NULL )
	{
		if ( (settings = GTK_WINDOW ( gtk_object_get_data ( GTK_OBJECT ( pane), GW_PLUGIN_SETTINGS_WINDOW))) != NULL )
		{
			gw_settings_window_box_set_modified ( settings, TRUE);

			result = 0;
		}
	}

	return result;
}


gint gw_plugin_settings_catalog_file_pane_apply ( GtkWidget *pane)
{
	gint result = -1;
	GtkEntry *entry = NULL;
	gchar *value = NULL;
	GtkWindow *settings = NULL;


#ifdef GW_DEBUG_PLUGIN_SETTINGS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( pane != NULL )
	{
		if ( (entry = GTK_ENTRY ( gtk_object_get_data ( GTK_OBJECT ( pane), GW_PLUGIN_SETTINGS_CATALOG_COMPRESSION_LEVEL_ENTRY))) != NULL)
		{
			value = gtk_entry_get_text ( GTK_ENTRY ( entry));
			gw_application_manager_set_settings ( GW_VALUE_APP_CATALOG_COMPRESSION_LEVEL, g_strdup( value));
		}

		entry = NULL;
		value = NULL;

		if ( (settings = GTK_WINDOW ( gtk_object_get_data ( GTK_OBJECT ( pane), GW_PLUGIN_SETTINGS_WINDOW))) != NULL )
		{
			gw_settings_window_box_set_modified ( settings, FALSE);
		}

		result = 0;
	}

	return result;
}
