/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include <string.h> /* strcmp */

#include "../data/CatalogFileInfo.h"
#include "../tools/GWString.h"
#include "StatusBar.h"

#include "../support.h"
#include "StatusBarCallBack.h"


gint status_bar_put_msg_catalog ( GtkWindow *w, GtkCTree *ctree, GtkCTreeNode *root, gint num, long double size)
{
//	GtkStatusbar *status = NULL;
//	guint numserie, nummsg;
	gchar *catalog_name;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( strcmp ( catalog_file_info_get_name ( gtk_ctree_node_get_row_data ( ctree, root)), "" ) == 0 )
		{
			catalog_name = catalog_file_info_get_title ( gtk_ctree_node_get_row_data ( ctree, root));
		}
		else
		{
			catalog_name = catalog_file_info_get_name ( gtk_ctree_node_get_row_data ( ctree, root));
		}

		msg1 = g_strdup_printf ( _( " %s [ %s ], total disks : %d"), catalog_name, catalog_file_info_get_full_name ( gtk_ctree_node_get_row_data ( ctree, root)), num);
		msg2 = g_strdup_printf ( _( " Listed Items : %d (total size : %s)"), num, octet2char ( size));

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}


gint status_bar_put_msg_disk ( GtkWindow *w, gchar *name, gchar *type, long double size, long double free, gint num)
{
//	GtkStatusbar *status;
//	guint numserie, nummsg;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		msg1 = g_strdup_printf ( _( " %s [ %s ], total : %s, free : %s"), name, type, octet2char ( size), octet2char ( free));

		if ( num == 0 )
		{
			msg2 = g_strdup_printf ( _( " Selected Item : %d (total size : %s)"), 1, octet2char ( size));
		}
		else
		{
			msg2 = g_strdup_printf ( _( " Listed Items : %d (total size : %s)"), num, octet2char ( size));
		}

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}


gint status_bar_put_msg_dir ( GtkWindow *w, gchar *name, long double size, gint nb_files, gint nb_subfolders, gint num)
{
//	GtkStatusbar *status;
//	guint numserie, nummsg;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		msg1 = g_strdup_printf ( _( " Folder \"%s\", %s (files %d, subfolders %d)"), name, octet2char ( size), nb_files, nb_subfolders);

		if ( num == 0 )
		{
			msg2 = g_strdup_printf ( _( " Selected Item : %d (total size : %s)"), 1, octet2char ( size));
		}
		else
		{
			msg2 = g_strdup_printf ( _( " Listed Items : %d (total size : %s)"), num, octet2char ( size));
		}

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}


gint status_bar_put_msg_file ( GtkWindow *w, gchar *name, long double size, gchar *date, gint num)
{
//	GtkStatusbar *status;
//	guint numserie, nummsg;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		msg1 = g_strdup_printf ( _( " %s, %s, %s"), name, octet2char ( size), date);
		msg2 = g_strdup_printf ( _( " Selected Item : %d (total size : %s)"), num, octet2char ( size));

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}


gint status_bar_set_catalog_explorer_page ( GtkWindow *w, gint n)
{
//	GtkStatusbar *status;
//	guint numserie, nummsg;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		msg1 = g_strdup_printf ( _( " Catalog explorer"));
		msg2 = g_strdup_printf ( _( " Selected Item : %d"), n);

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}


gint status_bar_set_catalog_search_page ( GtkWindow *w, gint n)
{
//	GtkStatusbar *status;
//	guint numserie, nummsg;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		msg1 = g_strdup_printf ( _( " Search engine"));
		msg2 = g_strdup_printf ( _( " Selected Item : %d"), n);

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}


gint status_bar_set_catalog_managment_page ( GtkWindow *w)
{
//	GtkStatusbar *status;
//	guint numserie, nummsg;
	gchar *msg1, *msg2;
	gint result = -1;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		msg1 = g_strdup_printf ( _( " Catalog managment"));
		msg2 = g_strdup_printf ( _( " "));

		result = status_bar_put_messages ( w, msg1, msg2);
	}

	return result;
}
