/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../support.h"
#include "File.h"

#include <stdio.h> /* perror */
#include <sys/types.h> /* mkdir */
#include <fcntl.h> /* open mkdir */
#include <string.h> /* strcmp strcpy malloc rindex */
#include <sys/stat.h> /* stat */
#include <unistd.h> /* close read write stat mkdir */
#include <stdlib.h> /* malloc on Cygwin */
#include <errno.h> /* errno */


int gw_file_copy ( char * source, char * dest)
{
	int src,dst,n;
	char buffer[512];


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( ( src = open ( source, O_RDONLY) ) == -1 )
	{
		perror ( "source");

		return -1;
	}

	if ( ( dst = open ( dest, O_WRONLY|O_TRUNC|O_CREAT, 0666) ) == -1 )
	{
		perror ( "destination");
		close ( src);

		return -1;
	}

	while ( ( n = read ( src, buffer, 512) ) > 0 )
	{
		if ( n != write ( dst, buffer, n) )
		{
			return -1;
		}
	}

	if ( n < 0 )
	{
		return -1;
	}

	close ( src);
	close ( dst);

	return 0;
}


char * gw_file_read_until_c ( int fic, char c, int len)
{
	char temp[2], tmp[len];
	char *buf;
	int i = 0, n;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	temp[1] = '\0';

	while ( ( n = read ( fic, temp, 1) ) > 0 )
	{
		if ( temp[0] == c )
		{
			if ( ( buf = (char*)malloc ( 2)) == NULL )
			{
				perror ( "malloc");

				return NULL;
			}

			strcpy ( buf, "");

			return buf;
		}

		tmp[i++] = temp[0];

		while ( temp[0] != c )
		{
			if ( ( n = read ( fic, temp, 1) ) == 0)
			{
				return NULL;
			}

			tmp[i++] = temp[0];
		}

		tmp[--i] = '\0';

		if ( ( buf = (char*)malloc ( i + 1) ) == NULL )
		{
			perror ( "malloc");

			return NULL;
		}

		strcpy ( buf, tmp);

		return buf;
	}

	return NULL;
}


char* gw_file_readline ( int fic, char* buf, int len)
{
#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	buf = gw_file_read_until_c ( fic, '\n', len);

	return buf;
}


char* gw_zfile_readline ( gzFile fic, char* buf, int len)
{
	int slen;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( gzgets ( fic, buf, len) == Z_NULL )
	{
		return NULL;
	}

	slen = strlen ( buf);

	if ( slen && buf[slen - 1] != '\n')
	{
		return NULL;
	}

	buf[slen - 1] = '\0';

	return buf;
}


long gw_file_get_size ( FILE * file)
{
	long pos;
	long len = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( file != NULL )
	{
		pos = ftell ( file);
		fseek ( file, 0L, SEEK_END);
		len = ftell ( file);
		fseek ( file, pos, SEEK_SET);
	}

	return len;
}


int gw_file_exists ( const char * pathname)
{
	int result = -1;
	struct stat fd;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : testing if %s exists\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, dir_name);
#endif

	if ( pathname != NULL )
	{
		if ( stat ( pathname, &fd) == 0 )
		{
			result = 0;
		}
	}

	return result;
}


char * gw_file_get_name ( const char * pathname)
{
	char * result = NULL;
	char * last = NULL;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : testing if %s exists\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, dir_name);
#endif

	if ( pathname != NULL )
	{
		last = rindex ( pathname, '/') + 1;

		if ( last != NULL )
		{
			result = g_strdup ( last);
		}
	}

	return result;
}


int gw_file_mkdir ( const char * pathname)
{
	int result = -1;
	struct stat fd;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : creating %s if not exists\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, pathname);
#endif

	if ( pathname != NULL )
	{
		if ( stat ( pathname, &fd) == 0 )
		{
			/* The folder already exists. */
			result = 0;
		}
		else
		{
			if ( mkdir ( pathname, S_IRUSR|S_IWUSR|S_IRGRP) == 0 )
			{
				result = 0;
			}
			else
			{
				switch ( errno )
				{
					case EEXIST :	result = 0;

							break;

					default :	break;
				}
			}
		}
	}

	return result;
}


int gw_file_mkdirs ( const char * pathname)
{
	int result = -1;
	struct stat fd;
	char * last = NULL;
	char * parent_pathname = NULL;
	int length = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : creating %s if not exists\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, pathname);
#endif

	if ( pathname != NULL )
	{
		if ( stat ( pathname, &fd) == 0 )
		{
			/* The folder already exists. */
			result = 0;
		}
		else
		{
			if ( mkdir ( pathname, S_IRUSR|S_IWUSR|S_IRGRP) == 0 )
			{
				result = 0;
			}
			else
			{
				switch ( errno )
				{
					case EEXIST :	result = 0;

							break;

					case ENOENT:	last = rindex ( pathname, '/');

							/* Bug to fix : pathname is a const char *, cannot be modified. */
/*							if ( last[1] == '\0' )
							{
								last[0] = '\0';

								last = NULL;
								last = rindex ( pathname, '/');
							}
*/
							if ( last != NULL )
							{
								length = last - pathname;
								g_print ( "length=%d\n", pathname - last);

								parent_pathname = (char *)malloc ( sizeof ( char) * length + 1);
								parent_pathname = strncpy ( parent_pathname, pathname, length);
								parent_pathname[length] = '\0';

								if ( gw_file_mkdirs ( parent_pathname) == 0 )
								{
									if ( gw_file_mkdirs ( pathname) == 0 )
									{
										result = 0;
									}
								}

								free ( parent_pathname);
							}

					default :	break;
				}
			}
		}
	}

	return result;
}
